#include "stdafx.h"
#include "APIWrapper.h"
#include "Galba.h"
#include "BurnImageDlg.h"

#include "DiscRecorder.h"


// CBurnImageDlg dialog

IMPLEMENT_DYNAMIC(CBurnImageDlg, CBurnTaskDialog)

CBurnImageDlg::CBurnImageDlg(IGalbaWindow* pGalbaWindow, CWnd* pParent/* = NULL*/) :
    CBurnTaskDialog(pGalbaWindow, CBurnImageDlg::IDD, pParent),
    alwaysDisableSpeedComboBox(FALSE)
{
}

CBurnImageDlg::~CBurnImageDlg()
{
}

void CBurnImageDlg::DoDataExchange(CDataExchange* pDX)
{
	CBurnTaskDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_PATH_EDIT, controls.pathEdit);
	DDX_Control(pDX, ID_OPEN_IMAGE, controls.openImageButton);
	DDX_Control(pDX, IDC_IMAGE_SIZE, controls.imageSize);
	DDX_Control(pDX, IDC_CLOSE_MEDIA_CHK, controls.closeMediaCheck);
	DDX_Control(pDX, IDC_EJECT_WHEN_FINISHED_CHK, controls.ejectCheck);
	DDX_Control(pDX, IDC_CAPACITY, controls.capacityProg);
	DDX_Control(pDX, IDC_DISC_NOT_BLANK, controls.discNotBlank);
	DDX_Control(pDX, IDC_MAX_TEXT, controls.maxText);
	DDX_Control(pDX, IDC_SPEED_COMBO, controls.speedCombo);
	DDX_Control(pDX, ID_BURN, controls.burnButton);
}


BEGIN_MESSAGE_MAP(CBurnImageDlg, CBurnTaskDialog)
	ON_WM_DESTROY()
	ON_WM_CTLCOLOR()
	ON_COMMAND(ID_OPEN_IMAGE, &CBurnImageDlg::OnOpenImage)
	ON_COMMAND(ID_BURN, &CBurnImageDlg::OnBurn)

	ON_MESSAGE(WM_IMAPI_BEGIN_BURN, &CBurnImageDlg::OnImapiBeginBurn)
	ON_MESSAGE(WM_IMAPI_END_BURN, &CBurnImageDlg::OnImapiEndBurn)
	ON_MESSAGE(WM_IMAPI_FORMAT_DATA_UPDATE, &CBurnImageDlg::OnImapiFormatDataUpdate)
	ON_MESSAGE(WM_EJECT_MEDIA, &CBurnImageDlg::OnEjectMedia)
END_MESSAGE_MAP()


BOOL CBurnImageDlg::PreTranslateMessage(MSG* pMsg)
{
	return CBurnTaskDialog::PreTranslateMessage(pMsg);
}


void CBurnImageDlg::CancelInterruptibleTask()
{
	if(runningInterruptibleTask) {
		recordingController.SetCancelFlag(true);
		if(pGalbaWindow)
			pGalbaWindow->EnableCancelButton(FALSE);
	}
}

void CBurnImageDlg::MediaChanged()
{
	if(initialized) {
		AddSpeedSettingsToComboBox();
		SetCapacity();
		CheckForBlankMedia();
		UpdateBlocksInUse();

		BOOL imageRecorderSelected = FALSE;
		if(pGalbaWindow)
			pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
		controls.closeMediaCheck.EnableWindow(!imageRecorderSelected);
		controls.ejectCheck.EnableWindow(!imageRecorderSelected);
	}
}


// CBurnImageDlg message handlers

BOOL CBurnImageDlg::OnInitDialog()
{
	CBurnTaskDialog::OnInitDialog();

	CBitmap bmp;
	bmp.LoadBitmap(IDB_OPEN);
	controls.openImageImageList.Create(16, 16, ILC_COLOR32 | ILC_MASK, 6, 0);
	controls.openImageImageList.Add(&bmp, reinterpret_cast<CBitmap*>(NULL));
	BUTTON_IMAGELIST iml = {0};
	iml.himl = controls.openImageImageList;
	iml.uAlign = BUTTON_IMAGELIST_ALIGN_CENTER;
	controls.openImageButton.SetImageList(&iml);
	bmp.DeleteObject();

	bmp.LoadBitmap(IDB_BURN);
	controls.burnImageList.Create(16, 16, ILC_COLOR32 | ILC_MASK, 6, 0);
	controls.burnImageList.Add(&bmp, reinterpret_cast<CBitmap*>(NULL));
	iml.himl = controls.burnImageList;
	iml.uAlign = BUTTON_IMAGELIST_ALIGN_LEFT;
	iml.margin.left = 5;
	iml.margin.right = 2;
	controls.burnButton.SetImageList(&iml);

	controls.capacityProg.SetRange(0, 100);
	controls.burnButton.EnableWindow(FALSE);

	VARIANT_BOOL b = theApp.settings.GetSetting(CString("BurnImage\\CloseMedia"));
	controls.closeMediaCheck.SetCheck(b == VARIANT_FALSE ? BST_UNCHECKED : BST_CHECKED);
	b = theApp.settings.GetSetting(CString("BurnImage\\EjectMedia"));
	controls.ejectCheck.SetCheck(b == VARIANT_FALSE ? BST_UNCHECKED : BST_CHECKED);

	initialized = true;
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CBurnImageDlg::OnDestroy()
{
	VARIANT_BOOL b = (controls.closeMediaCheck.GetCheck() == BST_CHECKED ? VARIANT_TRUE : VARIANT_FALSE);
	theApp.settings.SetSetting(CString("BurnImage\\CloseMedia"), b);
	b = (controls.ejectCheck.GetCheck() == BST_CHECKED ? VARIANT_TRUE : VARIANT_FALSE);
	theApp.settings.SetSetting(CString("BurnImage\\EjectMedia"), b);
}

HBRUSH CBurnImageDlg::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
	HBRUSH hBrush = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);
	if(nCtlColor == CTLCOLOR_STATIC && pWnd->GetDlgCtrlID() == IDC_DISC_NOT_BLANK) {
		pDC->SetTextColor(RGB(255, 0, 0));
		pDC->SetBkMode(TRANSPARENT);
	}
	return hBrush;
}

void CBurnImageDlg::OnOpenImage()
{
	CString filters;
	filters.LoadString(IDS_OPEN_IMAGE_FILTERS);
	CFileDialog fileDlg(TRUE, NULL, NULL, OFN_DONTADDTORECENT | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY, filters, this);
	if(IsVistaOrNewer())
		fileDlg.UpdateOFNFromShellDialog();
	CString fileName;
	if(fileDlg.DoModal() == IDOK) {
		fileName = fileDlg.GetPathName();
		controls.pathEdit.SetWindowText(fileName);
		UpdateBlocksInUse();
	}
	controls.burnButton.EnableWindow(PathFileExists(fileName));
	fileName.ReleaseBuffer();
}

void CBurnImageDlg::OnBurn()
{
	if(!pGalbaWindow)
		return;

	CString recorderID;
	BOOL imageRecorderSelected = FALSE;
	pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	if(imageRecorderSelected)
		recorderID = IMAGE_RECORDER_ID;
	else {
		CDiscRecorder* pRecorder = NULL;
		pGalbaWindow->GetRecorder(&pRecorder);
		if(!pRecorder)
			return;
		recorderID = pRecorder->GetUniqueId();
	}

	recordingController.SetRecorderID(recorderID);
	recordingController.SetClientName(CLIENT_NAME);
	recordingController.SetNotificationWindow(*this);
	recordingController.SetWriteSpeed(GetWriteSpeed());
	recordingController.EnableBurnProof(true);
	recordingController.SetCloseMedia(controls.closeMediaCheck.GetCheck() == BST_CHECKED);
	recordingController.SetEjectMedia(controls.ejectCheck.GetCheck() == BST_CHECKED);

	CComPtr<IStream> pData;
	CString fileName;
	controls.pathEdit.GetWindowText(fileName);
	ASSERT(!fileName.IsEmpty());

	if(imageRecorderSelected) {
		ULONGLONG imageSize = 0;
		CFile imageFile(fileName, CFile::modeRead | CFile::shareDenyWrite);
		imageSize = imageFile.GetLength();

		LONG l = static_cast<LONG>(imageSize / IMAPI_SECTOR_SIZE);
		recordingController.SetTotalBlocks(l);
		recordingController.SetBlockSize(IMAPI_SECTOR_SIZE);
	}

	HRESULT lastError = SHCreateStreamOnFileEx(fileName, STGM_READ | STGM_SHARE_DENY_WRITE, FILE_ATTRIBUTE_NORMAL, FALSE, NULL, &pData);
	if(FAILED(lastError)) {
		CString lastErrorMessage;
		lastErrorMessage.Format(IDS_ERR_COULD_NOT_READ_FILE, fileName, lastError);
		AfxMessageBox(lastErrorMessage, MB_ICONERROR | MB_OK);
		return;
	}

	if(!recordingController.Burn(pData))
		AfxMessageBox(recordingController.GetErrorMessage(), MB_ICONERROR | MB_OK);
}


LRESULT CBurnImageDlg::OnImapiBeginBurn(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	runningInterruptibleTask = true;

	EnableUI(FALSE);
	if(pGalbaWindow) {
		pGalbaWindow->EnableUI(FALSE);
		pGalbaWindow->EnableCancelButton(TRUE);
	}
	return 0;
}

LRESULT CBurnImageDlg::OnImapiEndBurn(WPARAM wParam, LPARAM /*lParam*/)
{
	runningInterruptibleTask = false;
	if(pGalbaWindow) {
		pGalbaWindow->SetupProgressMarquee(FALSE);
		pGalbaWindow->SetSystemBufferUsage(0);
		pGalbaWindow->SetTimes(-1, -1);
	}

	CString message;
	switch(wParam) {
		case S_OK:
			// success;
			message.LoadString(IDS_RECORDING_COMPLETE);
			AfxMessageBox(message, MB_ICONINFORMATION | MB_OK);
			break;
		case E_IMAPI_RECORDER_MEDIA_NO_MEDIA:
		case E_IMAPI_DF2DATA_INVALID_MEDIA_STATE:
			// no writable media inserted
			PostMessage(WM_EJECT_MEDIA, 0, 0);
			message.LoadString(IDS_RECORDING_FAILED_NO_WRITABLE_MEDIA);
			AfxMessageBox(message, MB_ICONERROR | MB_OK);
			break;
		default:
			// error
			AfxMessageBox(recordingController.GetErrorMessage(), MB_ICONERROR | MB_OK);
			break;
	}

	EnableUI(TRUE);
	if(pGalbaWindow) {
		CString s;
		pGalbaWindow->SetProgress(0, s);
		pGalbaWindow->EnableCancelButton(FALSE);
		pGalbaWindow->DeviceChange();
		pGalbaWindow->EnableUI(TRUE);
	}
	return 0;
}

LRESULT CBurnImageDlg::OnImapiFormatDataUpdate(WPARAM /*wParam*/, LPARAM lParam)
{
	if(pGalbaWindow) {
		PIMAPI_FORMAT_DATA_UPDATE_EVENT_ARGS pDetails = reinterpret_cast<PIMAPI_FORMAT_DATA_UPDATE_EVENT_ARGS>(lParam);
		CString progressText;
		switch(pDetails->currentAction) {
			case IMAPI_FORMAT2_DATA_WRITE_ACTION_VALIDATING_MEDIA:
				progressText.LoadString(IDS_STATUS_VALIDATING_MEDIA);
				break;
			case IMAPI_FORMAT2_DATA_WRITE_ACTION_FORMATTING_MEDIA:
				progressText.LoadString(IDS_STATUS_FORMATTING_MEDIA);
				break;
			case IMAPI_FORMAT2_DATA_WRITE_ACTION_INITIALIZING_HARDWARE:
				progressText.LoadString(IDS_STATUS_INITIALIZING_RECORDER);
				break;
			case IMAPI_FORMAT2_DATA_WRITE_ACTION_CALIBRATING_POWER:
				progressText.LoadString(IDS_STATUS_CALIBRATING_POWER);
				break;
			case IMAPI_FORMAT2_DATA_WRITE_ACTION_WRITING_DATA:
			{
				progressText.LoadString(IDS_STATUS_WRITING_DATA1);
				int percentage = 0;
				if(pDetails->totalSystemBuffer > 0)
					percentage = pDetails->usedSystemBuffer * 100 / pDetails->totalSystemBuffer;
				pGalbaWindow->SetSystemBufferUsage(percentage);
				pGalbaWindow->SetTimes(pDetails->elapsedTime, pDetails->remainingTime);
				break;
			}
			case IMAPI_FORMAT2_DATA_WRITE_ACTION_FINALIZATION:
				progressText.LoadString(IDS_STATUS_FINALIZING_MEDIA);
				pGalbaWindow->SetupProgressMarquee(TRUE);
				pGalbaWindow->SetTimes(-1, -1);
				pGalbaWindow->EnableCancelButton(FALSE);
				break;
			case IMAPI_FORMAT2_DATA_WRITE_ACTION_COMPLETED:
				progressText.LoadString(IDS_STATUS_RECORDING_COMPLETED);
				break;
			case IMAPI_FORMAT2_DATA_WRITE_ACTION_VERIFYING:
				progressText.LoadString(IDS_STATUS_VERIFYING_DATA);
				break;
		}
		int percentage = 0;
		if(pDetails->sectorCount > 0)
			percentage = pDetails->lastWrittenLba * 100 / pDetails->sectorCount;
		pGalbaWindow->SetProgress(percentage, progressText);
	}
	return (recordingController.GetCancelFlag() ? RETURN_CANCEL_WRITE : RETURN_CONTINUE_WRITE);
}

LRESULT CBurnImageDlg::OnEjectMedia(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	if(!pGalbaWindow)
		return 0;

	CDiscRecorder* pRecorder = NULL;
	pGalbaWindow->GetRecorder(&pRecorder);
	if(pRecorder)
		pRecorder->EjectMedia();
	return 0;
}


void CBurnImageDlg::AddSpeedSettingsToComboBox()
{
	alwaysDisableSpeedComboBox = FALSE;
	controls.speedCombo.ResetContent();
	CString speedString;
	speedString.LoadString(IDS_MAXIMUM_SPEED);
	int speedIndex = controls.speedCombo.AddString(speedString);
	controls.speedCombo.SetItemData(speedIndex, (sizeof(INT_PTR) == 4 ? 0xFFFFFFFF : 0xFFFFFFFFFFFFFFFF));

	if(!pGalbaWindow)
		return;

	BOOL imageRecorderSelected = FALSE;
	pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	if(imageRecorderSelected) {
		controls.speedCombo.SetCurSel(0);
		controls.speedCombo.EnableWindow(FALSE);
		alwaysDisableSpeedComboBox = TRUE;
		return;
	}

	CDiscRecorder* pRecorder = NULL;
	pGalbaWindow->GetRecorder(&pRecorder);
	if(!pRecorder)
		return;

	CDiscFormatData discFormatData;
	if(!discFormatData.Initialize(pRecorder, CLIENT_NAME, *this)) {
		AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
		return;
	}
	if(!discFormatData.IsCurrentMediaSupported(pRecorder)) {
		controls.speedCombo.SetCurSel(0);
		controls.speedCombo.EnableWindow(FALSE);
		alwaysDisableSpeedComboBox = TRUE;
		return;
	}
	controls.speedCombo.EnableWindow(TRUE);

	ULONG singleSpeed = GetMediaTypeSingleSpeed(discFormatData.GetCurrentPhysicalMediaType());
	ULONG c = discFormatData.GetTotalSupportedWriteSpeeds();
	for(ULONG i = 0; i < c; i++) {
		speedString = TEXT("");

		ULONG originalSpeed = discFormatData.GetSupportedWriteSpeed(i);
		ULONG speed = originalSpeed;
		if(singleSpeed == IMAPI_SECTORS_PER_SECOND_AT_1X_CD) {
			if(speed % 100 != 0) {
				// round-up to 100 sectors
				speed = ((speed / 100) + 1) * 100;
			}
			ULONG speedUp = speed / singleSpeed;
			speedString.Format(TEXT("%ix"), speedUp);
		} else {
			float speedUp = static_cast<float>(speed) / static_cast<float>(singleSpeed);
			speedUp *= 0.875f;
			speedString.Format(TEXT("%.1fx"), speedUp);
		}

		speedIndex = controls.speedCombo.AddString(speedString);
		controls.speedCombo.SetItemData(speedIndex, originalSpeed);
	}
	controls.speedCombo.SetCurSel(0);
}

void CBurnImageDlg::CheckForBlankMedia()
{
	if(!pGalbaWindow)
		return;

	BOOL blank = TRUE;

	BOOL imageRecorderSelected = FALSE;
	pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	if(!imageRecorderSelected) {
		CDiscRecorder* pRecorder = NULL;
		pGalbaWindow->GetRecorder(&pRecorder);
		if(!pRecorder)
			return;
		CDiscFormatData discFormatData;
		if(!discFormatData.Initialize(pRecorder, CLIENT_NAME, *this)) {
			AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
			return;
		}
		blank = discFormatData.IsMediaBlank();
	}

	//EnableUI(blank);
	controls.discNotBlank.ShowWindow(blank ? SW_HIDE : SW_SHOW);
}

IMAPI_MEDIA_PHYSICAL_TYPE CBurnImageDlg::GetCurrentMediaType(CDiscRecorder* pRecorder)
{
	CDiscFormatData discFormatData;
	if(!discFormatData.Initialize(pRecorder, CLIENT_NAME, *this)) {
		AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
		return IMAPI_MEDIA_TYPE_UNKNOWN;
	}

	IMAPI_MEDIA_PHYSICAL_TYPE mediaType = discFormatData.GetCurrentPhysicalMediaType();
	HRESULT lastError = discFormatData.GetHresult();
	// Some devices, e.g. from Samsung fail with E_IMAPI_RECORDER_INVALID_RESPONSE_FROM_DEVICE if there is no media.
	if(mediaType == IMAPI_MEDIA_TYPE_UNKNOWN && lastError != E_IMAPI_RECORDER_MEDIA_NO_MEDIA && lastError != E_IMAPI_RECORDER_INVALID_RESPONSE_FROM_DEVICE)
		AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
	return mediaType;
}

void CBurnImageDlg::EnableUI(BOOL enable)
{
	controls.openImageButton.EnableWindow(enable);
	controls.speedCombo.EnableWindow(!alwaysDisableSpeedComboBox && enable);
	controls.burnButton.EnableWindow(enable);

	BOOL imageRecorderSelected = FALSE;
	if(pGalbaWindow)
		pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	controls.closeMediaCheck.EnableWindow(!imageRecorderSelected && enable);
	controls.ejectCheck.EnableWindow(!imageRecorderSelected && enable);
}

ULONGLONG CBurnImageDlg::GetFreeBytesOnMedia(CDiscRecorder* pRecorder)
{
	CDiscFormatData discFormatData;
	if(!discFormatData.Initialize(pRecorder, CLIENT_NAME, *this)) {
		AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
		return 0;
	}

	LONG capacity = discFormatData.GetFreeSectorsOnMedia();
	if(capacity == -1) {
		AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
		return 0;
	}
	if(capacity == 0) {
		switch(GetCurrentMediaType(pRecorder)) {
			case IMAPI_MEDIA_TYPE_UNKNOWN:
				// no media inserted, assume a DVD
				capacity = 4700000000 / IMAPI_SECTOR_SIZE;
				break;
			case IMAPI_MEDIA_TYPE_CDROM:
				// no writable media inserted, assume a CD
				capacity = 734003200 / IMAPI_SECTOR_SIZE;
				break;
			case IMAPI_MEDIA_TYPE_DVDROM:
			case IMAPI_MEDIA_TYPE_HDDVDROM:
			case IMAPI_MEDIA_TYPE_BDROM:
			default:
				// no writable media inserted, assume a DVD
				capacity = 4700000000 / IMAPI_SECTOR_SIZE;
				break;
		}
	}
	ULONGLONG ll = capacity;
	ll *= IMAPI_SECTOR_SIZE;
	return ll;
}

ULONG	CBurnImageDlg::GetMediaTypeSingleSpeed(IMAPI_MEDIA_PHYSICAL_TYPE mediaType)
{
	switch(mediaType) {
		case IMAPI_MEDIA_TYPE_UNKNOWN:
		case IMAPI_MEDIA_TYPE_DISK:
		default:
			// assume CD
		case IMAPI_MEDIA_TYPE_CDROM:
		case IMAPI_MEDIA_TYPE_CDR:
		case IMAPI_MEDIA_TYPE_CDRW:
			return IMAPI_SECTORS_PER_SECOND_AT_1X_CD;
		case IMAPI_MEDIA_TYPE_DVDROM:
		case IMAPI_MEDIA_TYPE_DVDRAM:
		case IMAPI_MEDIA_TYPE_DVDPLUSR:
		case IMAPI_MEDIA_TYPE_DVDPLUSRW:
		case IMAPI_MEDIA_TYPE_DVDPLUSR_DUALLAYER:
		case IMAPI_MEDIA_TYPE_DVDDASHR:
		case IMAPI_MEDIA_TYPE_DVDDASHRW:
		case IMAPI_MEDIA_TYPE_DVDDASHR_DUALLAYER:
		case IMAPI_MEDIA_TYPE_DVDPLUSRW_DUALLAYER:
			return IMAPI_SECTORS_PER_SECOND_AT_1X_DVD;
		case IMAPI_MEDIA_TYPE_HDDVDROM:
		case IMAPI_MEDIA_TYPE_HDDVDR:
		case IMAPI_MEDIA_TYPE_HDDVDRAM:
			return IMAPI_SECTORS_PER_SECOND_AT_1X_HD_DVD;
		case IMAPI_MEDIA_TYPE_BDROM:
		case IMAPI_MEDIA_TYPE_BDR:
		case IMAPI_MEDIA_TYPE_BDRE:
			return IMAPI_SECTORS_PER_SECOND_AT_1X_BD;
	}
}

LONG CBurnImageDlg::GetWriteSpeed()
{
	int selection = controls.speedCombo.GetCurSel();
	if(selection >= 0)
		return static_cast<LONG>(controls.speedCombo.GetItemData(selection));
	return -1;
}

void CBurnImageDlg::SetCapacity()
{
	if(!pGalbaWindow)
		return;

	ULONGLONG capacity = 0;

	BOOL imageRecorderSelected = FALSE;
	pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	if(!imageRecorderSelected) {
		CDiscRecorder* pRecorder = NULL;
		pGalbaWindow->GetRecorder(&pRecorder);
		if(!pRecorder)
			return;
		capacity = GetFreeBytesOnMedia(pRecorder);
	}

	TCHAR pBuffer[100];
	StrFormatByteSize(capacity, pBuffer, 100);
	controls.maxText.SetWindowText(pBuffer);
}

void CBurnImageDlg::UpdateBlocksInUse()
{
	if(!pGalbaWindow)
		return;

	ULONGLONG capacity = 0;

	BOOL imageRecorderSelected = FALSE;
	pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	if(!imageRecorderSelected) {
		CDiscRecorder* pRecorder = NULL;
		pGalbaWindow->GetRecorder(&pRecorder);
		if(!pRecorder)
			return;
		capacity = GetFreeBytesOnMedia(pRecorder);
	}

	ULONGLONG imageSize = 0;
	CString fileName;
	controls.pathEdit.GetWindowText(fileName);
	if(!fileName.IsEmpty()) {
		CFile imageFile(fileName, CFile::modeRead | CFile::shareDenyWrite);
		imageSize = imageFile.GetLength();
	}
	TCHAR pBuffer[100];
	StrFormatByteSize(imageSize, pBuffer, 100);
	CString s;
	s.Format(IDS_IMAGE_SIZE, pBuffer);
	controls.imageSize.SetWindowText(s);

	int percentage = 0;
	if(capacity > 0)
		percentage = static_cast<int>(imageSize * static_cast<ULONGLONG>(100) / capacity);
	controls.capacityProg.SetPos(percentage);
	if(IsVistaOrNewer()) {
		if(percentage > 100)
			controls.capacityProg.SetState(PBST_ERROR);
		else
			controls.capacityProg.SetState(PBST_NORMAL);
	}
}