#include "stdafx.h"
#include "APIWrapper.h"
#include "Galba.h"
#include "CreateDataDiscDlg.h"
#include "NewDirectoryDlg.h"

#include "DiscRecorder.h"

#define ITEMTYPE_FILE		1
#define ITEMTYPE_FOLDER	2


// CCreateDataDiscDlg dialog

IMPLEMENT_DYNAMIC(CCreateDataDiscDlg, CBurnTaskDialog)

CCreateDataDiscDlg::CCreateDataDiscDlg(IGalbaWindow* pGalbaWindow, CWnd* pParent/* = NULL*/) :
    CBurnTaskDialog(pGalbaWindow, CCreateDataDiscDlg::IDD, pParent),
    pFsImage(NULL),
    pProgressItems(NULL),
    hAddedDataEvent(NULL),
    alwaysDisableSpeedComboBox(FALSE)
{
	hAddedDataEvent = CreateEvent(NULL, TRUE, TRUE, NULL);
}

CCreateDataDiscDlg::~CCreateDataDiscDlg()
{
	if(pProgressItems)
		pProgressItems->Release();
	if(pFsImage)
		delete pFsImage;
	if(hAddedDataEvent) {
		CloseHandle(hAddedDataEvent);
	}
}

void CCreateDataDiscDlg::DoDataExchange(CDataExchange* pDX)
{
	CBurnTaskDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_FILESYSTEM_COMBO, controls.fileSystemCombo);
	DDX_Control(pDX, IDC_ISOLEVEL_COMBO, controls.isoLevelCombo);
	DDX_Control(pDX, IDC_UDFVERSION_COMBO, controls.udfVersionCombo);
	DDX_Control(pDX, IDC_CLOSE_MEDIA_CHK, controls.closeMediaCheck);
	DDX_Control(pDX, IDC_EJECT_WHEN_FINISHED_CHK, controls.ejectCheck);
	DDX_Control(pDX, ID_RESET_COMPILATION, controls.resetCompilationButton);
	DDX_Control(pDX, IDC_FILES_TREE, controls.dataTree);
	DDX_Control(pDX, IDC_FILES_LIST, controls.dataList);
	DDX_Control(pDX, IDC_CAPACITY, controls.capacityProg);
	DDX_Control(pDX, IDC_CURRENT_SIZE, controls.currentSize);
	DDX_Control(pDX, IDC_DISC_NOT_BLANK, controls.discNotBlank);
	DDX_Control(pDX, IDC_MAX_TEXT, controls.maxText);
	DDX_Control(pDX, IDC_ADD_DATA_BUTTON, controls.addDataButton);
	DDX_Control(pDX, IDC_SPEED_COMBO, controls.speedCombo);
	DDX_Control(pDX, ID_BURN, controls.burnButton);
}


BEGIN_MESSAGE_MAP(CCreateDataDiscDlg, CBurnTaskDialog)
	ON_WM_DESTROY()
	ON_WM_CONTEXTMENU()
	ON_WM_CTLCOLOR()
	ON_CBN_SELCHANGE(IDC_FILESYSTEM_COMBO, &CCreateDataDiscDlg::OnCbnSelchangeFilesystemCombo)
	ON_CBN_SELCHANGE(IDC_ISOLEVEL_COMBO, &CCreateDataDiscDlg::OnCbnSelchangeIsolevelCombo)
	ON_CBN_SELCHANGE(IDC_UDFVERSION_COMBO, &CCreateDataDiscDlg::OnCbnSelchangeUdfversionCombo)
	ON_NOTIFY(TVN_BEGINLABELEDIT, IDC_FILES_TREE, &CCreateDataDiscDlg::OnTvnBeginlabeleditFilesTree)
	ON_NOTIFY(TVN_ENDLABELEDIT, IDC_FILES_TREE, &CCreateDataDiscDlg::OnTvnEndlabeleditFilesTree)
	ON_NOTIFY(TVN_KEYDOWN, IDC_FILES_TREE, &CCreateDataDiscDlg::OnTvnKeydownFilesTree)
	ON_NOTIFY(TVN_SELCHANGED, IDC_FILES_TREE, &CCreateDataDiscDlg::OnTvnSelchangedFilesTree)
	ON_NOTIFY(LVN_ITEMACTIVATE, IDC_FILES_LIST, &CCreateDataDiscDlg::OnLvnItemActivateFilesList)
	ON_NOTIFY(LVN_KEYDOWN, IDC_FILES_LIST, &CCreateDataDiscDlg::OnLvnKeydownFilesList)
	ON_BN_CLICKED(IDC_ADD_DATA_BUTTON, &CCreateDataDiscDlg::OnBnClickedAddDataButton)
	ON_COMMAND(ID_RESET_COMPILATION, &CCreateDataDiscDlg::OnResetCompilation)
	ON_COMMAND(ID_ADDDATA_DIRECTORY, &CCreateDataDiscDlg::OnAddDirectory)
	ON_COMMAND(ID_ADDDATA_DIRECTORYCONTENT, &CCreateDataDiscDlg::OnAddDirectoryContent)
	ON_COMMAND(ID_ADDDATA_FILE, &CCreateDataDiscDlg::OnAddFile)
	ON_COMMAND(ID_BURN, &CCreateDataDiscDlg::OnBurn)

	ON_MESSAGE(WM_IMAPI_BEGIN_ADD_DATA, &CCreateDataDiscDlg::OnImapiBeginAddData)
	ON_MESSAGE(WM_IMAPI_END_ADD_DATA, &CCreateDataDiscDlg::OnImapiEndAddData)
	ON_MESSAGE(WM_IMAPI_IMAGE_UPDATE, &CCreateDataDiscDlg::OnImapiImageUpdate)
	ON_MESSAGE(WM_IMAPI_BEGIN_BURN, &CCreateDataDiscDlg::OnImapiBeginBurn)
	ON_MESSAGE(WM_IMAPI_END_BURN, &CCreateDataDiscDlg::OnImapiEndBurn)
	ON_MESSAGE(WM_IMAPI_FORMAT_DATA_UPDATE, &CCreateDataDiscDlg::OnImapiFormatDataUpdate)
	ON_MESSAGE(WM_EJECT_MEDIA, &CCreateDataDiscDlg::OnEjectMedia)
	ON_MESSAGE(WM_UPDATE_BLOCKS_IN_USE, &CCreateDataDiscDlg::OnUpdateBlocksInUse)
	ON_MESSAGE(WM_UPDATE_COMPILATION_VIEW, &CCreateDataDiscDlg::OnUpdateCompilationView)
END_MESSAGE_MAP()


BOOL CCreateDataDiscDlg::PreTranslateMessage(MSG* pMsg)
{
	if(pMsg->message == WM_KEYDOWN) {
		CEdit* pEdit = controls.dataTree.GetEditControl();
		if(pEdit && GetFocus() == pEdit)
			return FALSE;

		if(GetFocus()->GetDlgCtrlID() == IDC_FILES_TREE) {
			if(pMsg->wParam == VK_F2) {
				HTREEITEM hItem = controls.dataTree.GetSelectedItem();
				if(hItem)
					controls.dataTree.SendMessage(TVM_EDITLABEL, 0, reinterpret_cast<LPARAM>(hItem));
			}
		}
	}
	return CBurnTaskDialog::PreTranslateMessage(pMsg);
}


void CCreateDataDiscDlg::CancelInterruptibleTask()
{
	ASSERT(pFsImage != NULL);

	if(runningInterruptibleTask) {
		pFsImage->SetCancelFlag(true);
		recordingController.SetCancelFlag(true);
		if(pGalbaWindow)
			pGalbaWindow->EnableCancelButton(FALSE);
	}
}

void CCreateDataDiscDlg::MediaChanged()
{
	if(initialized) {
		AddFileSystemsToComboBox();
		AddSpeedSettingsToComboBox();
		SetRootIcon();
		SetCapacity();
		CheckForBlankMedia();
		ImportOldSession();
		UpdateBlocksInUse();

		BOOL imageRecorderSelected = FALSE;
		if(pGalbaWindow)
			pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
		controls.closeMediaCheck.EnableWindow(!imageRecorderSelected);
		controls.ejectCheck.EnableWindow(!imageRecorderSelected);
	}
}

bool CCreateDataDiscDlg::BeginAddData(const CString& baseDir)
{
	if(!pFsImage)
		return false;

	filesToAdd.RemoveAll();
	currentBaseDir = baseDir;
	return true;
}

void CCreateDataDiscDlg::AddData(const CString& dataType, CString& path)
{
	if(!pFsImage)
		return;
	if(dataType == TEXT("File")) {
		filesToAdd.Add(path);
		return;
	}

	// don't try to add data while other data is being added
	if(WaitForSingleObject(hAddedDataEvent, INFINITE) == WAIT_OBJECT_0) {
		// prevent other data from being added
		// NOTE: If we ever allow AddData being called from different threads, we may run into race conditions here.
		ResetEvent(hAddedDataEvent);
		bool setEvent = false;

		CComPtr<IFsiDirectoryItem> pRootItem;
		pFsImage->GetRoot(&pRootItem);
		if(pRootItem) {
			CComPtr<IFsiDirectoryItem> pBaseDirectory;
			CComBSTR s = currentBaseDir;
			pRootItem->get_Item(s, reinterpret_cast<IFsiItem**>(&pBaseDirectory));
			if(pBaseDirectory) {
				if(dataType == TEXT("Directory") || dataType == TEXT("DirectoryContent")) {
					if(PathIsRoot(path)) {
						CComPtr<IFsiDirectoryItem> pDriveDirectory;
						HRESULT hr = pFsImage->CreateDirectoryItem(path.Left(1), &pDriveDirectory);
						if(FAILED(hr)) {
							AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
							setEvent = true;
						} else {
							CTime t = CTime::GetCurrentTime();
							SYSTEMTIME st;
							t.GetAsSystemTime(st);
							COleDateTime creationTime(st);
							pDriveDirectory->put_CreationTime(creationTime.m_dt);
							pDriveDirectory->put_LastModifiedTime(creationTime.m_dt);
							pDriveDirectory->put_LastAccessedTime(creationTime.m_dt);

							hr = pBaseDirectory->Add(pDriveDirectory);
							if(FAILED(hr)) {
								CString s;
								s.Format(IDS_ERR_ADD, path, hr);
								AfxMessageBox(s, MB_ICONERROR | MB_OK);
								setEvent = true;
							} else
								setEvent = !AddDirectoryToImage(pDriveDirectory, path, VARIANT_FALSE);
						}
					} else
						setEvent = !AddDirectoryToImage(pBaseDirectory, path, (dataType == TEXT("DirectoryContent") ? VARIANT_FALSE : VARIANT_TRUE));

				} else if(dataType == TEXT("NewDirectory")) {
					setEvent = true;
					CComPtr<IFsiDirectoryItem> pNewDirectory;
					if(FAILED(pFsImage->CreateDirectoryItem(path, &pNewDirectory))) {
						AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
					} else {
						CTime t = CTime::GetCurrentTime();
						SYSTEMTIME st;
						t.GetAsSystemTime(st);
						COleDateTime creationTime(st);
						pNewDirectory->put_CreationTime(creationTime.m_dt);
						pNewDirectory->put_LastModifiedTime(creationTime.m_dt);
						pNewDirectory->put_LastAccessedTime(creationTime.m_dt);

						HRESULT hr = pBaseDirectory->Add(pNewDirectory);
						if(SUCCEEDED(hr))
							PostMessage(WM_UPDATE_COMPILATION_VIEW, 0, 0);
						else {
							CString s;
							s.Format(IDS_ERR_ADD, path, hr);
							AfxMessageBox(s, MB_ICONERROR | MB_OK);
						}
					}
				} else
					setEvent = true;
			} else
				setEvent = true;
		} else
			setEvent = true;
		if(setEvent) {
			// allow other data to be added
			SetEvent(hAddedDataEvent);
		}
	}
}

void CCreateDataDiscDlg::CommitAddedData(void)
{
	if(!pFsImage)
		return;

	if(filesToAdd.GetCount() > 0) {
		// don't try to add data while other data is being added
		if(WaitForSingleObject(hAddedDataEvent, INFINITE) == WAIT_OBJECT_0) {
			// prevent other data from being added
			// NOTE: If we ever allow AddData being called from different threads, we may run into race conditions here.
			ResetEvent(hAddedDataEvent);
			bool setEvent = false;

			CComPtr<IFsiDirectoryItem> pRootItem;
			pFsImage->GetRoot(&pRootItem);
			if(pRootItem) {
				CComPtr<IFsiDirectoryItem> pBaseDirectory;
				CComBSTR s = currentBaseDir;
				pRootItem->get_Item(s, reinterpret_cast<IFsiItem**>(&pBaseDirectory));
				if(pBaseDirectory) {
					if(!pFsImage->AddFiles(pBaseDirectory, filesToAdd, FALSE)) {
						AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
						setEvent = true;
					}
				} else
					setEvent = true;
			} else
				setEvent = true;
			if(setEvent) {
				// allow other data to be added
				SetEvent(hAddedDataEvent);
			}
		}
		filesToAdd.RemoveAll();
	}
	currentBaseDir = TEXT("");
}

DROPEFFECT CCreateDataDiscDlg::OnFileDragOver(COleDataObject* /*pDataObject*/, DWORD /*keyState*/, CPoint point)
{
	POINT pt = point;
	ClientToScreen(&pt);
	CWnd* pWnd = WindowFromPoint(pt);
	if(pWnd) {
		if(pWnd->GetDlgCtrlID() == IDC_FILES_TREE) {
			controls.dataTree.ScreenToClient(&pt);
			UINT flags = 0;
			HTREEITEM hDropTarget = controls.dataTree.HitTest(pt, &flags);
			if(!hDropTarget && (flags & TVHT_NOWHERE) == TVHT_NOWHERE)
				hDropTarget = controls.dataTree.GetRootItem();
			controls.dataTree.SelectDropTarget(hDropTarget);
			return DROPEFFECT_COPY;
		} else if(pWnd->GetDlgCtrlID() == IDC_FILES_LIST) {
			controls.dataTree.SelectDropTarget(NULL);
			return DROPEFFECT_COPY;
		}
	}
	controls.dataTree.SelectDropTarget(NULL);
	return DROPEFFECT_NONE;
}

void CCreateDataDiscDlg::OnDragLeave(void)
{
	controls.dataTree.SelectDropTarget(NULL);
}

BOOL CCreateDataDiscDlg::OnFileDrop(COleDataObject* pDataObject, DWORD /*keyState*/, CPoint point)
{
	POINT pt = point;
	ClientToScreen(&pt);
	CWnd* pWnd = WindowFromPoint(pt);
	if(pWnd) {
		if(pWnd->GetDlgCtrlID() == IDC_FILES_TREE) {
			controls.dataTree.ScreenToClient(&pt);
			HTREEITEM hDropTarget = controls.dataTree.GetDropHilightItem();

			IFsiDirectoryItem* pBaseDirectory = GetDirectoryForTreeItem(hDropTarget);
			if(pBaseDirectory) {
				AddDroppedFilesToDirectory(pBaseDirectory, reinterpret_cast<HDROP>(pDataObject->GetGlobalData(CF_HDROP)));
				pBaseDirectory->Release();
			}

			controls.dataTree.SelectDropTarget(NULL);
			return TRUE;
		} else if(pWnd->GetDlgCtrlID() == IDC_FILES_LIST) {
			controls.dataTree.SelectDropTarget(NULL);

			IFsiDirectoryItem* pBaseDirectory = GetDirectoryForTreeItem(controls.dataTree.GetSelectedItem());
			if(pBaseDirectory) {
				AddDroppedFilesToDirectory(pBaseDirectory, reinterpret_cast<HDROP>(pDataObject->GetGlobalData(CF_HDROP)));
				pBaseDirectory->Release();
			}
			return TRUE;
		}
	}
	controls.dataTree.SelectDropTarget(NULL);
	return FALSE;
}


// CCreateDataDiscDlg message handlers

BOOL CCreateDataDiscDlg::OnInitDialog()
{
	CBurnTaskDialog::OnInitDialog();

	CBitmap bmp;
	bmp.LoadBitmap(IDB_BURN);
	controls.burnImageList.Create(16, 16, ILC_COLOR32 | ILC_MASK, 6, 0);
	controls.burnImageList.Add(&bmp, reinterpret_cast<CBitmap*>(NULL));
	BUTTON_IMAGELIST iml = {0};
	iml.himl = controls.burnImageList;
	iml.uAlign = BUTTON_IMAGELIST_ALIGN_LEFT;
	iml.margin.left = 5;
	iml.margin.right = 2;
	controls.burnButton.SetImageList(&iml);

	SetWindowTheme(controls.dataTree, L"explorer", NULL);
	SetWindowTheme(controls.dataList, L"explorer", NULL);
	controls.dataList.SetExtendedStyle(LVS_EX_DOUBLEBUFFER | LVS_EX_FULLROWSELECT | LVS_EX_LABELTIP);
	if(IsVistaOrNewer()) {
		controls.addDataButton.ModifyStyle(0, BS_SPLITBUTTON);
		controls.addDataButton.SetSplitStyle(controls.addDataButton.GetSplitStyle() | BCSS_NOSPLIT);
	}

	CString columnHeader;
	columnHeader.LoadString(IDS_COLUMN_HEADER_NAME);
	controls.dataList.InsertColumn(0, columnHeader, LVCFMT_LEFT, 220);
	columnHeader.LoadString(IDS_COLUMN_HEADER_SIZE);
	controls.dataList.InsertColumn(1, columnHeader, LVCFMT_RIGHT, 100);
	columnHeader.LoadString(IDS_COLUMN_HEADER_TYPE);
	controls.dataList.InsertColumn(2, columnHeader, LVCFMT_LEFT, 120);
	columnHeader.LoadString(IDS_COLUMN_HEADER_LAST_CHANGE);
	controls.dataList.InsertColumn(3, columnHeader, LVCFMT_LEFT, 120);
	columnHeader.LoadString(IDS_COLUMN_HEADER_PATH);
	controls.dataList.InsertColumn(4, columnHeader, LVCFMT_LEFT, 300);

	HIMAGELIST hImgLst = NULL;
	SHGetImageList(SHIL_SMALL, IID_IImageList, reinterpret_cast<LPVOID*>(&hImgLst));
	CImageList imageList;
	imageList.Attach(hImgLst);
	controls.dataTree.SetImageList(&imageList, TVSIL_NORMAL);
	controls.dataList.SetImageList(&imageList, LVSIL_SMALL);
	imageList.Detach();

	SHFILEINFO shfi = {0};
	SHGetFileInfo(TEXT("folder"), FILE_ATTRIBUTE_DIRECTORY, &shfi, sizeof(SHFILEINFO), SHGFI_SYSICONINDEX | SHGFI_USEFILEATTRIBUTES);
	STOCKICON_FOLDER = shfi.iIcon;
	SHGetFileInfo(TEXT("folder"), FILE_ATTRIBUTE_DIRECTORY, &shfi, sizeof(SHFILEINFO), SHGFI_SYSICONINDEX | SHGFI_USEFILEATTRIBUTES | SHGFI_OPENICON);
	STOCKICON_FOLDER_SELECTED = shfi.iIcon;
	SHGetFileInfo(TEXT("folder"), FILE_ATTRIBUTE_DIRECTORY, &shfi, sizeof(SHFILEINFO), SHGFI_TYPENAME | SHGFI_USEFILEATTRIBUTES);
	FILETYPE_FOLDER = shfi.szTypeName;

	pFsImage = new CFileSystemImage();
	ASSERT(pFsImage != NULL);
	if(!pFsImage->Initialize(*this)) {
		AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
		EndDialog(IDCANCEL);
		return TRUE;
	}

	HTREEITEM h = controls.dataTree.InsertItem(TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE, pFsImage->GetVolumeName(), STOCKICON_FOLDER, STOCKICON_FOLDER_SELECTED, 0, 0, 0, TVI_ROOT, TVI_FIRST);
	controls.dataTree.SelectItem(h);

	controls.capacityProg.SetRange(0, 100);

	VARIANT_BOOL b = theApp.settings.GetSetting(CString("CreateDataDisc\\CloseMedia"));
	controls.closeMediaCheck.SetCheck(b == VARIANT_FALSE ? BST_UNCHECKED : BST_CHECKED);
	b = theApp.settings.GetSetting(CString("CreateDataDisc\\EjectMedia"));
	controls.ejectCheck.SetCheck(b == VARIANT_FALSE ? BST_UNCHECKED : BST_CHECKED);

	ImportOldSession();
	initialized = true;
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CCreateDataDiscDlg::OnDestroy()
{
	VARIANT_BOOL b = (controls.closeMediaCheck.GetCheck() == BST_CHECKED ? VARIANT_TRUE : VARIANT_FALSE);
	theApp.settings.SetSetting(CString("CreateDataDisc\\CloseMedia"), b);
	b = (controls.ejectCheck.GetCheck() == BST_CHECKED ? VARIANT_TRUE : VARIANT_FALSE);
	theApp.settings.SetSetting(CString("CreateDataDisc\\EjectMedia"), b);
}

void CCreateDataDiscDlg::OnContextMenu(CWnd* pWnd, CPoint point)
{
	ASSERT(pFsImage != NULL);

	if(pWnd->GetDlgCtrlID() == IDC_FILES_TREE) {
		CPoint pt = point;
		controls.dataTree.ScreenToClient(&pt);
		HTREEITEM hItem = controls.dataTree.HitTest(pt);
		if(!hItem)
			return;

		CMenu menu;
		menu.LoadMenu(IDR_DATACONTEXT);
		CMenu* pContextMenu = menu.GetSubMenu(0);
		if(controls.dataTree.GetParentItem(hItem))
			pContextMenu->RemoveMenu(ID_EDITVOLUMENAME, MF_BYCOMMAND);
		else
			pContextMenu->RemoveMenu(ID_REMOVEDATA, MF_BYCOMMAND);
		pContextMenu->RemoveMenu(ID_REMOVEALLDATA, MF_BYCOMMAND);

		controls.dataTree.SelectItem(hItem);
		UINT menuItemID = pContextMenu->TrackPopupMenuEx(TPM_LEFTALIGN | TPM_TOPALIGN | TPM_RETURNCMD | TPM_VERTICAL | TPM_VERPOSANIMATION, point.x, point.y, this, NULL);

		switch(menuItemID) {
			case ID_EDITVOLUMENAME:
				controls.dataTree.EditLabel(controls.dataTree.GetSelectedItem());
				break;
			case ID_REMOVEDATA:
				RemoveTree(controls.dataTree.GetSelectedItem());
				break;
			case ID_NEWDIRECTORY:
				AddNewDirectory(controls.dataTree.GetSelectedItem());
				break;
		}
	} else if(pWnd->GetDlgCtrlID() == IDC_FILES_LIST) {
		CPoint pt = point;
		controls.dataList.ScreenToClient(&pt);

		CMenu menu;
		menu.LoadMenu(IDR_DATACONTEXT);
		CMenu* pContextMenu = menu.GetSubMenu(0);
		pContextMenu->RemoveMenu(ID_EDITVOLUMENAME, MF_BYCOMMAND);
		if(controls.dataList.GetSelectedCount() == 0)
			pContextMenu->RemoveMenu(ID_REMOVEDATA, MF_BYCOMMAND);
		if(controls.dataList.GetItemCount() == 0) {
			pContextMenu->RemoveMenu(ID_REMOVEALLDATA, MF_BYCOMMAND);
			pContextMenu->RemoveMenu(0, MF_BYPOSITION);
		}

		UINT menuItemID = pContextMenu->TrackPopupMenuEx(TPM_LEFTALIGN | TPM_TOPALIGN | TPM_RETURNCMD | TPM_VERTICAL | TPM_VERPOSANIMATION, point.x, point.y, this, NULL);

		switch(menuItemID) {
			case ID_REMOVEALLDATA:
				if(controls.dataTree.GetParentItem(controls.dataTree.GetSelectedItem())) {
					// remove any displayed items from the image
					CComPtr<IFsiDirectoryItem> pRootDirectory;
					pFsImage->GetRoot(&pRootDirectory);
					if(pRootDirectory) {
						for(int itemIndex = 0; itemIndex < controls.dataList.GetItemCount(); itemIndex++) {
							CComBSTR discImagePath = controls.dataList.GetItemText(itemIndex, 4);
							if(controls.dataList.GetItemData(itemIndex) == ITEMTYPE_FOLDER) {
								pRootDirectory->RemoveTree(discImagePath);
								CString s = discImagePath;
								HTREEITEM hItem = FindTreeItemFromDiscImagePath(s);
								if(hItem)
									controls.dataTree.DeleteItem(hItem);
							} else
								pRootDirectory->Remove(discImagePath);
						}
						DisplayDirectoryContentForTreeItem(controls.dataTree.GetSelectedItem());
					}
				} else {
					// remove any items from the image
					CComPtr<IFsiDirectoryItem> pRootDirectory;
					pFsImage->GetRoot(&pRootDirectory);
					if(pRootDirectory) {
						CComBSTR bstr = TEXT("/");
						pRootDirectory->RemoveTree(bstr);
						AddFoldersToTree(controls.dataTree.GetRootItem(), pRootDirectory);
					}
				}
				UpdateBlocksInUse();
				break;

			case ID_REMOVEDATA:
			{
				// remove any selected item from the image
				CComPtr<IFsiDirectoryItem> pRootDirectory;
				pFsImage->GetRoot(&pRootDirectory);
				if(pRootDirectory) {
					POSITION p = controls.dataList.GetFirstSelectedItemPosition();
					while(p) {
						int itemIndex = controls.dataList.GetNextSelectedItem(p);
						CComBSTR discImagePath = controls.dataList.GetItemText(itemIndex, 4);
						if(controls.dataList.GetItemData(itemIndex) == ITEMTYPE_FOLDER) {
							pRootDirectory->RemoveTree(discImagePath);
							CString s = discImagePath;
							HTREEITEM hItem = FindTreeItemFromDiscImagePath(s);
							if(hItem)
								controls.dataTree.DeleteItem(hItem);
						} else
							pRootDirectory->Remove(discImagePath);
					}
					DisplayDirectoryContentForTreeItem(controls.dataTree.GetSelectedItem());
				}
				UpdateBlocksInUse();
				break;
			}
			case ID_NEWDIRECTORY:
				AddNewDirectory(controls.dataTree.GetSelectedItem());
				break;
		}
	}
}

HBRUSH CCreateDataDiscDlg::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
	HBRUSH hBrush = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);
	if(nCtlColor == CTLCOLOR_STATIC && pWnd->GetDlgCtrlID() == IDC_DISC_NOT_BLANK) {
		pDC->SetTextColor(RGB(255, 0, 0));
		pDC->SetBkMode(TRANSPARENT);
	}
	return hBrush;
}

void CCreateDataDiscDlg::OnCbnSelchangeFilesystemCombo()
{
	ASSERT(pFsImage != NULL);

	int selection = controls.fileSystemCombo.GetCurSel();
	if(selection < 0)
		return;
	FsiFileSystems fs = static_cast<FsiFileSystems>(controls.fileSystemCombo.GetItemData(selection));
	controls.isoLevelCombo.ShowWindow(fs & FsiFileSystemISO9660);
	controls.udfVersionCombo.ShowWindow(fs & FsiFileSystemUDF);

	if(!pFsImage->SetFileSystemsToCreate(fs))
		AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
	UpdateBlocksInUse();
}

void CCreateDataDiscDlg::OnCbnSelchangeIsolevelCombo()
{
	int selection = controls.isoLevelCombo.GetCurSel();
	if(selection < 0)
		return;

	if(!pFsImage->SetISOLevel(static_cast<LONG>(controls.isoLevelCombo.GetItemData(selection))))
		AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
	UpdateBlocksInUse();
}

void CCreateDataDiscDlg::OnCbnSelchangeUdfversionCombo()
{
	int selection = controls.udfVersionCombo.GetCurSel();
	if(selection < 0)
		return;

	if(!pFsImage->SetUDFRevision(static_cast<LONG>(controls.udfVersionCombo.GetItemData(selection))))
		AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
	UpdateBlocksInUse();
}

void CCreateDataDiscDlg::OnTvnBeginlabeleditFilesTree(NMHDR* pNMHDR, LRESULT* pResult)
{
	LPNMTVDISPINFO pTVDispInfo = reinterpret_cast<LPNMTVDISPINFO>(pNMHDR);
	*pResult = (controls.dataTree.GetParentItem(pTVDispInfo->item.hItem) != NULL);
	if(!labelEditTextToSet.IsEmpty()) {
		CEdit* pEdit = controls.dataTree.GetEditControl();
		if(pEdit)
			pEdit->SetWindowText(labelEditTextToSet);
		labelEditTextToSet = TEXT("");
	}
}

void CCreateDataDiscDlg::OnTvnEndlabeleditFilesTree(NMHDR* pNMHDR, LRESULT* pResult)
{
	ASSERT(pFsImage != NULL);

	labelEditTextToSet = TEXT("");

	LPNMTVDISPINFO pTVDispInfo = reinterpret_cast<LPNMTVDISPINFO>(pNMHDR);
	CString volumeName = pTVDispInfo->item.pszText;
	if(!pFsImage->SetVolumeName(volumeName)) {
		AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
		*pResult = FALSE;
		labelEditTextToSet = volumeName;
		controls.dataTree.PostMessage(TVM_EDITLABEL, 0, reinterpret_cast<LPARAM>(pTVDispInfo->item.hItem));
	} else
		*pResult = TRUE;
}

void CCreateDataDiscDlg::OnTvnKeydownFilesTree(NMHDR* pNMHDR, LRESULT* pResult)
{
	LPNMTVKEYDOWN pTVKeyDown = reinterpret_cast<LPNMTVKEYDOWN>(pNMHDR);
	switch(pTVKeyDown->wVKey) {
		case VK_F2:
			controls.dataTree.EditLabel(controls.dataTree.GetSelectedItem());
			break;
		case VK_DELETE:
			if(controls.dataTree.GetParentItem(controls.dataTree.GetSelectedItem()))
				RemoveTree(controls.dataTree.GetSelectedItem());
			break;
	}
	*pResult = 0;
}

void CCreateDataDiscDlg::OnTvnSelchangedFilesTree(NMHDR* pNMHDR, LRESULT* pResult)
{
	LPNMTREEVIEW pNMTreeView = reinterpret_cast<LPNMTREEVIEW>(pNMHDR);
	*pResult = 0;
	DisplayDirectoryContentForTreeItem(pNMTreeView->itemNew.hItem);
}

void CCreateDataDiscDlg::OnLvnItemActivateFilesList(NMHDR* pNMHDR, LRESULT* pResult)
{
	LPNMITEMACTIVATE pNMIA = reinterpret_cast<LPNMITEMACTIVATE>(pNMHDR);
	*pResult = 0;

	if(controls.dataList.GetItemData(pNMIA->iItem) == ITEMTYPE_FOLDER) {
		CString itemText = controls.dataList.GetItemText(pNMIA->iItem, 0);
		HTREEITEM hItem = controls.dataTree.GetChildItem(controls.dataTree.GetSelectedItem());
		while(hItem) {
			if(controls.dataTree.GetItemText(hItem) == itemText) {
				controls.dataTree.SelectItem(hItem);
				break;
			}
			hItem = controls.dataTree.GetNextSiblingItem(hItem);
		}
	}
}

void CCreateDataDiscDlg::OnLvnKeydownFilesList(NMHDR* pNMHDR, LRESULT* pResult)
{
	ASSERT(pFsImage != NULL);

	LPNMLVKEYDOWN pLVKeyDow = reinterpret_cast<LPNMLVKEYDOWN>(pNMHDR);
	switch(pLVKeyDow->wVKey) {
		case VK_DELETE:
		{
			// remove any selected item from the image
			CComPtr<IFsiDirectoryItem> pRootDirectory;
			pFsImage->GetRoot(&pRootDirectory);
			if(pRootDirectory) {
				POSITION p = controls.dataList.GetFirstSelectedItemPosition();
				while(p) {
					int itemIndex = controls.dataList.GetNextSelectedItem(p);
					CComBSTR discImagePath = controls.dataList.GetItemText(itemIndex, 4);
					if(controls.dataList.GetItemData(itemIndex) == ITEMTYPE_FOLDER) {
						pRootDirectory->RemoveTree(discImagePath);
						CString s = discImagePath;
						HTREEITEM hItem = FindTreeItemFromDiscImagePath(s);
						if(hItem)
							controls.dataTree.DeleteItem(hItem);
					} else
						pRootDirectory->Remove(discImagePath);
				}
				DisplayDirectoryContentForTreeItem(controls.dataTree.GetSelectedItem());
				UpdateBlocksInUse();
			}
			break;
		}
	}
	*pResult = 0;
}

void CCreateDataDiscDlg::OnBnClickedAddDataButton()
{
	CMenu menu;
	menu.LoadMenu(IDR_ADDDATA);
	TPMPARAMS tpmexParams = {0};
	tpmexParams.cbSize = sizeof(tpmexParams);
	controls.addDataButton.GetWindowRect(&tpmexParams.rcExclude);
	CPoint ptMenu(tpmexParams.rcExclude.left, tpmexParams.rcExclude.bottom);
	InflateRect(&tpmexParams.rcExclude, -1, -1);

	controls.addDataButton.SetState(TRUE);
	UINT menuItemID = menu.GetSubMenu(0)->TrackPopupMenuEx(TPM_LEFTALIGN | TPM_TOPALIGN | TPM_RETURNCMD | TPM_VERTICAL | TPM_VERPOSANIMATION, ptMenu.x, ptMenu.y, this, &tpmexParams);
	controls.addDataButton.SetState(FALSE);
	PostMessage(WM_COMMAND, MAKEWPARAM(menuItemID, 0), 0);
}

void CCreateDataDiscDlg::OnResetCompilation()
{
	CFileSystemImage* pNewFsImage = new CFileSystemImage();
	ASSERT(pNewFsImage != NULL);

	if(!pNewFsImage->Initialize(*this)) {
		AfxMessageBox(pNewFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
		delete pNewFsImage;
		return;
	}
	int selection = controls.fileSystemCombo.GetCurSel();
	if(selection < 0) {
		delete pNewFsImage;
		return;
	}
	FsiFileSystems fs = static_cast<FsiFileSystems>(controls.fileSystemCombo.GetItemData(selection));
	if(!pNewFsImage->SetFileSystemsToCreate(fs))
		AfxMessageBox(pNewFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
	selection = controls.isoLevelCombo.GetCurSel();
	if(selection < 0) {
		delete pNewFsImage;
		return;
	}
	if(!pNewFsImage->SetISOLevel(static_cast<LONG>(controls.isoLevelCombo.GetItemData(selection))))
		AfxMessageBox(pNewFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
	selection = controls.udfVersionCombo.GetCurSel();
	if(selection < 0) {
		delete pNewFsImage;
		return;
	}
	if(!pNewFsImage->SetUDFRevision(static_cast<LONG>(controls.udfVersionCombo.GetItemData(selection))))
		AfxMessageBox(pNewFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);

	if(pFsImage)
		delete pFsImage;
	pFsImage = pNewFsImage;

	controls.dataTree.DeleteAllItems();
	controls.dataList.DeleteAllItems();
	HTREEITEM h = controls.dataTree.InsertItem(TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE, pFsImage->GetVolumeName(), STOCKICON_FOLDER, STOCKICON_FOLDER_SELECTED, 0, 0, 0, TVI_ROOT, TVI_FIRST);
	controls.dataTree.SelectItem(h);
	SetRootIcon();

	ImportOldSession();
	UpdateBlocksInUse();
}

void CCreateDataDiscDlg::OnAddDirectory()
{
	ASSERT(pFsImage != NULL);

	CString initialDir = theApp.settings.GetSetting(CString("CreateDataDisc\\LastDir"));

	BROWSEINFO browseInfo = {0};
	browseInfo.hwndOwner = *this;
	browseInfo.ulFlags = BIF_RETURNONLYFSDIRS | BIF_NEWDIALOGSTYLE | BIF_UAHINT;
	if(!initialDir.IsEmpty()) {
		browseInfo.lpfn = SelectInitialDir;
		browseInfo.lParam = reinterpret_cast<LPARAM>(initialDir.GetBuffer(initialDir.GetLength()));
	}
	PIDLIST_ABSOLUTE pIDL = SHBrowseForFolder(&browseInfo);
	initialDir.ReleaseBuffer();
	if(pIDL) {
		TCHAR pBuffer[1024];
		SHGetPathFromIDList(pIDL, pBuffer);
		ILFree(pIDL);
		CString dirToAdd = pBuffer;
		theApp.settings.SetSetting(CString("CreateDataDisc\\LastDir"), _bstr_t(dirToAdd));

		IFsiDirectoryItem* pBaseDirectory = GetDirectoryForTreeItem(controls.dataTree.GetSelectedItem());
		if(pBaseDirectory) {
			if(PathIsRoot(dirToAdd)) {
				CComPtr<IFsiDirectoryItem> pDriveDirectory;
				HRESULT hr = pFsImage->CreateDirectoryItem(dirToAdd.Left(1), &pDriveDirectory);
				if(FAILED(hr))
					AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
				else {
					CTime t = CTime::GetCurrentTime();
					SYSTEMTIME st;
					t.GetAsSystemTime(st);
					COleDateTime creationTime(st);
					pDriveDirectory->put_CreationTime(creationTime.m_dt);
					pDriveDirectory->put_LastModifiedTime(creationTime.m_dt);
					pDriveDirectory->put_LastAccessedTime(creationTime.m_dt);

					hr = pBaseDirectory->Add(pDriveDirectory);
					if(FAILED(hr)) {
						CString s;
						s.Format(IDS_ERR_ADD, dirToAdd, hr);
						AfxMessageBox(s, MB_ICONERROR | MB_OK);
					} else
						AddDirectoryToImage(pDriveDirectory, dirToAdd, VARIANT_FALSE);
				}
			} else
				AddDirectoryToImage(pBaseDirectory, dirToAdd, VARIANT_TRUE);
			pBaseDirectory->Release();
		}
	}
}

void CCreateDataDiscDlg::OnAddDirectoryContent()
{
	CString initialDir = theApp.settings.GetSetting(CString("CreateDataDisc\\LastDir"));

	BROWSEINFO browseInfo = {0};
	browseInfo.hwndOwner = *this;
	browseInfo.ulFlags = BIF_RETURNONLYFSDIRS | BIF_NEWDIALOGSTYLE | BIF_UAHINT;
	if(!initialDir.IsEmpty()) {
		browseInfo.lpfn = SelectInitialDir;
		browseInfo.lParam = reinterpret_cast<LPARAM>(initialDir.GetBuffer(initialDir.GetLength()));
	}
	PIDLIST_ABSOLUTE pIDL = SHBrowseForFolder(&browseInfo);
	initialDir.ReleaseBuffer();
	if(pIDL) {
		TCHAR pBuffer[1024];
		SHGetPathFromIDList(pIDL, pBuffer);
		ILFree(pIDL);
		CString dirToAdd = pBuffer;
		theApp.settings.SetSetting(CString("CreateDataDisc\\LastDir"), _bstr_t(dirToAdd));

		IFsiDirectoryItem* pBaseDirectory = GetDirectoryForTreeItem(controls.dataTree.GetSelectedItem());
		if(pBaseDirectory) {
			AddDirectoryToImage(pBaseDirectory, dirToAdd, VARIANT_FALSE);
			pBaseDirectory->Release();
		}
	}
}

void CCreateDataDiscDlg::OnAddFile()
{
	ASSERT(pFsImage != NULL);

	CString filters;
	filters.LoadString(IDS_ADD_FILES_FILTERS);
	CFileDialog fileDlg(TRUE, NULL, NULL, OFN_ALLOWMULTISELECT | OFN_DONTADDTORECENT | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY, filters, this);
	if(IsVistaOrNewer())
		fileDlg.UpdateOFNFromShellDialog();
	CString fileName;
	const int MAX_FILES = 100;
	const int BUFFERSIZE = (MAX_FILES * (MAX_PATH + 1)) + 1;
	fileDlg.GetOFN().lpstrFile = fileName.GetBuffer(BUFFERSIZE);
	fileDlg.GetOFN().nMaxFile = BUFFERSIZE;

	if(fileDlg.DoModal() == IDOK) {
		IFsiDirectoryItem* pBaseDirectory = GetDirectoryForTreeItem(controls.dataTree.GetSelectedItem());
		if(pBaseDirectory) {
			CArray<CString> files;
			POSITION p = fileDlg.GetStartPosition();
			while(p)
				files.Add(fileDlg.GetNextPathName(p));
			if(!pFsImage->AddFiles(pBaseDirectory, files, FALSE))
				AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
		}
	}
	fileName.ReleaseBuffer();
}

void CCreateDataDiscDlg::OnBurn()
{
	ASSERT(pFsImage != NULL);

	if(!pGalbaWindow)
		return;

	LONG directories = pFsImage->GetTotalDirectories();
	if(directories == -1) {
		AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
		return;
	}
	LONG files = pFsImage->GetTotalFiles();
	if(files == -1) {
		AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
		return;
	}

	// the root directory counts as directory
	if(directories <= 1 && files == 0) {
		CString message;
		message.LoadString(IDS_EMPTY_COMPILATION);
		int answer = AfxMessageBox(message, MB_ICONQUESTION | MB_YESNO);
		if(answer == IDNO)
			return;
	}

	CString recorderID;
	BOOL imageRecorderSelected = FALSE;
	pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	if(imageRecorderSelected) {
		recorderID = IMAGE_RECORDER_ID;
		CString s = pFsImage->GetVolumeName();
		recordingController.SetVolumeName(s);
	} else {
		CDiscRecorder* pRecorder = NULL;
		pGalbaWindow->GetRecorder(&pRecorder);
		if(!pRecorder)
			return;
		recorderID = pRecorder->GetUniqueId();
	}

	recordingController.SetRecorderID(recorderID);
	recordingController.SetClientName(CLIENT_NAME);
	recordingController.SetNotificationWindow(*this);
	recordingController.SetWriteSpeed(GetWriteSpeed());
	recordingController.EnableBurnProof(true);
	recordingController.SetCloseMedia(controls.closeMediaCheck.GetCheck() == BST_CHECKED);
	recordingController.SetEjectMedia(controls.ejectCheck.GetCheck() == BST_CHECKED);

	ASSERT(pProgressItems == NULL);
	CComPtr<IFileSystemImageResult> pImageResult;
	LPSTREAM pData = pFsImage->GetStream(&pImageResult);
	if(!pData) {
		AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
		return;
	}
	if(imageRecorderSelected) {
		LONG l = 0;
		pImageResult->get_TotalBlocks(&l);
		recordingController.SetTotalBlocks(l);
		l = 0;
		pImageResult->get_BlockSize(&l);
		recordingController.SetBlockSize(l);
	}

	pImageResult->get_ProgressItems(&pProgressItems);
	if(!recordingController.Burn(pData)) {
		if(pProgressItems) {
			pProgressItems->Release();
			pProgressItems = NULL;
		}
		AfxMessageBox(recordingController.GetErrorMessage(), MB_ICONERROR | MB_OK);
	}
	pData->Release();
}


LRESULT CCreateDataDiscDlg::OnImapiBeginAddData(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	runningInterruptibleTask = !!allowCancelationOfAddingData;

	EnableUI(FALSE);
	if(pGalbaWindow)
		pGalbaWindow->EnableCancelButton(allowCancelationOfAddingData);
	return 0;
}

LRESULT CCreateDataDiscDlg::OnImapiEndAddData(WPARAM wParam, LPARAM lParam)
{
	ASSERT(pFsImage != NULL);

	runningInterruptibleTask = false;
	allowCancelationOfAddingData = TRUE;

	switch(wParam) {
		case S_OK:
			// success;
		case S_FALSE:
			// canceled by user
			break;
		default:
			// error
			AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
			break;
	}

	if(lParam & DATATYPE_TREE) {
		IFsiDirectoryItem* pBaseDirectory = GetDirectoryForTreeItem(controls.dataTree.GetSelectedItem());
		if(pBaseDirectory) {
			BSTR fullPath;
			pBaseDirectory->get_FullPath(&fullPath);
			CString s = fullPath;
			s.Replace(TEXT("\\"), TEXT("/"));
			HTREEITEM hItem = FindTreeItemFromDiscImagePath(s);
			SysFreeString(fullPath);

			AddFoldersToTree(hItem, pBaseDirectory);
			controls.dataTree.Expand(controls.dataTree.GetRootItem(), TVE_EXPAND);
			controls.dataTree.Invalidate();

			pBaseDirectory->Release();
		}
	} else					// for DATATYPE_TREE, this is done by AddFoldersToTree
		DisplayDirectoryContentForTreeItem(controls.dataTree.GetSelectedItem());
	// UpdateBlocksInUse does some calls that won't work if we come from another thread
	PostMessage(WM_UPDATE_BLOCKS_IN_USE);

	EnableUI(TRUE);
	if(pGalbaWindow) {
		CString s;
		pGalbaWindow->SetProgress(0, s);
		pGalbaWindow->EnableCancelButton(FALSE);
	}

	// allow other data to be added
	SetEvent(hAddedDataEvent);
	return 0;
}

LRESULT CCreateDataDiscDlg::OnImapiImageUpdate(WPARAM /*wParam*/, LPARAM lParam)
{
	if(pGalbaWindow) {
		PIMAPI_IMAGE_UPDATE_EVENT_ARGS pDetails = reinterpret_cast<PIMAPI_IMAGE_UPDATE_EVENT_ARGS>(lParam);
		CString progressText;
		progressText.Format(IDS_STATUS_IMPORTING, pDetails->currentFile);
		int percentage = 0;
		if(pDetails->totalSectors > 0)
			percentage = pDetails->copiedSectors * 100 / pDetails->totalSectors;
		pGalbaWindow->SetProgress(percentage, progressText);
	}
	return 0;
}

LRESULT CCreateDataDiscDlg::OnImapiBeginBurn(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	runningInterruptibleTask = true;

	EnableUI(FALSE);
	if(pGalbaWindow) {
		pGalbaWindow->EnableUI(FALSE);
		pGalbaWindow->EnableCancelButton(TRUE);
	}
	return 0;
}

LRESULT CCreateDataDiscDlg::OnImapiEndBurn(WPARAM wParam, LPARAM /*lParam*/)
{
	runningInterruptibleTask = false;
	if(pGalbaWindow) {
		pGalbaWindow->SetupProgressMarquee(FALSE);
		pGalbaWindow->SetSystemBufferUsage(0);
		pGalbaWindow->SetTimes(-1, -1);
	}

	CString message;
	switch(wParam) {
		case S_OK:
			// success;
			message.LoadString(IDS_RECORDING_COMPLETE);
			AfxMessageBox(message, MB_ICONINFORMATION | MB_OK);
			break;
		case E_IMAPI_RECORDER_MEDIA_NO_MEDIA:
		case E_IMAPI_DF2DATA_INVALID_MEDIA_STATE:
			// no writable media inserted
			PostMessage(WM_EJECT_MEDIA, 0, 0);
			message.LoadString(IDS_RECORDING_FAILED_NO_WRITABLE_MEDIA);
			AfxMessageBox(message, MB_ICONERROR | MB_OK);
			break;
		default:
			// error
			AfxMessageBox(recordingController.GetErrorMessage(), MB_ICONERROR | MB_OK);
			break;
	}

	if(pProgressItems) {
		pProgressItems->Release();
		pProgressItems = NULL;
	}

	EnableUI(TRUE);
	if(pGalbaWindow) {
		CString s;
		pGalbaWindow->SetProgress(0, s);
		pGalbaWindow->EnableCancelButton(FALSE);
		pGalbaWindow->DeviceChange();
		pGalbaWindow->EnableUI(TRUE);
	}
	return 0;
}

LRESULT CCreateDataDiscDlg::OnImapiFormatDataUpdate(WPARAM /*wParam*/, LPARAM lParam)
{
	if(pGalbaWindow) {
		PIMAPI_FORMAT_DATA_UPDATE_EVENT_ARGS pDetails = reinterpret_cast<PIMAPI_FORMAT_DATA_UPDATE_EVENT_ARGS>(lParam);
		CString progressText;
		switch(pDetails->currentAction) {
			case IMAPI_FORMAT2_DATA_WRITE_ACTION_VALIDATING_MEDIA:
				progressText.LoadString(IDS_STATUS_VALIDATING_MEDIA);
				break;
			case IMAPI_FORMAT2_DATA_WRITE_ACTION_FORMATTING_MEDIA:
				progressText.LoadString(IDS_STATUS_FORMATTING_MEDIA);
				break;
			case IMAPI_FORMAT2_DATA_WRITE_ACTION_INITIALIZING_HARDWARE:
				progressText.LoadString(IDS_STATUS_INITIALIZING_RECORDER);
				break;
			case IMAPI_FORMAT2_DATA_WRITE_ACTION_CALIBRATING_POWER:
				progressText.LoadString(IDS_STATUS_CALIBRATING_POWER);
				break;
			case IMAPI_FORMAT2_DATA_WRITE_ACTION_WRITING_DATA:
			{
				BOOL b = FALSE;
				if(pProgressItems) {
					CComPtr<IProgressItem> pProgressItem;
					pProgressItems->ProgressItemFromBlock(pDetails->lastWrittenLba, &pProgressItem);
					if(pProgressItem) {
						CComBSTR bstr;
						if(SUCCEEDED(pProgressItem->get_Description(&bstr))) {
							b = TRUE;
							CString s = bstr;
							s.Replace(TEXT("\\"), TEXT("/"));
							progressText.Format(IDS_STATUS_WRITING_DATA2, s);
						}
					}
				}
				if(!b)
					progressText.LoadString(IDS_STATUS_WRITING_DATA1);

				int percentage = 0;
				if(pDetails->totalSystemBuffer > 0)
					percentage = pDetails->usedSystemBuffer * 100 / pDetails->totalSystemBuffer;
				pGalbaWindow->SetSystemBufferUsage(percentage);
				pGalbaWindow->SetTimes(pDetails->elapsedTime, pDetails->remainingTime);
				break;
			}
			case IMAPI_FORMAT2_DATA_WRITE_ACTION_FINALIZATION:
				progressText.LoadString(IDS_STATUS_FINALIZING_MEDIA);
				pGalbaWindow->SetupProgressMarquee(TRUE);
				pGalbaWindow->SetTimes(-1, -1);
				pGalbaWindow->EnableCancelButton(FALSE);
				break;
			case IMAPI_FORMAT2_DATA_WRITE_ACTION_COMPLETED:
				progressText.LoadString(IDS_STATUS_RECORDING_COMPLETED);
				break;
			case IMAPI_FORMAT2_DATA_WRITE_ACTION_VERIFYING:
				progressText.LoadString(IDS_STATUS_VERIFYING_DATA);
				break;
		}
		int percentage = 0;
		if(pDetails->sectorCount > 0)
			percentage = pDetails->lastWrittenLba * 100 / pDetails->sectorCount;
		pGalbaWindow->SetProgress(percentage, progressText);
	}
	return (recordingController.GetCancelFlag() ? RETURN_CANCEL_WRITE : RETURN_CONTINUE_WRITE);
}

LRESULT CCreateDataDiscDlg::OnEjectMedia(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	if(!pGalbaWindow)
		return 0;

	CDiscRecorder* pRecorder = NULL;
	pGalbaWindow->GetRecorder(&pRecorder);
	if(pRecorder)
		pRecorder->EjectMedia();
	return 0;
}

LRESULT CCreateDataDiscDlg::OnUpdateBlocksInUse(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	UpdateBlocksInUse();
	return 0;
}

LRESULT CCreateDataDiscDlg::OnUpdateCompilationView(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	DisplayDirectoryContentForTreeItem(controls.dataTree.GetRootItem());
	UpdateBlocksInUse();
	return 0;
}


int CCreateDataDiscDlg::CompareItems(LPARAM item1, LPARAM item2, LPARAM lParamSort)
{
	int itemIndex1 = static_cast<int>(item1);
	int itemIndex2 = static_cast<int>(item2);

	HWND hWndLV = reinterpret_cast<HWND>(lParamSort);
	LVITEM item = {0};
	item.mask = LVIF_PARAM;
	item.iItem = itemIndex1;
	::SendMessage(hWndLV, LVM_GETITEM, 0, reinterpret_cast<LPARAM>(&item));
	LPARAM itemType1 = item.lParam;
	item.iItem = itemIndex2;
	::SendMessage(hWndLV, LVM_GETITEM, 0, reinterpret_cast<LPARAM>(&item));
	LPARAM itemType2 = item.lParam;

	if(itemType1 != itemType2) {
		if(itemType1 == ITEMTYPE_FOLDER)
			return -1;
		else
			return 1;
	} else {
		TCHAR pBuffer[1024];
		item.pszText = pBuffer;
		item.cchTextMax = 1024;
		::SendMessage(hWndLV, LVM_GETITEMTEXT, itemIndex1, reinterpret_cast<LPARAM>(&item));
		CString itemText1 = pBuffer;
		::SendMessage(hWndLV, LVM_GETITEMTEXT, itemIndex2, reinterpret_cast<LPARAM>(&item));
		CString itemText2 = pBuffer;
		return itemText1.CompareNoCase(itemText2);
	}
}

int CCreateDataDiscDlg::SelectInitialDir(HWND hWnd, UINT message, LPARAM /*lParam*/, LPARAM pData)
{
	if(message == BFFM_INITIALIZED) {
		::SendMessage(hWnd, BFFM_SETSELECTION, TRUE, pData);
	}
	return 0;
}


void CCreateDataDiscDlg::AddFileSystemsToComboBox()
{
	ASSERT(pFsImage != NULL);

	controls.fileSystemCombo.ResetContent();
	controls.isoLevelCombo.ResetContent();
	controls.udfVersionCombo.ResetContent();

	FsiFileSystems fs = pFsImage->GetSupportedFileSystems();
	if(fs == FsiFileSystemNone) {
		AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
		OnCbnSelchangeFilesystemCombo();
		return;
	}

	if(fs & FsiFileSystemISO9660) {
		int itemIndex = controls.fileSystemCombo.AddString(TEXT("ISO9660"));
		controls.fileSystemCombo.SetItemData(itemIndex, FsiFileSystemISO9660);
		controls.fileSystemCombo.SetCurSel(itemIndex);
	}
	if(fs & FsiFileSystemJoliet) {
		if(fs & FsiFileSystemISO9660) {
			int itemIndex = controls.fileSystemCombo.AddString(TEXT("ISO9660 + Joliet"));
			controls.fileSystemCombo.SetItemData(itemIndex, FsiFileSystemISO9660 | FsiFileSystemJoliet);
			controls.fileSystemCombo.SetCurSel(itemIndex);
		}
		// Joliet requires ISO9660
		/*int itemIndex = controls.fileSystemCombo.AddString(TEXT("Joliet"));
		controls.fileSystemCombo.SetItemData(itemIndex, FsiFileSystemJoliet);*/
	}
	if(fs & FsiFileSystemUDF) {
		if(fs & FsiFileSystemJoliet) {
			if(fs & FsiFileSystemISO9660) {
				int itemIndex = controls.fileSystemCombo.AddString(TEXT("ISO9660 + Joliet + UDF"));
				controls.fileSystemCombo.SetItemData(itemIndex, FsiFileSystemISO9660 | FsiFileSystemJoliet | FsiFileSystemUDF);
				if(CurrentMediaIsDVDOrLarger())
					controls.fileSystemCombo.SetCurSel(itemIndex);
			}
			// Joliet requires ISO9660
			/*int itemIndex = controls.fileSystemCombo.AddString(TEXT("Joliet + UDF"));
			controls.fileSystemCombo.SetItemData(itemIndex, FsiFileSystemJoliet | FsiFileSystemUDF);*/
		}
		int itemIndex = controls.fileSystemCombo.AddString(TEXT("UDF"));
		controls.fileSystemCombo.SetItemData(itemIndex, FsiFileSystemUDF);
	}

	if(fs & FsiFileSystemISO9660) {
		ULONG c = pFsImage->GetTotalSupportedISOLevels();
		if(c > 0) {
			for(ULONG i = 0; i < c; i++) {
				LONG level = pFsImage->GetSupportedISOLevel(i);
				CString s;
				s.Format(TEXT("ISO9660 Level %i"), level);
				int itemIndex = controls.isoLevelCombo.AddString(s);
				controls.isoLevelCombo.SetItemData(itemIndex, level);
			}
			// default to Level 2
			controls.isoLevelCombo.SetCurSel(controls.isoLevelCombo.GetCount() > 1 ? 1 : 0);
		} else
			AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
	}

	if(fs & FsiFileSystemUDF) {
		ULONG c = pFsImage->GetTotalSupportedUDFRevisions();
		if(c > 0) {
			for(ULONG i = 0; i < c; i++) {
				LONG revision = pFsImage->GetSupportedUDFRevision(i);
				CString s;
				s.Format(TEXT("UDF v%i.%02i"), revision / 0x100, revision % 0x100);
				int itemIndex = controls.udfVersionCombo.AddString(s);
				controls.udfVersionCombo.SetItemData(itemIndex, revision);
			}
			controls.udfVersionCombo.SetCurSel(0);
		} else
			AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
	}
	OnCbnSelchangeFilesystemCombo();
}

void CCreateDataDiscDlg::AddSpeedSettingsToComboBox()
{
	alwaysDisableSpeedComboBox = FALSE;
	controls.speedCombo.ResetContent();
	CString speedString;
	speedString.LoadString(IDS_MAXIMUM_SPEED);
	int speedIndex = controls.speedCombo.AddString(speedString);
	controls.speedCombo.SetItemData(speedIndex, (sizeof(INT_PTR) == 4 ? 0xFFFFFFFF : 0xFFFFFFFFFFFFFFFF));

	if(!pGalbaWindow)
		return;

	BOOL imageRecorderSelected = FALSE;
	pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	if(imageRecorderSelected) {
		controls.speedCombo.SetCurSel(0);
		controls.speedCombo.EnableWindow(FALSE);
		alwaysDisableSpeedComboBox = TRUE;
		return;
	}

	CDiscRecorder* pRecorder = NULL;
	pGalbaWindow->GetRecorder(&pRecorder);
	if(!pRecorder)
		return;

	CDiscFormatData discFormatData;
	if(!discFormatData.Initialize(pRecorder, CLIENT_NAME, *this)) {
		AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
		return;
	}
	if(!discFormatData.IsCurrentMediaSupported(pRecorder)) {
		controls.speedCombo.SetCurSel(0);
		controls.speedCombo.EnableWindow(FALSE);
		alwaysDisableSpeedComboBox = TRUE;
		return;
	}
	controls.speedCombo.EnableWindow(TRUE);

	ULONG singleSpeed = GetMediaTypeSingleSpeed(discFormatData.GetCurrentPhysicalMediaType());
	ULONG c = discFormatData.GetTotalSupportedWriteSpeeds();
	for(ULONG i = 0; i < c; i++) {
		ULONG originalSpeed = discFormatData.GetSupportedWriteSpeed(i);
		ULONG speed = originalSpeed;
		if(singleSpeed == IMAPI_SECTORS_PER_SECOND_AT_1X_CD) {
			if(speed % 100 != 0) {
				// round-up to 100 sectors
				speed = ((speed / 100) + 1) * 100;
			}
			ULONG speedUp = speed / singleSpeed;
			speedString.Format(TEXT("%ix"), speedUp);
		} else {
			float speedUp = static_cast<float>(speed) / static_cast<float>(singleSpeed);
			speedUp *= 0.875f;
			speedString.Format(TEXT("%.1fx"), speedUp);
		}

		speedIndex = controls.speedCombo.AddString(speedString);
		controls.speedCombo.SetItemData(speedIndex, originalSpeed);
	}
	controls.speedCombo.SetCurSel(0);
}

void CCreateDataDiscDlg::CheckForBlankMedia()
{
	if(!pGalbaWindow)
		return;

	BOOL blank = TRUE;

	BOOL imageRecorderSelected = FALSE;
	pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	if(!imageRecorderSelected) {
		CDiscRecorder* pRecorder = NULL;
		pGalbaWindow->GetRecorder(&pRecorder);
		if(!pRecorder)
			return;
		CDiscFormatData discFormatData;
		if(!discFormatData.Initialize(pRecorder, CLIENT_NAME, *this)) {
			AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
			return;
		}
		blank = discFormatData.IsMediaBlank();
	}

	//EnableUI(blank);
	controls.discNotBlank.ShowWindow(blank ? SW_HIDE : SW_SHOW);
}

IMAPI_MEDIA_PHYSICAL_TYPE CCreateDataDiscDlg::GetCurrentMediaType(CDiscRecorder* pRecorder)
{
	CDiscFormatData discFormatData;
	if(!discFormatData.Initialize(pRecorder, CLIENT_NAME, *this)) {
		AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
		return IMAPI_MEDIA_TYPE_UNKNOWN;
	}

	IMAPI_MEDIA_PHYSICAL_TYPE mediaType = discFormatData.GetCurrentPhysicalMediaType();
	HRESULT lastError = discFormatData.GetHresult();
	// Some devices, e.g. from Samsung fail with E_IMAPI_RECORDER_INVALID_RESPONSE_FROM_DEVICE if there is no media.
	if(mediaType == IMAPI_MEDIA_TYPE_UNKNOWN && lastError != E_IMAPI_RECORDER_MEDIA_NO_MEDIA && lastError != E_IMAPI_RECORDER_INVALID_RESPONSE_FROM_DEVICE)
		AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
	return mediaType;
}

bool CCreateDataDiscDlg::CurrentMediaIsCD()
{
	if(!pGalbaWindow)
		return false;

	BOOL imageRecorderSelected = FALSE;
	pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	if(!imageRecorderSelected) {
		CDiscRecorder* pRecorder = NULL;
		pGalbaWindow->GetRecorder(&pRecorder);
		if(!pRecorder)
			return false;

		switch(GetCurrentMediaType(pRecorder)) {
			case IMAPI_MEDIA_TYPE_CDROM:
			case IMAPI_MEDIA_TYPE_CDR:
			case IMAPI_MEDIA_TYPE_CDRW:
				return true;
		}
	}
	return false;
}

bool CCreateDataDiscDlg::CurrentMediaIsDVDOrLarger()
{
	if(!pGalbaWindow)
		return false;

	BOOL imageRecorderSelected = FALSE;
	pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	if(imageRecorderSelected)
		// assume a DVD
		return true;
	else {
		CDiscRecorder* pRecorder = NULL;
		pGalbaWindow->GetRecorder(&pRecorder);
		if(!pRecorder)
			return false;

		switch(GetCurrentMediaType(pRecorder)) {
			case IMAPI_MEDIA_TYPE_DVDROM:
			case IMAPI_MEDIA_TYPE_DVDRAM:
			case IMAPI_MEDIA_TYPE_DVDPLUSR:
			case IMAPI_MEDIA_TYPE_DVDPLUSRW:
			case IMAPI_MEDIA_TYPE_DVDPLUSR_DUALLAYER:
			case IMAPI_MEDIA_TYPE_DVDDASHR:
			case IMAPI_MEDIA_TYPE_DVDDASHRW:
			case IMAPI_MEDIA_TYPE_DVDDASHR_DUALLAYER:
			case IMAPI_MEDIA_TYPE_DVDPLUSRW_DUALLAYER:
			case IMAPI_MEDIA_TYPE_HDDVDROM:
			case IMAPI_MEDIA_TYPE_HDDVDR:
			case IMAPI_MEDIA_TYPE_HDDVDRAM:
			case IMAPI_MEDIA_TYPE_BDROM:
			case IMAPI_MEDIA_TYPE_BDR:
			case IMAPI_MEDIA_TYPE_BDRE:
				return true;
			case IMAPI_MEDIA_TYPE_UNKNOWN:
				// no media inserted, assume a DVD
				return true;
		}
		return false;
	}
}

void CCreateDataDiscDlg::EnableUI(BOOL enable)
{
	controls.fileSystemCombo.EnableWindow(enable);
	controls.isoLevelCombo.EnableWindow(enable);
	controls.udfVersionCombo.EnableWindow(enable);
	controls.resetCompilationButton.EnableWindow(enable);
	controls.dataTree.EnableWindow(enable);
	controls.dataList.EnableWindow(enable);
	controls.addDataButton.EnableWindow(enable);
	controls.speedCombo.EnableWindow(!alwaysDisableSpeedComboBox && enable);
	controls.burnButton.EnableWindow(enable);

	BOOL imageRecorderSelected = FALSE;
	if(pGalbaWindow)
		pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	controls.closeMediaCheck.EnableWindow(!imageRecorderSelected && enable);
	controls.ejectCheck.EnableWindow(!imageRecorderSelected && enable);
}

LONGLONG CCreateDataDiscDlg::GetFreeBytesOnMedia(CDiscRecorder* pRecorder)
{
	CDiscFormatData discFormatData;
	if(!discFormatData.Initialize(pRecorder, CLIENT_NAME, *this)) {
		AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
		return 0;
	}

	LONG capacity = discFormatData.GetFreeSectorsOnMedia();
	if(capacity == -1) {
		AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
		return 0;
	}
	if(capacity == 0) {
		switch(GetCurrentMediaType(pRecorder)) {
			case IMAPI_MEDIA_TYPE_UNKNOWN:
				// no media inserted, assume a DVD
				capacity = 4700000000 / IMAPI_SECTOR_SIZE;
				break;
			case IMAPI_MEDIA_TYPE_CDROM:
				// no writable media inserted, assume a CD
				capacity = 734003200 / IMAPI_SECTOR_SIZE;
				break;
			case IMAPI_MEDIA_TYPE_DVDROM:
			case IMAPI_MEDIA_TYPE_HDDVDROM:
			case IMAPI_MEDIA_TYPE_BDROM:
			default:
				// no writable media inserted, assume a DVD
				capacity = 4700000000 / IMAPI_SECTOR_SIZE;
				break;
		}
	}
	LONGLONG ll = capacity;
	ll *= IMAPI_SECTOR_SIZE;
	return ll;
}

ULONG	CCreateDataDiscDlg::GetMediaTypeSingleSpeed(IMAPI_MEDIA_PHYSICAL_TYPE mediaType)
{
	switch(mediaType) {
		case IMAPI_MEDIA_TYPE_UNKNOWN:
		case IMAPI_MEDIA_TYPE_DISK:
		default:
			// assume CD
		case IMAPI_MEDIA_TYPE_CDROM:
		case IMAPI_MEDIA_TYPE_CDR:
		case IMAPI_MEDIA_TYPE_CDRW:
			return IMAPI_SECTORS_PER_SECOND_AT_1X_CD;
		case IMAPI_MEDIA_TYPE_DVDROM:
		case IMAPI_MEDIA_TYPE_DVDRAM:
		case IMAPI_MEDIA_TYPE_DVDPLUSR:
		case IMAPI_MEDIA_TYPE_DVDPLUSRW:
		case IMAPI_MEDIA_TYPE_DVDPLUSR_DUALLAYER:
		case IMAPI_MEDIA_TYPE_DVDDASHR:
		case IMAPI_MEDIA_TYPE_DVDDASHRW:
		case IMAPI_MEDIA_TYPE_DVDDASHR_DUALLAYER:
		case IMAPI_MEDIA_TYPE_DVDPLUSRW_DUALLAYER:
			return IMAPI_SECTORS_PER_SECOND_AT_1X_DVD;
		case IMAPI_MEDIA_TYPE_HDDVDROM:
		case IMAPI_MEDIA_TYPE_HDDVDR:
		case IMAPI_MEDIA_TYPE_HDDVDRAM:
			return IMAPI_SECTORS_PER_SECOND_AT_1X_HD_DVD;
		case IMAPI_MEDIA_TYPE_BDROM:
		case IMAPI_MEDIA_TYPE_BDR:
		case IMAPI_MEDIA_TYPE_BDRE:
			return IMAPI_SECTORS_PER_SECOND_AT_1X_BD;
	}
}

SHSTOCKICONID CCreateDataDiscDlg::GetMediaTypeStockIcon(IMAPI_MEDIA_PHYSICAL_TYPE mediaType)
{
	switch(mediaType) {
		case IMAPI_MEDIA_TYPE_CDROM:
			return SIID_MEDIACDROM;
		case IMAPI_MEDIA_TYPE_CDR:
			return SIID_MEDIACDR;
		case IMAPI_MEDIA_TYPE_CDRW:
			return SIID_MEDIACDRW;
		case IMAPI_MEDIA_TYPE_DVDROM:
			return SIID_MEDIADVDROM;
		case IMAPI_MEDIA_TYPE_DVDRAM:
			return SIID_MEDIADVDRAM;
		case IMAPI_MEDIA_TYPE_DVDPLUSR:
		case IMAPI_MEDIA_TYPE_DVDPLUSR_DUALLAYER:
			return SIID_MEDIADVDPLUSR;
		case IMAPI_MEDIA_TYPE_DVDPLUSRW:
		case IMAPI_MEDIA_TYPE_DVDPLUSRW_DUALLAYER:
			return SIID_MEDIADVDPLUSRW;
		case IMAPI_MEDIA_TYPE_DVDDASHR:
		case IMAPI_MEDIA_TYPE_DVDDASHR_DUALLAYER:
			return SIID_MEDIADVDR;
		case IMAPI_MEDIA_TYPE_DVDDASHRW:
			return SIID_MEDIADVDRW;
		case IMAPI_MEDIA_TYPE_HDDVDROM:
			return SIID_MEDIAHDDVDROM;
		case IMAPI_MEDIA_TYPE_HDDVDR:
			return SIID_MEDIAHDDVDR;
		case IMAPI_MEDIA_TYPE_HDDVDRAM:
			return SIID_MEDIAHDDVDRAM;
		case IMAPI_MEDIA_TYPE_BDROM:
			return SIID_MEDIABDROM;
		case IMAPI_MEDIA_TYPE_BDR:
			return SIID_MEDIABDR;
		case IMAPI_MEDIA_TYPE_BDRE:
			return SIID_MEDIABDRE;
		default:
			return SIID_MEDIABLANKCD;
	}
}

LONG CCreateDataDiscDlg::GetWriteSpeed()
{
	int selection = controls.speedCombo.GetCurSel();
	if(selection >= 0)
		return static_cast<LONG>(controls.speedCombo.GetItemData(selection));
	return -1;
}

void CCreateDataDiscDlg::ImportOldSession()
{
	if(!pGalbaWindow)
		return;
	ASSERT(pFsImage != NULL);

	CDiscRecorder* pRecorder = NULL;
	pGalbaWindow->GetRecorder(&pRecorder);
	if(!pRecorder)
		return;

	CDiscFormatData dataWriter;
	if(dataWriter.Initialize(pRecorder, CLIENT_NAME, *this)) {
		SAFEARRAY* pMultiSession = NULL;
		HRESULT hr = dataWriter.GetMultisessionInterfaces(&pMultiSession);
		if(SUCCEEDED(hr)) {
			hr = pFsImage->SetMultisessionInterfaces(pMultiSession);
			if(FAILED(hr))
				SafeArrayDestroy(pMultiSession);
			else
				pFsImage->ImportDefaultSession();
		}
	}

	CComPtr<IFsiDirectoryItem> pRootDirectory;
	pFsImage->GetRoot(&pRootDirectory);
	if(pRootDirectory)
		AddFoldersToTree(controls.dataTree.GetRootItem(), pRootDirectory);
}

void CCreateDataDiscDlg::SetCapacity()
{
	if(!pGalbaWindow)
		return;

	LONGLONG capacity = 0;

	BOOL imageRecorderSelected = FALSE;
	pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	if(!imageRecorderSelected) {
		CDiscRecorder* pRecorder = NULL;
		pGalbaWindow->GetRecorder(&pRecorder);
		if(!pRecorder)
			return;
		capacity = GetFreeBytesOnMedia(pRecorder);
	}

	TCHAR pBuffer[100];
	StrFormatByteSize(capacity, pBuffer, 100);
	controls.maxText.SetWindowText(pBuffer);
}

void CCreateDataDiscDlg::SetRootIcon()
{
	if(!pGalbaWindow || !IsVistaOrNewer())
		return;

	SHSTOCKICONID icon;

	BOOL imageRecorderSelected = FALSE;
	pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	if(imageRecorderSelected)
		icon = GetMediaTypeStockIcon(IMAPI_MEDIA_TYPE_BDR);
	else {
		CDiscRecorder* pRecorder = NULL;
		pGalbaWindow->GetRecorder(&pRecorder);
		if(!pRecorder)
			return;
		icon = GetMediaTypeStockIcon(GetCurrentMediaType(pRecorder));
	}

	if(CAPIWrapper::IsSupported_SHGetStockIconInfo()) {
		SHSTOCKICONINFO sii = {0};
		sii.cbSize = sizeof(sii);
		CAPIWrapper::SHGetStockIconInfo(icon, SHGSI_SYSICONINDEX, &sii, NULL);
		controls.dataTree.SetItemImage(controls.dataTree.GetRootItem(), sii.iSysImageIndex, sii.iSysImageIndex);
	}
}

void CCreateDataDiscDlg::UpdateBlocksInUse()
{
	ASSERT(pFsImage != NULL);

	if(!pGalbaWindow)
		return;

	LONG freeBlocks = 0;

	BOOL imageRecorderSelected = FALSE;
	pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	if(!imageRecorderSelected) {
		CDiscRecorder* pRecorder = NULL;
		pGalbaWindow->GetRecorder(&pRecorder);
		if(!pRecorder)
			return;
		CDiscFormatData discFormatData;
		if(!discFormatData.Initialize(pRecorder, CLIENT_NAME, *this)) {
			AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
			return;
		}
		freeBlocks = discFormatData.GetFreeSectorsOnMedia();
		if(freeBlocks == 0) {
			switch(GetCurrentMediaType(pRecorder)) {
				case IMAPI_MEDIA_TYPE_UNKNOWN:
					// no media inserted, assume a DVD
					freeBlocks = 4700000000 / IMAPI_SECTOR_SIZE;
					break;
				case IMAPI_MEDIA_TYPE_CDROM:
					// no writable media inserted, assume a CD
					freeBlocks = 734003200 / IMAPI_SECTOR_SIZE;
					break;
				case IMAPI_MEDIA_TYPE_DVDROM:
				case IMAPI_MEDIA_TYPE_HDDVDROM:
				case IMAPI_MEDIA_TYPE_BDROM:
				default:
					// no writable media inserted, assume a DVD
					freeBlocks = 4700000000 / IMAPI_SECTOR_SIZE;
					break;
			}
		}
		if(freeBlocks == -1) {
			AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
			return;
		}
	}

	LONG blocksInUse = pFsImage->GetBlocksInUse();
	if(blocksInUse > -1) {
		int percentage = 0;
		if(freeBlocks > 0)
			percentage = blocksInUse * 100 / freeBlocks;
		controls.capacityProg.SetPos(percentage);
		if(IsVistaOrNewer()) {
			if(percentage > 100)
				controls.capacityProg.SetState(PBST_ERROR);
			else
				controls.capacityProg.SetState(PBST_NORMAL);
		}

		LONGLONG ll = blocksInUse;
		ll *= IMAPI_SECTOR_SIZE;
		TCHAR pBuffer[100];
		StrFormatByteSize(ll, pBuffer, 100);
		CString s;
		s.Format(IDS_COMPILATION_SIZE, pBuffer);
		controls.currentSize.SetWindowText(s);
	} else
		AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
}


void CCreateDataDiscDlg::AddNewDirectory(HTREEITEM hParentItem)
{
	ASSERT(pFsImage != NULL);

	CNewDirectoryDlg dlg;
	if(dlg.DoModal() == IDOK) {
		CString dirName = dlg.GetName();
		IFsiDirectoryItem* pBaseDirectory = GetDirectoryForTreeItem(hParentItem);
		if(pBaseDirectory) {
			CComPtr<IFsiDirectoryItem> pNewDirectory;
			if(FAILED(pFsImage->CreateDirectoryItem(dirName, &pNewDirectory)))
				AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
			else {
				CTime t = CTime::GetCurrentTime();
				SYSTEMTIME st;
				t.GetAsSystemTime(st);
				COleDateTime creationTime(st);
				pNewDirectory->put_CreationTime(creationTime.m_dt);
				pNewDirectory->put_LastModifiedTime(creationTime.m_dt);
				pNewDirectory->put_LastAccessedTime(creationTime.m_dt);

				HRESULT hr = pBaseDirectory->Add(pNewDirectory);
				if(SUCCEEDED(hr)) {
					AddFoldersToTree(hParentItem, pBaseDirectory);
					controls.dataTree.Expand(controls.dataTree.GetRootItem(), TVE_EXPAND);
					controls.dataTree.Invalidate();
				} else {
					CString s;
					s.Format(IDS_ERR_ADD, dirName, hr);
					AfxMessageBox(s, MB_ICONERROR | MB_OK);
				}
			}
			pBaseDirectory->Release();
		}
	}
}

bool CCreateDataDiscDlg::AddDirectoryToImage(IFsiDirectoryItem* pBaseDirectory, CString& dirPath, VARIANT_BOOL includeBaseDirectory)
{
	ASSERT(pFsImage != NULL);

	allowCancelationOfAddingData = FALSE;
	if(!pFsImage->AddTree(pBaseDirectory, dirPath, includeBaseDirectory)) {
		AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
		return false;
	}
	return true;
}

void CCreateDataDiscDlg::AddDroppedFilesToDirectory(IFsiDirectoryItem* pBaseDirectory, HDROP hDroppedFiles)
{
	ASSERT(pFsImage != NULL);

	CArray<CString> files;
	UINT numberOfFiles = DragQueryFile(hDroppedFiles, 0xFFFFFFFF, NULL, 0);
	TCHAR pBuffer[1024];
	for(UINT i = 0; i < numberOfFiles; i++) {
		DragQueryFile(hDroppedFiles, i, pBuffer, 1024);
		files.Add(pBuffer);
	}
	if(!pFsImage->AddFiles(pBaseDirectory, files, TRUE))
		AfxMessageBox(pFsImage->GetErrorMessage(), MB_ICONERROR | MB_OK);
}

void CCreateDataDiscDlg::AddFoldersToTree(HTREEITEM hParentItem, IFsiDirectoryItem* pParentDirectory)
{
	// remove all sub-items of hParentItem
	HTREEITEM hItem = controls.dataTree.GetChildItem(hParentItem);
	while(hItem) {
		HTREEITEM h = controls.dataTree.GetNextSiblingItem(hItem);
		controls.dataTree.DeleteItem(hItem);
		hItem = h;
	}

	CComPtr<IEnumVARIANT> pEnum;
	pParentDirectory->get__NewEnum(&pEnum);
	VARIANT v;
	VariantInit(&v);
	while(pEnum->Next(1, &v, NULL) == S_OK) {
		if(v.vt == VT_DISPATCH) {
			CComQIPtr<IFsiDirectoryItem> pSubItem = v.pdispVal;
			if(pSubItem) {
				BSTR folderName;
				pSubItem->get_Name(&folderName);
				CString s = folderName;
				hItem = controls.dataTree.InsertItem(TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE, s, STOCKICON_FOLDER, STOCKICON_FOLDER_SELECTED, 0, 0, 0, hParentItem, TVI_LAST);
				SysFreeString(folderName);
				AddFoldersToTree(hItem, pSubItem);
			}
		}
		VariantClear(&v);
	}

	if(hParentItem == controls.dataTree.GetSelectedItem())
		DisplayDirectoryContentForTreeItem(hParentItem);
}

void CCreateDataDiscDlg::DisplayDirectoryContent(IFsiDirectoryItem* pDirectory)
{
	controls.dataList.DeleteAllItems();

	SHFILEINFO shfi = {0};
	CComPtr<IEnumVARIANT> pEnum;
	pDirectory->get__NewEnum(&pEnum);
	VARIANT v;
	VariantInit(&v);
	while(pEnum->Next(1, &v, NULL) == S_OK) {
		if(v.vt == VT_DISPATCH) {
			CComQIPtr<IFsiDirectoryItem> pSubFolder = v.pdispVal;
			if(pSubFolder) {
				BSTR folderName;
				pSubFolder->get_Name(&folderName);
				CString s = folderName;
				int itemIndex = controls.dataList.InsertItem(LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM, 0x7FFFFFFF, s, 0, 0, STOCKICON_FOLDER, ITEMTYPE_FOLDER);
				SysFreeString(folderName);

				controls.dataList.SetItemText(itemIndex, 1, TEXT(""));
				controls.dataList.SetItemText(itemIndex, 2, FILETYPE_FOLDER);

				COleDateTime lastModifiedTime;
				pSubFolder->get_LastModifiedTime(&lastModifiedTime.m_dt);
				lastModifiedTime.SetStatus(COleDateTime::valid);
				s = lastModifiedTime.Format();
				controls.dataList.SetItemText(itemIndex, 3, s);

				pSubFolder->get_FullPath(&folderName);
				s = folderName;
				s.Replace(TEXT("\\"), TEXT("/"));
				controls.dataList.SetItemText(itemIndex, 4, s);
				SysFreeString(folderName);
			} else {
				CComQIPtr<IFsiFileItem> pSubItem = v.pdispVal;
				if(pSubItem) {
					BSTR fileName;
					pSubItem->get_Name(&fileName);
					CString s = fileName;
					CString extension = PathFindExtension(s);
					SHGetFileInfo(extension, FILE_ATTRIBUTE_NORMAL, &shfi, sizeof(SHFILEINFO), SHGFI_SYSICONINDEX | SHGFI_USEFILEATTRIBUTES);
					int itemIndex = controls.dataList.InsertItem(LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM, 0x7FFFFFFF, s, 0, 0, shfi.iIcon, ITEMTYPE_FILE);
					SysFreeString(fileName);

					LONGLONG size = 0;
					pSubItem->get_DataSize(&size);
					TCHAR pBuffer[100];
					StrFormatByteSize(size, pBuffer, 100);
					controls.dataList.SetItemText(itemIndex, 1, pBuffer);

					SHGetFileInfo(extension, FILE_ATTRIBUTE_NORMAL, &shfi, sizeof(SHFILEINFO), SHGFI_TYPENAME | SHGFI_USEFILEATTRIBUTES);
					controls.dataList.SetItemText(itemIndex, 2, shfi.szTypeName);

					COleDateTime lastModifiedTime;
					pSubItem->get_LastModifiedTime(&lastModifiedTime.m_dt);
					lastModifiedTime.SetStatus(COleDateTime::valid);
					s = lastModifiedTime.Format();
					controls.dataList.SetItemText(itemIndex, 3, s);

					pSubItem->get_FullPath(&fileName);
					s = fileName;
					s.Replace(TEXT("\\"), TEXT("/"));
					controls.dataList.SetItemText(itemIndex, 4, s);
					SysFreeString(fileName);
				}
			}
		}
		VariantClear(&v);
	}

	controls.dataList.SortItemsEx(CompareItems, reinterpret_cast<LPARAM>(controls.dataList.m_hWnd));
}

void CCreateDataDiscDlg::DisplayDirectoryContentForTreeItem(HTREEITEM hItem)
{
	IFsiDirectoryItem* pDirectory = GetDirectoryForTreeItem(hItem);
	if(pDirectory) {
		DisplayDirectoryContent(pDirectory);
		pDirectory->Release();
	}
}

HTREEITEM CCreateDataDiscDlg::FindTreeItemFromDiscImagePath(CString& discImagePath)
{
	bool found = false;
	HTREEITEM hItem = controls.dataTree.GetRootItem();
	if(discImagePath.GetLength() > 0) {
		int p = 0;
		do {
			p = discImagePath.Find(TEXT("/"), p + 1);
			if(p == -1)
				p = discImagePath.GetLength() + 1;
			CString segment = discImagePath.Mid(1, p - 1);
			discImagePath = discImagePath.Mid(p);

			found = false;
			hItem = controls.dataTree.GetChildItem(hItem);
			while(hItem) {
				if(controls.dataTree.GetItemText(hItem) == segment) {
					found = true;
					break;
				}
				hItem = controls.dataTree.GetNextSiblingItem(hItem);
			}
		} while(discImagePath.GetLength() > 0 && found);
	} else
		found = true;

	return (found ? hItem : NULL);
}

IFsiDirectoryItem* CCreateDataDiscDlg::GetDirectoryForTreeItem(HTREEITEM hItem)
{
	ASSERT(pFsImage != NULL);

	CString discImagePath = GetDiscImagePathForTreeItem(hItem);
	CComPtr<IFsiDirectoryItem> pRootItem;
	pFsImage->GetRoot(&pRootItem);
	if(pRootItem) {
		IFsiDirectoryItem* pItem = NULL;
		CComBSTR s = discImagePath;
		pRootItem->get_Item(s, reinterpret_cast<IFsiItem**>(&pItem));
		return pItem;
	}
	return NULL;
}

CString CCreateDataDiscDlg::GetDiscImagePathForTreeItem(HTREEITEM hItem)
{
	if(!controls.dataTree.GetParentItem(hItem))
		return TEXT("/");

	CString discImagePath;
	while(controls.dataTree.GetParentItem(hItem)) {
		discImagePath = TEXT("/") + controls.dataTree.GetItemText(hItem) + discImagePath;
		hItem = controls.dataTree.GetParentItem(hItem);
	}
	return discImagePath;
}

void CCreateDataDiscDlg::RemoveTree(HTREEITEM hItem)
{
	ASSERT(pFsImage != NULL);

	CComBSTR discImagePath = GetDiscImagePathForTreeItem(hItem);
	CComPtr<IFsiDirectoryItem> pRootDirectory;
	pFsImage->GetRoot(&pRootDirectory);
	if(pRootDirectory)
		pRootDirectory->RemoveTree(discImagePath);
	controls.dataTree.DeleteItem(hItem);

	DisplayDirectoryContentForTreeItem(controls.dataTree.GetSelectedItem());
	UpdateBlocksInUse();
}