#pragma once
#include "afxcmn.h"

#include "common.h"
#include "TaskDialog.h"
#include "FileSystemImage.h"
#include "DiscFormatData.h"
#include "RecordingController.h"


// CCreateDataDiscDlg dialog

class CCreateDataDiscDlg :
    public CBurnTaskDialog
{
	DECLARE_DYNAMIC(CCreateDataDiscDlg)

public:
	CCreateDataDiscDlg(IGalbaWindow* pGalbaWindow, CWnd* pParent = NULL);   // standard constructor
	virtual ~CCreateDataDiscDlg();

	// Dialog Data
	enum { IDD = IDD_CREATEDATADISC_DIALOG };

	BOOL PreTranslateMessage(MSG* pMsg);

	virtual void CancelInterruptibleTask();
	virtual void MediaChanged();
	virtual bool BeginAddData(const CString& baseDir);
	virtual void AddData(const CString& dataType, CString& path);
	virtual void CommitAddedData(void);
	virtual DROPEFFECT OnFileDragOver(COleDataObject* /*pDataObject*/, DWORD /*keyState*/, CPoint point);
	virtual void OnDragLeave(void);
	virtual BOOL OnFileDrop(COleDataObject* pDataObject, DWORD /*keyState*/, CPoint point);

protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

	DECLARE_MESSAGE_MAP()

protected:
	static int CALLBACK CompareItems(LPARAM item1, LPARAM item2, LPARAM lParamSort);
	static int CALLBACK SelectInitialDir(HWND hWnd, UINT message, LPARAM /*lParam*/, LPARAM pData);

	void AddFileSystemsToComboBox();
	void AddSpeedSettingsToComboBox();
	void CheckForBlankMedia();
	IMAPI_MEDIA_PHYSICAL_TYPE GetCurrentMediaType(__in CDiscRecorder* pRecorder);
	bool CurrentMediaIsCD();
	bool CurrentMediaIsDVDOrLarger();
	void EnableUI(BOOL enable);
	LONGLONG GetFreeBytesOnMedia(__in CDiscRecorder* pRecorder);
	ULONG	GetMediaTypeSingleSpeed(IMAPI_MEDIA_PHYSICAL_TYPE mediaType);
	SHSTOCKICONID GetMediaTypeStockIcon(IMAPI_MEDIA_PHYSICAL_TYPE mediaType);
	LONG GetWriteSpeed();
	void ImportOldSession();
	void SetCapacity();
	void SetRootIcon();
	void UpdateBlocksInUse();

	void AddNewDirectory(__in HTREEITEM hParentItem);
	bool AddDirectoryToImage(__in IFsiDirectoryItem* pBaseDirectory, CString& dirPath, VARIANT_BOOL includeBaseDirectory);
	void AddDroppedFilesToDirectory(__in IFsiDirectoryItem* pBaseDirectory, __in HDROP hDroppedFiles);
	void AddFoldersToTree(__in HTREEITEM hParentItem, __in IFsiDirectoryItem* pParentDirectory);
	void DisplayDirectoryContent(__in IFsiDirectoryItem* pDirectory);
	void DisplayDirectoryContentForTreeItem(__in HTREEITEM hItem);
	HTREEITEM FindTreeItemFromDiscImagePath(CString& discImagePath);
	IFsiDirectoryItem* GetDirectoryForTreeItem(__in HTREEITEM hItem);
	CString GetDiscImagePathForTreeItem(__in HTREEITEM hItem);
	void RemoveTree(__in HTREEITEM hItem);

	CFileSystemImage* pFsImage;
	IProgressItems* pProgressItems;
	CRecordingController recordingController;
	CString labelEditTextToSet;
	BOOL alwaysDisableSpeedComboBox;
	BOOL allowCancelationOfAddingData;
	HANDLE hAddedDataEvent;
	CString currentBaseDir;
	CArray<CString> filesToAdd;

	CString FILETYPE_FOLDER;
	int STOCKICON_FOLDER;
	int STOCKICON_FOLDER_SELECTED;

	struct
	{
		CComboBox fileSystemCombo;
		CComboBox isoLevelCombo;
		CComboBox udfVersionCombo;
		CButton closeMediaCheck;
		CButton ejectCheck;
		CButton resetCompilationButton;
		CTreeCtrl dataTree;
		CListCtrl dataList;
		CProgressCtrl capacityProg;
		CStatic currentSize;
		CStatic discNotBlank;
		CStatic maxText;
		CButton addDataButton;
		CComboBox speedCombo;
		CButton burnButton;
		CImageList burnImageList;
	} controls;

	// Generated message map functions
	virtual BOOL OnInitDialog();
	afx_msg void OnDestroy();
	afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
	afx_msg HBRUSH OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor);
	afx_msg void OnCbnSelchangeFilesystemCombo();
	afx_msg void OnCbnSelchangeIsolevelCombo();
	afx_msg void OnCbnSelchangeUdfversionCombo();
	afx_msg void OnTvnBeginlabeleditFilesTree(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnTvnEndlabeleditFilesTree(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnTvnKeydownFilesTree(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnTvnSelchangedFilesTree(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnLvnItemActivateFilesList(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnLvnKeydownFilesList(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnBnClickedAddDataButton();
	afx_msg void OnResetCompilation();
	afx_msg void OnAddDirectory();
	afx_msg void OnAddDirectoryContent();
	afx_msg void OnAddFile();
	afx_msg void OnBurn();

	afx_msg LRESULT OnImapiBeginAddData(WPARAM /*wParam*/, LPARAM /*lParam*/);
	afx_msg LRESULT OnImapiEndAddData(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnImapiImageUpdate(WPARAM /*wParam*/, LPARAM lParam);
	afx_msg LRESULT OnImapiBeginBurn(WPARAM /*wParam*/, LPARAM /*lParam*/);
	afx_msg LRESULT OnImapiEndBurn(WPARAM wParam, LPARAM /*lParam*/);
	afx_msg LRESULT OnImapiFormatDataUpdate(WPARAM /*wParam*/, LPARAM lParam);
	afx_msg LRESULT OnEjectMedia(WPARAM /*wParam*/, LPARAM /*lParam*/);
	afx_msg LRESULT OnUpdateBlocksInUse(WPARAM /*wParam*/, LPARAM /*lParam*/);
	afx_msg LRESULT OnUpdateCompilationView(WPARAM /*wParam*/, LPARAM /*lParam*/);
};