///////////////////////////////////////////////////////////////////////
// DiscFormatDataEvent.cpp
//
// Wrapper for DDiscFormat2DataEvents interface events

#include "stdafx.h"
#include "DiscFormatDataEvent.h"
#include "DiscFormatData.h"


// CDiscFormatDataEvent

IMPLEMENT_DYNAMIC(CDiscFormatDataEvent, CCmdTarget)

BEGIN_INTERFACE_MAP(CDiscFormatDataEvent, CCmdTarget)
	INTERFACE_PART(CDiscFormatDataEvent, IID_IDispatch, FormatDataEvents)
	INTERFACE_PART(CDiscFormatDataEvent, IID_DDiscFormat2DataEvents, FormatDataEvents)
END_INTERFACE_MAP()

CDiscFormatDataEvent::CDiscFormatDataEvent() :
    hNotificationWnd(NULL),
    pTypeInfo(NULL),
    cookie(0),
    pUnknownSink(NULL),
    pUnknownSource(NULL)
{
}

CDiscFormatDataEvent::~CDiscFormatDataEvent()
{
	if(cookie && pUnknownSource && pUnknownSink)
		AfxConnectionUnadvise(pUnknownSource, IID_DDiscFormat2DataEvents, pUnknownSink, TRUE, cookie);
}


BEGIN_MESSAGE_MAP(CDiscFormatDataEvent, CCmdTarget)
END_MESSAGE_MAP()


ULONG FAR EXPORT CDiscFormatDataEvent::XFormatDataEvents::AddRef()
{
	METHOD_PROLOGUE(CDiscFormatDataEvent, FormatDataEvents)
	return pThis->ExternalAddRef();
}

ULONG FAR EXPORT CDiscFormatDataEvent::XFormatDataEvents::Release()
{
	METHOD_PROLOGUE(CDiscFormatDataEvent, FormatDataEvents)
	return pThis->ExternalRelease();
}

STDMETHODIMP CDiscFormatDataEvent::XFormatDataEvents::QueryInterface(REFIID riid, LPVOID FAR* ppvObj)
{
	METHOD_PROLOGUE(CDiscFormatDataEvent, FormatDataEvents)
	return pThis->ExternalQueryInterface(&riid, ppvObj);
}

STDMETHODIMP CDiscFormatDataEvent::XFormatDataEvents::GetTypeInfoCount(UINT FAR* pctinfo)
{
	METHOD_PROLOGUE(CDiscFormatDataEvent, FormatDataEvents)
	*pctinfo = 1;
	return NOERROR;
}

STDMETHODIMP CDiscFormatDataEvent::XFormatDataEvents::GetTypeInfo(UINT itinfo, LCID /*lcid*/, ITypeInfo FAR* FAR* pptinfo)
{
	METHOD_PROLOGUE(CDiscFormatDataEvent, FormatDataEvents)
	*pptinfo = NULL;

	if(itinfo != 0)
		return ResultFromScode(DISP_E_BADINDEX);
	pThis->pTypeInfo->AddRef();
	*pptinfo = pThis->pTypeInfo;
	return NOERROR;
}

STDMETHODIMP CDiscFormatDataEvent::XFormatDataEvents::GetIDsOfNames(REFIID /*riid*/, OLECHAR FAR* FAR* rgszNames, UINT cNames, LCID /*lcid*/, DISPID FAR* rgdispid)
{
	METHOD_PROLOGUE(CDiscFormatDataEvent, FormatDataEvents)
	return DispGetIDsOfNames(pThis->pTypeInfo, rgszNames, cNames, rgdispid);
}

STDMETHODIMP CDiscFormatDataEvent::XFormatDataEvents::Invoke(DISPID dispidMember, REFIID /*riid*/, LCID /*lcid*/, WORD wFlags, DISPPARAMS FAR* pdispparams, VARIANT FAR* pvarResult, EXCEPINFO FAR* pexcepinfo, UINT FAR* puArgErr)
{
	METHOD_PROLOGUE(CDiscFormatDataEvent, FormatDataEvents)
	return DispInvoke(&pThis->m_xFormatDataEvents, pThis->pTypeInfo, dispidMember, wFlags, pdispparams, pvarResult, pexcepinfo, puArgErr);
}


CDiscFormatDataEvent* CDiscFormatDataEvent::CreateEventSink()
{
	CDiscFormatDataEvent* pDiscFormatDataEvent = new CDiscFormatDataEvent();
	pDiscFormatDataEvent->EnableAutomation();
	pDiscFormatDataEvent->ExternalAddRef();
	return pDiscFormatDataEvent;
}

bool CDiscFormatDataEvent::ConnectDiscFormatData(CDiscFormatData* pDiscFormatData)
{
	pUnknownSink = GetIDispatch(FALSE);
	pUnknownSource = pDiscFormatData->GetInterface();

	LPTYPELIB pTypeLib = NULL;
	HRESULT hr = LoadRegTypeLib(LIBID_IMAPILib2, IMAPILib2_MajorVersion, IMAPILib2_MinorVersion, LOCALE_SYSTEM_DEFAULT, &pTypeLib);
	if(FAILED(hr))
		return false;
	hr = pTypeLib->GetTypeInfoOfGuid(IID_DDiscFormat2DataEvents, &pTypeInfo);
	pTypeLib->Release();
	if(FAILED(hr))
		return false;

	BOOL b = AfxConnectionAdvise(pUnknownSource, IID_DDiscFormat2DataEvents, pUnknownSink, TRUE, &cookie);
	if(b)
		return true;
	return false;
}


STDMETHODIMP_(HRESULT) CDiscFormatDataEvent::XFormatDataEvents::Update(LPDISPATCH pObjectDispatch, LPDISPATCH pProgressDispatch)
{
	METHOD_PROLOGUE(CDiscFormatDataEvent, FormatDataEvents)

	CComQIPtr<IDiscFormat2DataEventArgs> pProgress = pProgressDispatch;

	IMAPI_FORMAT_DATA_UPDATE_EVENT_ARGS eventArgs;
	ZeroMemory(&eventArgs, sizeof(eventArgs));
	eventArgs.currentAction = IMAPI_FORMAT2_DATA_WRITE_ACTION_VALIDATING_MEDIA;

	pProgress->get_CurrentAction(&eventArgs.currentAction);
	if(eventArgs.currentAction == IMAPI_FORMAT2_DATA_WRITE_ACTION_WRITING_DATA || eventArgs.currentAction == IMAPI_FORMAT2_DATA_WRITE_ACTION_FINALIZATION) {
		pProgress->get_ElapsedTime(&eventArgs.elapsedTime);
		pProgress->get_RemainingTime(&eventArgs.remainingTime);
		pProgress->get_TotalTime(&eventArgs.totalTime);
		if(eventArgs.currentAction == IMAPI_FORMAT2_DATA_WRITE_ACTION_WRITING_DATA) {
			pProgress->get_StartLba(&eventArgs.startLba);
			pProgress->get_SectorCount(&eventArgs.sectorCount);
			pProgress->get_LastReadLba(&eventArgs.lastReadLba);
			pProgress->get_LastWrittenLba(&eventArgs.lastWrittenLba);
			pProgress->get_TotalSystemBuffer(&eventArgs.totalSystemBuffer);
			pProgress->get_UsedSystemBuffer(&eventArgs.usedSystemBuffer);
			pProgress->get_FreeSystemBuffer(&eventArgs.freeSystemBuffer);
		}
	}

	LRESULT ret = SendMessage(pThis->hNotificationWnd, WM_IMAPI_FORMAT_DATA_UPDATE, 0, reinterpret_cast<LPARAM>(&eventArgs));
	if(ret == RETURN_CANCEL_WRITE) {
		CComQIPtr<IDiscFormat2Data> pDF2Data = pObjectDispatch;
		pDF2Data->CancelWrite();
	}
	return S_OK;
}