///////////////////////////////////////////////////////////////////////
// DiscFormatErase.cpp
//
// Wrapper for IDiscFormat2Erase interface

#include "stdafx.h"
#include "res/resource.h"
#include "DiscFormatErase.h"


CDiscFormatErase::CDiscFormatErase(void) :
    pDF2Erase(NULL),
    lastError(S_OK),
    hNotificationWnd(NULL),
    pSupportedMediaTypes(NULL)
{
}

CDiscFormatErase::~CDiscFormatErase(void)
{
	if(pSupportedMediaTypes)
		SafeArrayDestroy(pSupportedMediaTypes);
	if(pDF2Erase)
		pDF2Erase->Release();
}

bool CDiscFormatErase::Initialize(CDiscRecorder* pDiscRecorder, const CString& clientName, HWND hNotificationWnd)
{
	ASSERT(pDF2Erase == NULL);
	this->hNotificationWnd = hNotificationWnd;

	CString lastErrorMessage;
	HRESULT lastError = CoCreateInstance(__uuidof(MsftDiscFormat2Erase), NULL, CLSCTX_INPROC_SERVER, IID_PPV_ARGS(&pDF2Erase));
	if(FAILED(lastError)) {
		lastErrorMessage.Format(IDS_ERR_INIT_DISCFORMAT2ERASE, lastError);
		SetLastError(lastError, lastErrorMessage);
		return false;
	}

	VARIANT_BOOL isSupported = VARIANT_FALSE;
	lastError = pDF2Erase->IsRecorderSupported(pDiscRecorder->GetInterface(), &isSupported);
	if(FAILED(lastError)) {
		lastErrorMessage.Format(IDS_ERR_ISRECORDERSUPPORTED_ERASE, lastError);
		SetLastError(lastError, lastErrorMessage);
		return false;
	}
	if(isSupported == VARIANT_FALSE) {
		lastErrorMessage.LoadString(IDS_ERR_RECORDER_NOT_SUPPORTED);
		SetLastError(lastError, lastErrorMessage);
		return false;
	}
	lastError = pDF2Erase->put_Recorder(pDiscRecorder->GetInterface());
	if(FAILED(lastError)) {
		lastErrorMessage.Format(IDS_ERR_PUTRECORDER_ERASE, lastError);
		SetLastError(lastError, lastErrorMessage);
		return false;
	}
	CComBSTR bstr = clientName;
	lastError = pDF2Erase->put_ClientName(bstr);
	if(FAILED(lastError)) {
		lastErrorMessage.Format(IDS_ERR_PUTCLIENTNAME_ERASE, lastError);
		SetLastError(lastError, lastErrorMessage);
		return false;
	}

	SetLastError(lastError);
	return true;
}

void CDiscFormatErase::SetLastError(HRESULT lastError)
{
	this->lastError = lastError;
}

void CDiscFormatErase::SetLastError(HRESULT lastError, CString lastErrorMessage)
{
	this->lastError = lastError;
	this->lastErrorMessage = lastErrorMessage;
}

bool CDiscFormatErase::IsCurrentMediaSupported(CDiscRecorder* pDiscRecorder)
{
	ASSERT(pDF2Erase != NULL);
	CString lastErrorMessage;

	VARIANT_BOOL b = VARIANT_FALSE;
	HRESULT lastError = pDF2Erase->IsCurrentMediaSupported(pDiscRecorder->GetInterface(), &b);
	if(FAILED(lastError)) {
		lastErrorMessage.Format(IDS_ERR_ISMEDIASUPPORTED_ERASE, lastError);
		SetLastError(lastError, lastErrorMessage);
	} else
		SetLastError(lastError);

	return (b != VARIANT_FALSE);
}

BOOL CDiscFormatErase::IsMediaBlank()
{
	ASSERT(pDF2Erase != NULL);

	VARIANT_BOOL b = VARIANT_FALSE;
	HRESULT lastError = pDF2Erase->get_MediaHeuristicallyBlank(&b);
	if(FAILED(lastError)) {
		lastErrorMessage.Format(IDS_ERR_GETMEDIAHEURISTICALLYBLANK_ERASE, lastError);
		SetLastError(lastError, lastErrorMessage);
	} else
		SetLastError(lastError);

	return (b != VARIANT_FALSE);
}

ULONG CDiscFormatErase::GetTotalSupportedMediaTypes()
{
	ASSERT(pDF2Erase != NULL);
	CString lastErrorMessage;

	if(pSupportedMediaTypes) {
		SafeArrayDestroy(pSupportedMediaTypes);
		pSupportedMediaTypes = NULL;
	}

	HRESULT lastError = pDF2Erase->get_SupportedMediaTypes(&pSupportedMediaTypes);
	if(FAILED(lastError)) {
		lastErrorMessage.Format(IDS_ERR_GETSUPPORTEDMEDIATYPES_ERASE, lastError);
		SetLastError(lastError, lastErrorMessage);
		return 0;
	}
	SetLastError(lastError);
	return pSupportedMediaTypes->rgsabound[0].cElements;
}

IMAPI_MEDIA_PHYSICAL_TYPE CDiscFormatErase::GetSupportedMediaType(LONG index)
{
	if(!pSupportedMediaTypes)
		GetTotalSupportedMediaTypes();
	ASSERT(pSupportedMediaTypes != NULL);

	IMAPI_MEDIA_PHYSICAL_TYPE mediaType = IMAPI_MEDIA_TYPE_UNKNOWN;
	if(pSupportedMediaTypes) {
		VARIANT v;
		SafeArrayGetElement(pSupportedMediaTypes, &index, reinterpret_cast<LPVOID>(&v));
		ASSERT(v.vt == VT_I4);
		mediaType = static_cast<IMAPI_MEDIA_PHYSICAL_TYPE>(v.lVal);
		VariantClear(&v);
	}
	return mediaType;
}

HRESULT CDiscFormatErase::SetEraseMethod(VARIANT_BOOL fullErase)
{
	ASSERT(pDF2Erase != NULL);
	HRESULT lastError = pDF2Erase->put_FullErase(fullErase);
	if(FAILED(lastError)) {
		lastErrorMessage.Format(IDS_ERR_PUTFULLERASE, lastError);
		SetLastError(lastError, lastErrorMessage);
	} else
		SetLastError(lastError);
	return lastError;
}