///////////////////////////////////////////////////////////////////////
// DiscMaster.cpp
//
// Wrapper for IDiscMaster2 interface

#include "stdafx.h"
#include "res/resource.h"
#include "DiscMaster.h"


CDiscMaster::CDiscMaster(void) :
    pDiscMaster(NULL),
    lastError(S_OK)
{
}

CDiscMaster::~CDiscMaster(void)
{
	if(pDiscMaster)
		pDiscMaster->Release();
}

bool CDiscMaster::Initialize()
{
	ASSERT(pDiscMaster == NULL);

	CString lastErrorMessage;
	HRESULT lastError = E_FAIL;
	if(!pDiscMaster) {
		lastError = CoCreateInstance(__uuidof(MsftDiscMaster2), NULL, CLSCTX_INPROC_SERVER, IID_PPV_ARGS(&pDiscMaster));
		if(FAILED(lastError)) {
			lastErrorMessage.Format(IDS_ERR_INIT_DISCMASTER2, lastError);
			SetLastError(lastError, lastErrorMessage);
			return false;
		}
	}

	VARIANT_BOOL isSupported = VARIANT_FALSE;
	lastError = pDiscMaster->get_IsSupportedEnvironment(&isSupported);
	if(FAILED(lastError)) {
		lastErrorMessage.Format(IDS_ERR_GETISSUPPORTEDENVIRONMENT, lastError);
		SetLastError(lastError, lastErrorMessage);
		return false;
	}
	if(isSupported == VARIANT_FALSE) {
		lastErrorMessage.Format(IDS_ERR_NO_WRITABLE_DEVICE, lastError);
		SetLastError(lastError, lastErrorMessage);
		return false;
	}
	SetLastError(lastError);
	return true;
}

void CDiscMaster::SetLastError(HRESULT lastError)
{
	this->lastError = lastError;
}

void CDiscMaster::SetLastError(HRESULT lastError, CString lastErrorMessage)
{
	this->lastError = lastError;
	this->lastErrorMessage = lastErrorMessage;
}

LONG CDiscMaster::GetTotalDevices()
{
	ASSERT(pDiscMaster != NULL);
	CString lastErrorMessage;

	LONG c = 0;
	HRESULT lastError = pDiscMaster->get_Count(&c);
	if(FAILED(lastError)) {
		lastErrorMessage.Format(IDS_ERR_GETDEVICECOUNT, lastError);
		SetLastError(lastError, lastErrorMessage);
	} else
		SetLastError(lastError);
	return c;
}

CString CDiscMaster::GetDeviceUniqueID(LONG index)
{
	ASSERT(pDiscMaster != NULL);
	CString lastErrorMessage;

	BSTR uniqueID = NULL;
	HRESULT lastError = pDiscMaster->get_Item(index, &uniqueID);
	if(FAILED(lastError)) {
		lastErrorMessage.Format(IDS_ERR_GETDEVICEITEM, index, lastError);
		SetLastError(lastError, lastErrorMessage);
		return TEXT("");
	} else
		SetLastError(lastError);

	CString s = uniqueID;
	SysFreeString(uniqueID);
	return s;
}