#include "stdafx.h"
#include "APIWrapper.h"
#include "Galba.h"
#include "EraseDiscDlg.h"

#include "DiscRecorder.h"


// CEraseDiscDlg dialog

IMPLEMENT_DYNAMIC(CEraseDiscDlg, CBurnTaskDialog)

CEraseDiscDlg::CEraseDiscDlg(IGalbaWindow* pGalbaWindow, CWnd* pParent/* = NULL*/) :
    CBurnTaskDialog(pGalbaWindow, CEraseDiscDlg::IDD, pParent)
{
}

CEraseDiscDlg::~CEraseDiscDlg()
{
}

void CEraseDiscDlg::DoDataExchange(CDataExchange* pDX)
{
	CBurnTaskDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_ERASEMETHOD_COMBO, controls.eraseMethodCombo);
	DDX_Control(pDX, IDC_EJECT_WHEN_FINISHED_CHK, controls.ejectCheck);
	DDX_Control(pDX, IDC_CAPACITY, controls.capacityProg);
	DDX_Control(pDX, IDC_NO_MEDIA, controls.noMedia);
	DDX_Control(pDX, IDC_MAX_TEXT, controls.maxText);
	DDX_Control(pDX, ID_ERASE, controls.eraseButton);
}


BEGIN_MESSAGE_MAP(CEraseDiscDlg, CBurnTaskDialog)
	ON_WM_DESTROY()
	ON_WM_CTLCOLOR()
	ON_COMMAND(ID_ERASE, &CEraseDiscDlg::OnErase)

	ON_MESSAGE(WM_IMAPI_BEGIN_ERASE, &CEraseDiscDlg::OnImapiBeginErase)
	ON_MESSAGE(WM_IMAPI_END_ERASE, &CEraseDiscDlg::OnImapiEndErase)
	ON_MESSAGE(WM_IMAPI_FORMAT_ERASE_UPDATE, &CEraseDiscDlg::OnImapiFormatEraseUpdate)
END_MESSAGE_MAP()


BOOL CEraseDiscDlg::PreTranslateMessage(MSG* pMsg)
{
	return CBurnTaskDialog::PreTranslateMessage(pMsg);
}


void CEraseDiscDlg::CancelInterruptibleTask()
{
	if(runningInterruptibleTask) {
		erasingController.SetCancelFlag(true);
		if(pGalbaWindow)
			pGalbaWindow->EnableCancelButton(FALSE);
	}
}

void CEraseDiscDlg::MediaChanged()
{
	if(initialized) {
		SetCapacity();
		CheckForMedia();
		UpdateBlocksInUse();
	}
}


// CEraseDiscDlg message handlers

BOOL CEraseDiscDlg::OnInitDialog()
{
	CBurnTaskDialog::OnInitDialog();

	CBitmap bmp;
	bmp.LoadBitmap(IDB_ERASE);
	controls.eraseImageList.Create(16, 16, ILC_COLOR32 | ILC_MASK, 6, 0);
	controls.eraseImageList.Add(&bmp, reinterpret_cast<CBitmap*>(NULL));
	BUTTON_IMAGELIST iml = {0};
	iml.himl = controls.eraseImageList;
	iml.uAlign = BUTTON_IMAGELIST_ALIGN_LEFT;
	iml.margin.left = 5;
	iml.margin.right = 2;
	controls.eraseButton.SetImageList(&iml);

	controls.capacityProg.SetRange(0, 100);
	CString s;
	s.LoadString(IDS_ERASE_OPTION_FAST);
	int itemIndex = controls.eraseMethodCombo.AddString(s);
	controls.eraseMethodCombo.SetItemData(itemIndex, static_cast<DWORD_PTR>(VARIANT_FALSE));
	s.LoadString(IDS_ERASE_OPTION_FULL);
	itemIndex = controls.eraseMethodCombo.AddString(s);
	controls.eraseMethodCombo.SetItemData(itemIndex, static_cast<DWORD_PTR>(VARIANT_TRUE));

	int i = theApp.settings.GetSetting(CString("EraseDisc\\EraseMethod"));
	controls.eraseMethodCombo.SetCurSel(i);
	VARIANT_BOOL b = theApp.settings.GetSetting(CString("EraseDisc\\EjectMedia"));
	controls.ejectCheck.SetCheck(b == VARIANT_FALSE ? BST_UNCHECKED : BST_CHECKED);

	initialized = true;
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CEraseDiscDlg::OnDestroy()
{
	int i = controls.eraseMethodCombo.GetCurSel();
	theApp.settings.SetSetting(CString("EraseDisc\\EraseMethod"), i);
	VARIANT_BOOL b = (controls.ejectCheck.GetCheck() == BST_CHECKED ? VARIANT_TRUE : VARIANT_FALSE);
	theApp.settings.SetSetting(CString("EraseDisc\\EjectMedia"), b);
}

HBRUSH CEraseDiscDlg::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
	HBRUSH hBrush = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);
	if(nCtlColor == CTLCOLOR_STATIC && pWnd->GetDlgCtrlID() == IDC_NO_MEDIA) {
		pDC->SetTextColor(RGB(255, 0, 0));
		pDC->SetBkMode(TRANSPARENT);
	}
	return hBrush;
}

void CEraseDiscDlg::OnErase()
{
	if(!pGalbaWindow)
		return;

	CDiscRecorder* pRecorder = NULL;
	pGalbaWindow->GetRecorder(&pRecorder);
	if(!pRecorder)
		return;

	CString recorderID = pRecorder->GetUniqueId();
	erasingController.SetRecorderID(recorderID);
	erasingController.SetClientName(CLIENT_NAME);
	erasingController.SetNotificationWindow(*this);
	int selection = controls.eraseMethodCombo.GetCurSel();
	if(selection >= 0)
		erasingController.SetEraseMethod(static_cast<VARIANT_BOOL>(controls.eraseMethodCombo.GetItemData(selection)));
	erasingController.SetEjectMedia(controls.ejectCheck.GetCheck() == BST_CHECKED);

	if(!erasingController.Erase())
		AfxMessageBox(erasingController.GetErrorMessage(), MB_ICONERROR | MB_OK);
}


LRESULT CEraseDiscDlg::OnImapiBeginErase(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	runningInterruptibleTask = true;

	EnableUI(FALSE);
	if(pGalbaWindow) {
		pGalbaWindow->EnableUI(FALSE);
		pGalbaWindow->EnableCancelButton(FALSE);
	}
	return 0;
}

LRESULT CEraseDiscDlg::OnImapiEndErase(WPARAM wParam, LPARAM /*lParam*/)
{
	runningInterruptibleTask = false;
	if(pGalbaWindow)
		pGalbaWindow->SetTimes(-1, -1);

	CString message;
	switch(wParam) {
		case S_OK:
			// success;
			message.LoadString(IDS_ERASING_COMPLETE);
			AfxMessageBox(message, MB_ICONINFORMATION | MB_OK);
			break;
		default:
			// error
			AfxMessageBox(erasingController.GetErrorMessage(), MB_ICONERROR | MB_OK);
			break;
	}

	EnableUI(TRUE);
	if(pGalbaWindow) {
		CString s;
		pGalbaWindow->SetProgress(0, s);
		pGalbaWindow->EnableCancelButton(FALSE);
		pGalbaWindow->DeviceChange();
		pGalbaWindow->EnableUI(TRUE);
	}
	return 0;
}

LRESULT CEraseDiscDlg::OnImapiFormatEraseUpdate(WPARAM /*wParam*/, LPARAM lParam)
{
	if(pGalbaWindow) {
		PIMAPI_FORMAT_ERASE_UPDATE_EVENT_ARGS pDetails = reinterpret_cast<PIMAPI_FORMAT_ERASE_UPDATE_EVENT_ARGS>(lParam);
		CString progressText;
		progressText.LoadString(IDS_STATUS_ERASING);
		pGalbaWindow->SetTimes(pDetails->elapsedTime, pDetails->totalTime - pDetails->elapsedTime);
		int percentage = 0;
		if(pDetails->totalTime > 0)
			percentage = pDetails->elapsedTime * 100 / pDetails->totalTime;
		pGalbaWindow->SetProgress(percentage, progressText);
	}
	//return (erasingController.GetCancelFlag() ? RETURN_CANCEL_WRITE : RETURN_CONTINUE_WRITE);
	return 0;
}


void CEraseDiscDlg::CheckForMedia()
{
	if(!pGalbaWindow)
		return;

	BOOL imageRecorderSelected = FALSE;
	pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	EnableUI(!imageRecorderSelected);
	if(imageRecorderSelected) {
		controls.noMedia.ShowWindow(SW_HIDE);
		return;
	}

	CDiscRecorder* pRecorder = NULL;
	pGalbaWindow->GetRecorder(&pRecorder);
	if(!pRecorder)
		return;
	CDiscFormatData discFormatData;
	if(!discFormatData.Initialize(pRecorder, CLIENT_NAME, *this)) {
		AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
		return;
	}

	IMAPI_MEDIA_PHYSICAL_TYPE mediaType = discFormatData.GetCurrentPhysicalMediaType();
	HRESULT lastError = discFormatData.GetHresult();
	// Some devices, e.g. from Samsung fail with E_IMAPI_RECORDER_INVALID_RESPONSE_FROM_DEVICE if there is no media.
	BOOL noMedia = (lastError == E_IMAPI_RECORDER_MEDIA_NO_MEDIA || lastError == E_IMAPI_RECORDER_INVALID_RESPONSE_FROM_DEVICE);
	if(mediaType == IMAPI_MEDIA_TYPE_UNKNOWN && !noMedia)
		AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
	EnableUI(!noMedia);
	controls.noMedia.ShowWindow(noMedia ? SW_SHOW : SW_HIDE);
}

IMAPI_MEDIA_PHYSICAL_TYPE CEraseDiscDlg::GetCurrentMediaType(CDiscRecorder* pRecorder)
{
	CDiscFormatData discFormatData;
	if(!discFormatData.Initialize(pRecorder, CLIENT_NAME, *this)) {
		AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
		return IMAPI_MEDIA_TYPE_UNKNOWN;
	}

	IMAPI_MEDIA_PHYSICAL_TYPE mediaType = discFormatData.GetCurrentPhysicalMediaType();
	HRESULT lastError = discFormatData.GetHresult();
	// Some devices, e.g. from Samsung fail with E_IMAPI_RECORDER_INVALID_RESPONSE_FROM_DEVICE if there is no media.
	if(mediaType == IMAPI_MEDIA_TYPE_UNKNOWN && lastError != E_IMAPI_RECORDER_MEDIA_NO_MEDIA && lastError != E_IMAPI_RECORDER_INVALID_RESPONSE_FROM_DEVICE)
		AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
	return mediaType;
}

void CEraseDiscDlg::EnableUI(BOOL enable)
{
	controls.eraseMethodCombo.EnableWindow(enable);
	controls.eraseButton.EnableWindow(enable);

	BOOL imageRecorderSelected = FALSE;
	if(pGalbaWindow)
		pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	controls.ejectCheck.EnableWindow(!imageRecorderSelected && enable);
}

LONGLONG CEraseDiscDlg::GetTotalBytesOnMedia(CDiscRecorder* pRecorder)
{
	BOOL imageRecorderSelected = FALSE;
	pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	if(imageRecorderSelected)
		return 0;

	CDiscFormatData discFormatData;
	if(!discFormatData.Initialize(pRecorder, CLIENT_NAME, *this)) {
		AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
		return 0;
	}

	LONG capacity = discFormatData.GetTotalSectorsOnMedia();
	if(capacity == -1) {
		if(discFormatData.GetHresult() == E_IMAPI_DF2DATA_INVALID_MEDIA_STATE || discFormatData.GetHresult() == E_IMAPI_BURN_VERIFICATION_FAILED) {
			CString drive;
			if(SUCCEEDED(pGalbaWindow->GetRecorderPath(drive))) {
				ULARGE_INTEGER totalNumberOfBytes;
				if(GetDiskFreeSpaceEx(drive, NULL, &totalNumberOfBytes, NULL))
					return totalNumberOfBytes.QuadPart;
			}
		} else {
			HRESULT lastError = discFormatData.GetHresult();
			// Some devices, e.g. from Samsung fail with E_IMAPI_RECORDER_INVALID_RESPONSE_FROM_DEVICE if there is no media.
			if(lastError != E_IMAPI_RECORDER_MEDIA_NO_MEDIA && lastError != E_IMAPI_RECORDER_INVALID_RESPONSE_FROM_DEVICE)
				AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
		}
		return 0;
	}
	LONGLONG ll = capacity;
	ll *= IMAPI_SECTOR_SIZE;
	return ll;
}

void CEraseDiscDlg::SetCapacity()
{
	if(!pGalbaWindow)
		return;

	LONGLONG capacity = 0;

	BOOL imageRecorderSelected = FALSE;
	pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	if(!imageRecorderSelected) {
		CDiscRecorder* pRecorder = NULL;
		pGalbaWindow->GetRecorder(&pRecorder);
		if(!pRecorder)
			return;
		capacity = GetTotalBytesOnMedia(pRecorder);
	}

	TCHAR pBuffer[100];
	StrFormatByteSize(capacity, pBuffer, 100);
	controls.maxText.SetWindowText(pBuffer);
}

void CEraseDiscDlg::UpdateBlocksInUse()
{
	if(!pGalbaWindow)
		return;

	BOOL imageRecorderSelected = FALSE;
	pGalbaWindow->IsImageRecorderSelected(imageRecorderSelected);
	if(imageRecorderSelected) {
		controls.capacityProg.SetPos(0);
		return;
	}

	CDiscRecorder* pRecorder = NULL;
	pGalbaWindow->GetRecorder(&pRecorder);
	if(!pRecorder)
		return;

	LONGLONG capacity = GetTotalBytesOnMedia(pRecorder);

	CString drive;
	if(SUCCEEDED(pGalbaWindow->GetRecorderPath(drive))) {
		ULARGE_INTEGER totalNumberOfBytes;
		if(GetDiskFreeSpaceEx(drive, NULL, &totalNumberOfBytes, NULL)) {
			int percentage = 0;
			if(capacity > 0)
				percentage = static_cast<int>(totalNumberOfBytes.QuadPart * 100 / capacity);
			controls.capacityProg.SetPos(percentage);
		}
	}
}