///////////////////////////////////////////////////////////////////////
// FileSystemImage.h
//
// Wrapper for IFileSystemImage Interface
#pragma once

#include "common.h"


class CFileSystemImage
{
public:
	CFileSystemImage(void);
	~CFileSystemImage(void);

	inline HRESULT GetHresult()
	{
		CSingleLock singleLock(&criticalSection);
		return lastError;
	}

	inline CString GetErrorMessage()
	{
		CSingleLock singleLock(&criticalSection);
		return lastErrorMessage;
	}

	inline IFileSystemImage* GetInterface()
	{
		return pFSImage;
	}

	bool Initialize(HWND hNotificationWnd);
	bool GetCancelFlag();
	void SetCancelFlag(bool cancel);
	void SetLastError(HRESULT lastError);
	void SetLastError(HRESULT lastError, CString lastErrorMessage);

	CString GetVolumeName();
	bool SetVolumeName(CString& volumeName);
	FsiFileSystems GetSupportedFileSystems();
	bool SetFileSystemsToCreate(FsiFileSystems fileSystem);
	ULONG GetTotalSupportedISOLevels();
	LONG GetSupportedISOLevel(LONG index);
	bool SetISOLevel(LONG level);
	ULONG GetTotalSupportedUDFRevisions();
	LONG GetSupportedUDFRevision(LONG index);
	bool SetUDFRevision(LONG revision);
	LONG GetBlocksInUse();
	HRESULT GetRoot(__deref_out IFsiDirectoryItem** ppRootDirectory);
	HRESULT CreateDirectoryItem(CString itemName, __deref_out IFsiDirectoryItem** ppNewItem);
	__checkReturn LPSTREAM GetStream(__deref_out_opt IFileSystemImageResult** ppImageResult);
	LONG GetTotalDirectories();
	LONG GetTotalFiles();

	HRESULT SetMultisessionInterfaces(__in LPSAFEARRAY pMultiSession);
	HRESULT ImportDefaultSession();
	// The imported volume name is provided for user information and is not automatically carried forward to subsequent sessions.
	//CString GetImportedVolumeName();

	bool AddFile(__in IFsiDirectoryItem* pBaseDirectory, CString& file);
	bool AddFiles(__in IFsiDirectoryItem* pBaseDirectory, CArray<CString>& files, BOOL mixedData);
	bool AddTree(__in IFsiDirectoryItem* pBaseDirectory, CString& dirPath, VARIANT_BOOL includeBaseDirectory);
	static UINT __cdecl AddFilesThread(LPVOID pParam);
	static UINT __cdecl AddTreeThread(LPVOID pParam);

protected:
	CWinThread* pWorkerThread;
	CCriticalSection criticalSection;
	IFileSystemImage*	pFSImage;
	HRESULT lastError;
	CString lastErrorMessage;
	HWND hNotificationWnd;
	bool cancel;

private:
	SAFEARRAY* pSupportedISOLevels;
	SAFEARRAY* pSupportedUDFRevisions;
};

typedef struct THREAD_PARAMS_ADDFILES
{
	CFileSystemImage* pThis;
	CArray<CString> files;
	IFsiDirectoryItem* pBaseDirectory;
	BOOL mixedData;
} THREAD_PARAMS_ADDFILES, *PTHREAD_PARAMS_ADDFILES;

typedef struct THREAD_PARAMS_ADDTREE
{
	CFileSystemImage* pThis;
	CString dirPath;
	IFsiDirectoryItem* pBaseDirectory;
	VARIANT_BOOL includeBaseDirectory;
} THREAD_PARAMS_ADDTREE, *PTHREAD_PARAMS_ADDTREE;