///////////////////////////////////////////////////////////////////////
// FileSystemImageEvent.cpp
//
// Wrapper for DFileSystemImageEvents interface events

#include "stdafx.h"
#include "FileSystemImageEvent.h"
#include "FileSystemImage.h"


// CFileSystemImageEvent

IMPLEMENT_DYNAMIC(CFileSystemImageEvent, CCmdTarget)

BEGIN_INTERFACE_MAP(CFileSystemImageEvent, CCmdTarget)
	INTERFACE_PART(CFileSystemImageEvent, IID_IDispatch, FileSystemImageEvents)
	INTERFACE_PART(CFileSystemImageEvent, IID_DFileSystemImageEvents, FileSystemImageEvents)
END_INTERFACE_MAP()

CFileSystemImageEvent::CFileSystemImageEvent() :
    hNotificationWnd(NULL),
    pTypeInfo(NULL),
    cookie(0),
    pUnknownSink(NULL),
    pUnknownSource(NULL)
{
}

CFileSystemImageEvent::~CFileSystemImageEvent()
{
	if(cookie && pUnknownSource && pUnknownSink)
		AfxConnectionUnadvise(pUnknownSource, IID_DFileSystemImageEvents, pUnknownSink, TRUE, cookie);
}


BEGIN_MESSAGE_MAP(CFileSystemImageEvent, CCmdTarget)
END_MESSAGE_MAP()


ULONG FAR EXPORT CFileSystemImageEvent::XFileSystemImageEvents::AddRef()
{
	METHOD_PROLOGUE(CFileSystemImageEvent, FileSystemImageEvents)
	return pThis->ExternalAddRef();
}

ULONG FAR EXPORT CFileSystemImageEvent::XFileSystemImageEvents::Release()
{
	METHOD_PROLOGUE(CFileSystemImageEvent, FileSystemImageEvents)
	return pThis->ExternalRelease();
}

STDMETHODIMP CFileSystemImageEvent::XFileSystemImageEvents::QueryInterface(REFIID riid, LPVOID FAR* ppvObj)
{
	METHOD_PROLOGUE(CFileSystemImageEvent, FileSystemImageEvents)
	return pThis->ExternalQueryInterface(&riid, ppvObj);
}

STDMETHODIMP CFileSystemImageEvent::XFileSystemImageEvents::GetTypeInfoCount(UINT FAR* pctinfo)
{
	METHOD_PROLOGUE(CFileSystemImageEvent, FileSystemImageEvents)
	*pctinfo = 1;
	return NOERROR;
}

STDMETHODIMP CFileSystemImageEvent::XFileSystemImageEvents::GetTypeInfo(UINT itinfo, LCID /*lcid*/, ITypeInfo FAR* FAR* pptinfo)
{
	METHOD_PROLOGUE(CFileSystemImageEvent, FileSystemImageEvents)
	*pptinfo = NULL;

	if(itinfo != 0)
		return ResultFromScode(DISP_E_BADINDEX);
	pThis->pTypeInfo->AddRef();
	*pptinfo = pThis->pTypeInfo;
	return NOERROR;
}

STDMETHODIMP CFileSystemImageEvent::XFileSystemImageEvents::GetIDsOfNames(REFIID /*riid*/, OLECHAR FAR* FAR* rgszNames, UINT cNames, LCID /*lcid*/, DISPID FAR* rgdispid)
{
	METHOD_PROLOGUE(CFileSystemImageEvent, FileSystemImageEvents)
	return DispGetIDsOfNames(pThis->pTypeInfo, rgszNames, cNames, rgdispid);
}

STDMETHODIMP CFileSystemImageEvent::XFileSystemImageEvents::Invoke(DISPID dispidMember, REFIID /*riid*/, LCID /*lcid*/, WORD wFlags, DISPPARAMS FAR* pdispparams, VARIANT FAR* pvarResult, EXCEPINFO FAR* pexcepinfo, UINT FAR* puArgErr)
{
	METHOD_PROLOGUE(CFileSystemImageEvent, FileSystemImageEvents)
	return DispInvoke(&pThis->m_xFileSystemImageEvents, pThis->pTypeInfo, dispidMember, wFlags, pdispparams, pvarResult, pexcepinfo, puArgErr);
}


CFileSystemImageEvent* CFileSystemImageEvent::CreateEventSink()
{
	CFileSystemImageEvent* pFileSystemImageEvent = new CFileSystemImageEvent();
	pFileSystemImageEvent->EnableAutomation();
	pFileSystemImageEvent->ExternalAddRef();
	return pFileSystemImageEvent;
}

bool CFileSystemImageEvent::ConnectFileSystemImage(CFileSystemImage* pFileSystemImage)
{
	pUnknownSink = GetIDispatch(FALSE);
	pUnknownSource = pFileSystemImage->GetInterface();

	LPTYPELIB pTypeLib = NULL;
	HRESULT hr = LoadRegTypeLib(LIBID_IMAPI2FS, IMAPI2FS_MajorVersion, IMAPI2FS_MinorVersion, LOCALE_SYSTEM_DEFAULT, &pTypeLib);
	if(FAILED(hr))
		return false;
	hr = pTypeLib->GetTypeInfoOfGuid(IID_DFileSystemImageEvents, &pTypeInfo);
	pTypeLib->Release();
	if(FAILED(hr))
		return false;

	BOOL b = AfxConnectionAdvise(pUnknownSource, IID_DFileSystemImageEvents, pUnknownSink, TRUE, &cookie);
	if(b)
		return true;
	return false;
}


STDMETHODIMP_(HRESULT) CFileSystemImageEvent::XFileSystemImageEvents::Update(LPDISPATCH /*pObjectDispatch*/, BSTR currentFile, LONG copiedSectors, LONG totalSectors)
{
	METHOD_PROLOGUE(CFileSystemImageEvent, FileSystemImageEvents)

	IMAPI_IMAGE_UPDATE_EVENT_ARGS eventArgs;
	eventArgs.currentFile = currentFile;
	if(eventArgs.currentFile.GetLength() == 0)
		eventArgs.currentFile = pThis->currentFile;
	else
		eventArgs.currentFile.Replace(TEXT("\\"), TEXT("/"));
	eventArgs.copiedSectors = copiedSectors;
	eventArgs.totalSectors = totalSectors;

	SendMessage(pThis->hNotificationWnd, WM_IMAPI_IMAGE_UPDATE, 0, reinterpret_cast<LPARAM>(&eventArgs));
	return S_OK;
}