#include "stdafx.h"
#include "common.h"
#include "APIWrapper.h"
#include "Galba.h"
#include "MainDlg.h"

#ifdef _DEBUG
	#define new DEBUG_NEW
#endif


// CGalbaApp

BEGIN_MESSAGE_MAP(CGalbaApp, CWinApp)
	ON_COMMAND(ID_HELP, &CWinApp::OnHelp)
END_MESSAGE_MAP()


// CGalbaApp construction

CGalbaApp::CGalbaApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}


// The one and only CGalbaApp object

CGalbaApp theApp;


// CGalbaApp initialization

BOOL CGalbaApp::InitInstance()
{
	INITCOMMONCONTROLSEX icc;
	icc.dwSize = sizeof(icc);
	icc.dwICC = ICC_WIN95_CLASSES;
	InitCommonControlsEx(&icc);

	CWinApp::InitInstance();

	AfxEnableControlContainer();

	AfxOleInit();
	//::CoInitializeEx(NULL, COINIT_MULTITHREADED);
	::CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);

	// retrieve settings
	CString settingsFile;
	TCHAR pBuffer[1024];
	if(SUCCEEDED(SHGetFolderPath(NULL, CSIDL_APPDATA, NULL, 0, pBuffer))) {
		PathAppend(pBuffer, TEXT("Galba\\config.xml"));
		settingsFile = pBuffer;
	}
	LoadSettings(settingsFile);

	// retrieve the resource dll to use
	CString resourceDll = GetResourceDll();
	HINSTANCE hDefInstance = AfxGetResourceHandle();
	HINSTANCE hResDll = LoadLibraryEx(resourceDll, NULL, LOAD_LIBRARY_AS_DATAFILE | LOAD_LIBRARY_AS_IMAGE_RESOURCE);
	if(hResDll)
		AfxSetResourceHandle(hResDll);

	CMainDlg dlg;
	m_pMainWnd = &dlg;
	dlg.DoModal();

	AfxSetResourceHandle(hDefInstance);
	if(hResDll)
		FreeLibrary(hResDll);

	SaveSettings(settingsFile);

	::CoUninitialize();

	// Since the dialog has been closed, return FALSE so that we exit the
	// application, rather than start the application's message pump.
	return FALSE;
}

CString CGalbaApp::GetResourceDll()
{
	TCHAR pBuffer[1024];
	GetModuleFileName(NULL, pBuffer, 1024);
	PathRemoveFileSpec(pBuffer);
	PathAddBackslash(pBuffer);
	CString appPath = pBuffer;

	CString resourceDll;
	TCHAR pLanguage[9];
	TCHAR pCountry[9];

	// 1st try: <language>-<country>.dll, e. g. de-DE.dll
	if(CAPIWrapper::IsSupported_GetUserDefaultLocaleName()) {
		WCHAR pBuffer[LOCALE_NAME_MAX_LENGTH];
		CAPIWrapper::GetUserDefaultLocaleName(pBuffer, LOCALE_NAME_MAX_LENGTH, NULL);
		resourceDll = pBuffer;
	} else {
		LANGID lcid = GetUserDefaultUILanguage();
		GetLocaleInfo(lcid, LOCALE_SISO639LANGNAME, pLanguage, 9);
		GetLocaleInfo(lcid, LOCALE_SISO3166CTRYNAME, pCountry, 9);
		resourceDll = pLanguage;
		resourceDll += TEXT("-");
		resourceDll += pCountry;
	}
	resourceDll += TEXT(".dll");

	if(PathFileExists(appPath + resourceDll))
		return appPath + resourceDll;

	// 2nd try: <language>.dll, e. g. de.dll
	if(CAPIWrapper::IsSupported_GetUserDefaultLocaleName()) {
		int p = resourceDll.Find(TEXT("-"));
		if(p >= 0)
			resourceDll = resourceDll.Left(p);
	} else {
		resourceDll = pLanguage;
	}
	resourceDll += TEXT(".dll");

	if(PathFileExists(appPath + resourceDll))
		return appPath + resourceDll;

	// last try: <language>-*.dll, e. g. de-AT.dll or de-CH.dll
	if(CAPIWrapper::IsSupported_GetUserDefaultLocaleName()) {
		int p = resourceDll.Find(TEXT(".dll"));
		if(p >= 0)
			resourceDll = resourceDll.Left(p);
	} else {
		resourceDll = pLanguage;
	}
	resourceDll += TEXT("-*.dll");
	WIN32_FIND_DATA findData = {0};
	BOOL found = FALSE;
	HANDLE hFind = FindFirstFile(appPath + resourceDll, &findData);
	if(hFind != INVALID_HANDLE_VALUE) {
		do {
			if(!(findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
				// found a file
				resourceDll = findData.cFileName;
				found = TRUE;
				break;
			}
			if(!FindNextFile(hFind, &findData)) {
				// no more files
				break;
			}
		} while(TRUE);
		FindClose(hFind);
	}

	if(found)
		return appPath + resourceDll;

	// no dll found, so use the built-in en-US GUI
	resourceDll = TEXT("");
	return resourceDll;
}

void CGalbaApp::LoadSettings(CString& settingsFile)
{
	LoadDefaultSettings();
	settings.Load(settingsFile);
}

void CGalbaApp::LoadDefaultSettings()
{
	settings.SetSetting(CString("Main\\SelectedRecorder"), _bstr_t(""));
	settings.SetSetting(CString("Main\\AlwaysOnTop"), VARIANT_FALSE);
	settings.SetSetting(CString("CreateDataDisc\\CloseMedia"), VARIANT_TRUE);
	settings.SetSetting(CString("CreateDataDisc\\EjectMedia"), VARIANT_FALSE);
	settings.SetSetting(CString("CreateDataDisc\\LastDir"), _bstr_t(""));
	settings.SetSetting(CString("BurnImage\\CloseMedia"), VARIANT_TRUE);
	settings.SetSetting(CString("BurnImage\\EjectMedia"), VARIANT_FALSE);
	settings.SetSetting(CString("EraseDisc\\EraseMethod"), 0);
	settings.SetSetting(CString("EraseDisc\\EjectMedia"), VARIANT_FALSE);
}

void CGalbaApp::SaveSettings(CString& settingsFile)
{
	settings.Save(settingsFile);
}