#include "stdafx.h"
#include "Galba.h"
#include "MainDlg.h"
#include "CreateDataDiscDlg.h"
#include "BurnImageDlg.h"
#include "EraseDiscDlg.h"

#include "DiscMaster.h"
#include "DiscRecorder.h"
#include "DiscFormatData.h"


// CMainDlg dialog

IMPLEMENT_DYNAMIC(CMainDlg, CDialog)


CMainDlg::CMainDlg(CWnd* pParent/* = NULL*/) :
    CDialog(CMainDlg::IDD, pParent),
    pTaskBarList(NULL),
    pPipeServerThread(NULL),
    pCurrentTaskDialog(NULL),
    currentDeviceSelection(-1),
    /*isAlwaysOnTop(false),*/
    dropTarget(this)
{
	hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	CoCreateInstance(CLSID_TaskbarList, NULL, CLSCTX_INPROC_SERVER, IID_PPV_ARGS(&pTaskBarList));
	if(pTaskBarList)
		pTaskBarList->HrInit();
}

CMainDlg::~CMainDlg()
{
	if(pPipeServerThread) {
		// connect to the named pipe to end the blocking call
		HANDLE hPipe = CreateFile(NAMED_PIPE_NAME, GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_FLAG_WRITE_THROUGH, NULL);
		if(hPipe != INVALID_HANDLE_VALUE) {
			LPCTSTR pMessage = TEXT("<Command Type=\"ClosePipe\" />");
			DWORD bytesWritten = 0;
			WriteFile(hPipe, pMessage, lstrlen(pMessage) * sizeof(TCHAR), &bytesWritten, NULL);
			CloseHandle(hPipe);
		}
		WaitForSingleObject(*pPipeServerThread, INFINITE);
	}
	if(pTaskBarList)
		pTaskBarList->Release();
}

void CMainDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_TASK_LIST, controls.taskList);
	DDX_Control(pDX, IDC_DEVICE_COMBO, controls.deviceCombo);
	DDX_Control(pDX, ID_EJECT, controls.ejectButton);
	DDX_Control(pDX, ID_RELOAD_DEVICES, controls.reloadButton);
	DDX_Control(pDX, IDC_MEDIATYPE, controls.mediaType);
	DDX_Control(pDX, IDC_FEATURES_LIST, controls.featuresList);
	DDX_Control(pDX, IDC_ALWAYS_ON_TOP_CHK, controls.alwaysOnTopCheck);
	DDX_Control(pDX, IDC_STATIC_PLACEHOLDER, controls.dialogPlaceHolder);
	DDX_Control(pDX, IDC_PROGRESS, controls.progress);
	DDX_Control(pDX, IDC_PROGRESS_TEXT, controls.progressText);
	DDX_Control(pDX, IDC_TIME_ELAPSED, controls.timeElapsed);
	DDX_Control(pDX, IDC_TIME_REMAINING, controls.timeRemaining);
	DDX_Control(pDX, IDC_BUFFER_PROG, controls.bufferProgress);
	DDX_Control(pDX, IDCANCEL, controls.cancelButton);
}


BEGIN_MESSAGE_MAP(CMainDlg, CDialog)
	ON_WM_SYSCOMMAND()
	ON_WM_DESTROY()
	ON_WM_DEVICECHANGE()
	ON_LBN_SELCHANGE(IDC_TASK_LIST, &CMainDlg::OnLbnSelchangeTaskList)
	ON_CBN_SELCHANGE(IDC_DEVICE_COMBO, &CMainDlg::OnCbnSelchangeDeviceCombo)
	ON_BN_CLICKED(IDC_ALWAYS_ON_TOP_CHK, &CMainDlg::OnBnClickedAlwaysOnTopCheck)
	ON_COMMAND(ID_EJECT, &CMainDlg::OnEject)
	ON_COMMAND(ID_RELOAD_DEVICES, &CMainDlg::OnReloadDevices)

	ON_MESSAGE(WM_DEVICE_CHANGE, &CMainDlg::OnDeviceChange)
	ON_MESSAGE(WM_PIPE_SERVER_INITIALIZED, &CMainDlg::OnPipeServerInitialized)
END_MESSAGE_MAP()


BOOL CMainDlg::PreTranslateMessage(MSG* pMsg)
{
	if(pCurrentTaskDialog && pCurrentTaskDialog->IsChild(GetFocus()))
		return pCurrentTaskDialog->PreTranslateMessage(pMsg);
	return CDialog::PreTranslateMessage(pMsg);
}


// implementation of IGalbaWindow

HRESULT CMainDlg::DeviceChange()
{
	PostMessage(WM_DEVICE_CHANGE);
	return S_OK;
}

HRESULT CMainDlg::IsImageRecorderSelected(BOOL& imageRecorderSelected)
{
	int selection = controls.deviceCombo.GetCurSel();
	imageRecorderSelected = (selection == 0);
	return S_OK;
}

HRESULT CMainDlg::GetRecorder(CDiscRecorder** ppRecorder)
{
	*ppRecorder = NULL;

	int selection = controls.deviceCombo.GetCurSel();
	if(selection >= 0)
		*ppRecorder = reinterpret_cast<CDiscRecorder*>(controls.deviceCombo.GetItemDataPtr(selection));
	return S_OK;
}

HRESULT CMainDlg::GetRecorderPath(CString& path)
{
	path = TEXT("");
	int selection = controls.deviceCombo.GetCurSel();
	if(selection == 0)
		path = IMAGE_RECORDER_ID;
	else if(selection > 0) {
		controls.deviceCombo.GetLBText(selection, path);
		path = path.Left(2);
		path += TEXT("\\");
	}
	return S_OK;
}

HRESULT CMainDlg::EnableUI(BOOL enable)
{
	controls.taskList.EnableWindow(enable);
	controls.deviceCombo.EnableWindow(enable);
	controls.reloadButton.EnableWindow(enable);

	CDiscRecorder* pRecorder = NULL;
	GetRecorder(&pRecorder);
	controls.ejectButton.EnableWindow(enable && (pRecorder != NULL));
	return S_OK;
}

HRESULT CMainDlg::EnableCancelButton(BOOL enable)
{
	controls.cancelButton.EnableWindow(enable);

	CMenu* pSystemMenu = GetSystemMenu(FALSE);
	if(pSystemMenu) {
		UINT flags = MF_BYCOMMAND;
		if(pCurrentTaskDialog && pCurrentTaskDialog->IsRunningInterruptibleTask())
			flags |= enable ? 0 : MF_GRAYED;
		pSystemMenu->EnableMenuItem(SC_CLOSE, flags);
	}
	return S_OK;
}

HRESULT CMainDlg::SetupProgressMarquee(BOOL enable)
{
	if(enable) {
		controls.progress.ModifyStyle(0, PBS_MARQUEE);
		controls.progress.SetMarquee(TRUE, 0);
	} else {
		controls.progress.SetMarquee(FALSE, 0);
		controls.progress.ModifyStyle(PBS_MARQUEE, 0);
	}

	if(pTaskBarList) {
		TBPFLAG state = enable ? TBPF_INDETERMINATE : TBPF_NOPROGRESS;
		pTaskBarList->SetProgressState(*this, state);
	}
	return S_OK;
}

HRESULT CMainDlg::SetProgress(int percentsDone, CString& text)
{
	controls.progress.SetPos(percentsDone);
	CString currentText;
	controls.progressText.GetWindowText(currentText);
	if(text != currentText)
		controls.progressText.SetWindowText(text);

	if(pTaskBarList) {
		TBPFLAG state = percentsDone > 0 ? TBPF_NORMAL : TBPF_NOPROGRESS;
		pTaskBarList->SetProgressState(*this, state);
		if(state == TBPF_NORMAL)
			pTaskBarList->SetProgressValue(*this, percentsDone, 100);
	}
	return S_OK;
}

HRESULT CMainDlg::SetSystemBufferUsage(int percents)
{
	controls.bufferProgress.SetPos(percents);
	return S_OK;
}

HRESULT CMainDlg::SetTimes(LONG timeElapsed, LONG timeRemaining)
{
	CString s;
	if(timeElapsed >= 0)
		s.Format(IDS_TIME_ELAPSED1, timeElapsed / 60, timeElapsed % 60);
	else
		s.LoadString(IDS_TIME_ELAPSED2);
	controls.timeElapsed.SetWindowText(s);

	if(timeRemaining >= 0)
		s.Format(IDS_TIME_REMAINING1, timeRemaining / 60, timeRemaining % 60);
	else
		s.LoadString(IDS_TIME_REMAINING2);
	controls.timeRemaining.SetWindowText(s);
	return S_OK;
}


UINT CMainDlg::PipeServerThread(LPVOID pParam)
{
	PTHREAD_PARAMS_PIPESERVER pThreadData = reinterpret_cast<PTHREAD_PARAMS_PIPESERVER>(pParam);

	HANDLE hPipe = CreateNamedPipe(NAMED_PIPE_NAME, PIPE_ACCESS_INBOUND | FILE_FLAG_FIRST_PIPE_INSTANCE, PIPE_TYPE_MESSAGE | PIPE_READMODE_MESSAGE | PIPE_WAIT | PIPE_REJECT_REMOTE_CLIENTS, 1, 1024, 1024, 0, NULL);
	if(hPipe != INVALID_HANDLE_VALUE) {
		::PostMessage(pThreadData->hNotificationWnd, WM_PIPE_SERVER_INITIALIZED, 0, reinterpret_cast<LPARAM>(hPipe));

		const int PIPE_READ_BUFFER_SIZE = 1024;
		TCHAR pBuffer[PIPE_READ_BUFFER_SIZE];
		CString commandString;
		BOOL gotAllData;
		DWORD bytesRead;
		while(ConnectNamedPipe(hPipe, NULL) || GetLastError() == ERROR_PIPE_CONNECTED) {
			commandString.Empty();
			do {
				bytesRead = 0;
				gotAllData = (ReadFile(hPipe, reinterpret_cast<LPVOID>(pBuffer), PIPE_READ_BUFFER_SIZE * sizeof(TCHAR), &bytesRead, NULL) || GetLastError() != ERROR_MORE_DATA);
				if(bytesRead > 0)
					commandString.Append(pBuffer, bytesRead / sizeof(TCHAR));
			} while(!gotAllData);
			DisconnectNamedPipe(hPipe);
			if(!pThreadData->pThis->ProcessXmlCommand(commandString, pThreadData->pReceivingDialog))
				break;
		}
		CloseHandle(hPipe);
	}

	pThreadData->pThis->pPipeServerThread = NULL;
	delete pThreadData;
	return 0;
}


DROPEFFECT CMainDlg::OnFileDragOver(COleDataObject* pDataObject, DWORD keyState, CPoint point)
{
	DROPEFFECT effect = DROPEFFECT_NONE;

	POINT pt = point;
	ClientToScreen(&pt);
	CWnd* pWnd = WindowFromPoint(pt);
	if(pWnd && pCurrentTaskDialog) {
		if(pWnd->GetSafeHwnd() == pCurrentTaskDialog->GetSafeHwnd() || pCurrentTaskDialog->IsChild(pWnd)) {
			pCurrentTaskDialog->ScreenToClient(&pt);
			effect = pCurrentTaskDialog->OnFileDragOver(pDataObject, keyState, pt);
		}
	}

	static CLIPFORMAT cfDropDescription = 0;
	if(IsVistaOrNewer()) {
		if(cfDropDescription == 0)
			cfDropDescription = static_cast<CLIPFORMAT>(RegisterClipboardFormat(CFSTR_DROPDESCRIPTION));
		DROPDESCRIPTION dd;

		if(effect == DROPEFFECT_COPY) {
			CString dropDescription;
			dropDescription.LoadString(IDS_DROPDESC_ADD_TO_COMPILATION);

			dd.type = DROPIMAGE_COPY;
			StringCchCopyW(dd.szMessage, ARRAYSIZE(dd.szMessage), dropDescription);
			StringCchCopyW(dd.szInsert, ARRAYSIZE(dd.szInsert), L"");
		} else {
			dd.type = DROPIMAGE_INVALID;
			StringCchCopyW(dd.szMessage, ARRAYSIZE(dd.szMessage), L"");
			StringCchCopyW(dd.szInsert, ARRAYSIZE(dd.szInsert), L"");
		}

		LPVOID pv = GlobalAlloc(GPTR, sizeof(dd));
		HRESULT hr = pv ? S_OK : E_OUTOFMEMORY;
		if(SUCCEEDED(hr)) {
			CopyMemory(pv, &dd, sizeof(dd));
			FORMATETC fmt = {cfDropDescription, NULL, DVASPECT_CONTENT, -1, TYMED_HGLOBAL};
			STGMEDIUM medium = {};
			medium.tymed = TYMED_HGLOBAL;
			medium.hGlobal = pv;
			hr = pDataObject->GetIDataObject(FALSE)->SetData(&fmt, &medium, TRUE);
			if(FAILED(hr))
				GlobalFree(pv);
		}
	}

	return effect;
}

void CMainDlg::OnDragLeave(void)
{
	if(pCurrentTaskDialog)
		pCurrentTaskDialog->OnDragLeave();
}

BOOL CMainDlg::OnFileDrop(COleDataObject* pDataObject, DWORD keyState, CPoint point)
{
	POINT pt = point;
	ClientToScreen(&pt);
	CWnd* pWnd = WindowFromPoint(pt);
	if(pWnd && pCurrentTaskDialog) {
		if(pWnd->GetSafeHwnd() == pCurrentTaskDialog->GetSafeHwnd() || pCurrentTaskDialog->IsChild(pWnd)) {
			pCurrentTaskDialog->ScreenToClient(&pt);
			return pCurrentTaskDialog->OnFileDrop(pDataObject, keyState, pt);
		}
	}
	return FALSE;
}


// CMainDlg message handlers

BOOL CMainDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	CString caption;
	caption.Format(IDS_CAPTION, VERSION_MAJOR, VERSION_MINOR);
	SetWindowText(caption);

	/*// ID_ALWAYS_ON_TOP must be in the system command range.
	ASSERT((ID_ALWAYS_ON_TOP & 0xFFF0) == ID_ALWAYS_ON_TOP);
	ASSERT(ID_ALWAYS_ON_TOP < 0xF000);
	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if(pSysMenu) {
		CString alwaysOnTopMenu;
		alwaysOnTopMenu.LoadString(IDS_ALWAYS_ON_TOP);
		if(!alwaysOnTopMenu.IsEmpty()) {
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, ID_ALWAYS_ON_TOP, alwaysOnTopMenu);
		}
	}*/

	CString taskName;
	taskName.LoadString(IDS_TASK_CREATE_DATA_DISC);
	int taskIndex = controls.taskList.AddString(taskName);
	CBurnTaskDialog* pTaskDialog = new CCreateDataDiscDlg(this);
	pTaskDialog->Create(CCreateDataDiscDlg::IDD, this);
	if(pTaskDialog->IsInitialized()) {
		pTaskDialog->SetParent(this);
		controls.taskList.SetItemDataPtr(taskIndex, pTaskDialog);
	} else {
		pTaskDialog->DestroyWindow();
		delete pTaskDialog;
	}
	taskName.LoadString(IDS_TASK_BURN_IMAGE);
	taskIndex = controls.taskList.AddString(taskName);
	pTaskDialog = new CBurnImageDlg(this);
	pTaskDialog->Create(CBurnImageDlg::IDD, this);
	if(pTaskDialog->IsInitialized()) {
		pTaskDialog->SetParent(this);
		controls.taskList.SetItemDataPtr(taskIndex, pTaskDialog);
	} else {
		pTaskDialog->DestroyWindow();
		delete pTaskDialog;
	}
	taskName.LoadString(IDS_TASK_ERASE_MEDIA);
	taskIndex = controls.taskList.AddString(taskName);
	pTaskDialog = new CEraseDiscDlg(this);
	pTaskDialog->Create(CEraseDiscDlg::IDD, this);
	if(pTaskDialog->IsInitialized()) {
		pTaskDialog->SetParent(this);
		controls.taskList.SetItemDataPtr(taskIndex, pTaskDialog);
	} else {
		pTaskDialog->DestroyWindow();
		delete pTaskDialog;
	}
	controls.taskList.SetCurSel(0);
	OnLbnSelchangeTaskList();

	CBitmap bmp;
	bmp.LoadBitmap(IDB_RELOAD_DEVICES);
	controls.reloadDevicesImageList.Create(16, 16, ILC_COLOR32 | ILC_MASK, 6, 0);
	controls.reloadDevicesImageList.Add(&bmp, reinterpret_cast<CBitmap*>(NULL));
	BUTTON_IMAGELIST iml = {0};
	iml.himl = controls.reloadDevicesImageList;
	iml.uAlign = BUTTON_IMAGELIST_ALIGN_CENTER;
	controls.reloadButton.SetImageList(&iml);

	controls.progress.SetRange(0, 100);
	controls.progressText.SetWindowText(TEXT(""));
	controls.mediaType.SetWindowText(TEXT(""));

	if(!AddRecordersToComboBox()) {
		EndDialog(IDOK);
		return TRUE;
	}
	CString recorderToSelect = theApp.settings.GetSetting(CString("Main\\SelectedRecorder"));
	BOOL found = FALSE;
	if(!recorderToSelect.IsEmpty()) {
		int itemCount = controls.deviceCombo.GetCount();
		if(recorderToSelect == TEXT("Image Recorder")) {
			if(itemCount > 0) {
				found = TRUE;
				controls.deviceCombo.SetCurSel(0);
				DeviceChange();
			}
		} else {
			for(int i = 0; i < itemCount; i++) {
				CDiscRecorder* pRecorder = reinterpret_cast<CDiscRecorder*>(controls.deviceCombo.GetItemDataPtr(i));
				if(pRecorder && (pRecorder->GetUniqueId() == recorderToSelect)) {
					found = TRUE;
					controls.deviceCombo.SetCurSel(i);
					DeviceChange();
					break;
				}
			}
		}
	}
	if(!found) {
		if(controls.deviceCombo.GetCount() > 0) {
			controls.deviceCombo.SetCurSel(controls.deviceCombo.GetCount() > 1 ? 1 : 0);
			DeviceChange();
		}
	}

	VARIANT_BOOL b = theApp.settings.GetSetting(CString("Main\\AlwaysOnTop"));
	controls.alwaysOnTopCheck.SetCheck(b == VARIANT_FALSE ? BST_UNCHECKED : BST_CHECKED);
	OnBnClickedAlwaysOnTopCheck();

	SetIcon(hIcon, TRUE);
	SetIcon(hIcon, FALSE);

	dropTarget.Register(this);

	PTHREAD_PARAMS_PIPESERVER pThreadData = new THREAD_PARAMS_PIPESERVER;
	pThreadData->pThis = this;
	pThreadData->hNotificationWnd = *this;
	pThreadData->pReceivingDialog = reinterpret_cast<CBurnTaskDialog*>(controls.taskList.GetItemDataPtr(0));
	pPipeServerThread = AfxBeginThread(PipeServerThread, pThreadData, THREAD_PRIORITY_NORMAL);
	if(!pPipeServerThread)
		delete pThreadData;

	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CMainDlg::OnCancel()
{
	if(pCurrentTaskDialog && pCurrentTaskDialog->IsRunningInterruptibleTask()) {
		CString message;
		message.LoadString(IDS_ABORT_OPERATION);
		int result = AfxMessageBox(message, MB_ICONQUESTION | MB_YESNO);
		if(result == IDYES && pCurrentTaskDialog->IsRunningInterruptibleTask())
			pCurrentTaskDialog->CancelInterruptibleTask();
	}
}

void CMainDlg::OnOK()
{
}

void CMainDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	switch(nID & 0xFFF0) {
		case SC_CLOSE:
			if(pCurrentTaskDialog && pCurrentTaskDialog->IsRunningInterruptibleTask()) {
				CString message;
				message.LoadString(IDS_ABORT_OPERATION);
				int result = AfxMessageBox(message, MB_ICONQUESTION | MB_YESNO);
				if(result == IDYES && pCurrentTaskDialog->IsRunningInterruptibleTask())
					pCurrentTaskDialog->CancelInterruptibleTask();
				else
					return;
			}
			EndDialog(IDCANCEL);
			break;
		/*case ID_ALWAYS_ON_TOP:
			isAlwaysOnTop = !isAlwaysOnTop;
			CMenu* pSysMenu = GetSystemMenu(FALSE);
			if(pSysMenu)
				pSysMenu->CheckMenuItem(ID_ALWAYS_ON_TOP, MF_BYCOMMAND | (isAlwaysOnTop ? MF_CHECKED : MF_UNCHECKED));
			if(isAlwaysOnTop)
				SetWindowPos(&wndTopMost, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);
			else
				SetWindowPos(&wndNoTopMost, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);
			break;*/
	}
	CDialog::OnSysCommand(nID, lParam);
}

void CMainDlg::OnDestroy()
{
	dropTarget.Revoke();

	CString s;
	int selection = controls.deviceCombo.GetCurSel();
	if(selection == 0) {
		s = TEXT("Image Recorder");
	} else if(selection > 0) {
		CDiscRecorder* pRecorder = reinterpret_cast<CDiscRecorder*>(controls.deviceCombo.GetItemDataPtr(selection));
		if(pRecorder)
			s = pRecorder->GetUniqueId();
	}
	theApp.settings.SetSetting(CString("Main\\SelectedRecorder"), _bstr_t(s));
	VARIANT_BOOL b = (controls.alwaysOnTopCheck.GetCheck() == BST_CHECKED ? VARIANT_TRUE : VARIANT_FALSE);
	theApp.settings.SetSetting(CString("Main\\AlwaysOnTop"), b);

	int itemCount = controls.taskList.GetCount();
	for(int i = 0; i < itemCount; i++) {
		CBurnTaskDialog* pTaskDialog = reinterpret_cast<CBurnTaskDialog*>(controls.taskList.GetItemDataPtr(i));
		if(pTaskDialog) {
			pTaskDialog->DestroyWindow();
			delete pTaskDialog;
		}
	}

	itemCount = controls.deviceCombo.GetCount();
	for(int i = 0; i < itemCount; i++) {
		CDiscRecorder* p = reinterpret_cast<CDiscRecorder*>(controls.deviceCombo.GetItemDataPtr(i));
		if(p)
			delete p;
	}
}

BOOL CMainDlg::OnDeviceChange(UINT nEventType, DWORD_PTR dwData)
{
	if(nEventType == DBT_DEVICEARRIVAL || nEventType == DBT_DEVICEREMOVECOMPLETE) {
		PDEV_BROADCAST_HDR pDetails = reinterpret_cast<PDEV_BROADCAST_HDR>(dwData);
		if(pDetails->dbch_devicetype == DBT_DEVTYP_VOLUME) {
			PDEV_BROADCAST_VOLUME pVolumeDetails = reinterpret_cast<PDEV_BROADCAST_VOLUME>(pDetails);
			if(pVolumeDetails->dbcv_flags & DBTF_MEDIA)
				DeviceChange();
		}
	}
	return TRUE;
}

LRESULT CMainDlg::OnDeviceChange(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	RetrieveMediaType();
	AddDriveFeaturesToListBox();

	int c = controls.taskList.GetCount();
	for(int i = 0; i < c; i++) {
		CBurnTaskDialog* pTaskDialog = reinterpret_cast<CBurnTaskDialog*>(controls.taskList.GetItemDataPtr(i));
		if(pTaskDialog && pTaskDialog->IsInitialized())
			pTaskDialog->MediaChanged();
	}
	return 0;
}

LRESULT CMainDlg::OnPipeServerInitialized(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	return 0;
}

void CMainDlg::OnLbnSelchangeTaskList()
{
	int selection = controls.taskList.GetCurSel();

	CString task;
	controls.taskList.GetText(selection, task);
	controls.dialogPlaceHolder.SetWindowText(task);

	CRect dialogRect;
	controls.dialogPlaceHolder.GetClientRect(&dialogRect);
	controls.dialogPlaceHolder.MapWindowPoints(this, &dialogRect);

	CBurnTaskDialog* pTaskDialog = reinterpret_cast<CBurnTaskDialog*>(controls.taskList.GetItemDataPtr(selection));
	if(pTaskDialog)
		pTaskDialog->MoveWindow(dialogRect.left, dialogRect.top, dialogRect.Width(), dialogRect.Height());

	if(pCurrentTaskDialog)
		pCurrentTaskDialog->ShowWindow(SW_HIDE);
	pCurrentTaskDialog = pTaskDialog;
	if(pCurrentTaskDialog)
		pCurrentTaskDialog->ShowWindow(SW_SHOW);
}

void CMainDlg::OnCbnSelchangeDeviceCombo()
{
	int selection = controls.deviceCombo.GetCurSel();
	if(selection != currentDeviceSelection) {
		DeviceChange();
		currentDeviceSelection = selection;

		CDiscRecorder* pRecorder = NULL;
		GetRecorder(&pRecorder);
		controls.ejectButton.EnableWindow(pRecorder != NULL);
	}
}

void CMainDlg::OnBnClickedAlwaysOnTopCheck()
{
	if(controls.alwaysOnTopCheck.GetCheck() == BST_CHECKED)
		SetWindowPos(&wndTopMost, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);
	else
		SetWindowPos(&wndNoTopMost, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);
}

void CMainDlg::OnEject()
{
	CDiscRecorder* pRecorder = NULL;
	GetRecorder(&pRecorder);
	if(pRecorder)
		pRecorder->EjectMedia();
}

void CMainDlg::OnReloadDevices()
{
	CString selectedRecorderID;
	CDiscRecorder* pRecorder = NULL;
	GetRecorder(&pRecorder);
	if(pRecorder)
		selectedRecorderID = pRecorder->GetUniqueId();

	AddRecordersToComboBox();

	CDiscRecorder* p;
	for(int i = 0; i < controls.deviceCombo.GetCount(); i++) {
		p = reinterpret_cast<CDiscRecorder*>(controls.deviceCombo.GetItemDataPtr(i));
		if(p && p->GetUniqueId() == selectedRecorderID) {
			controls.deviceCombo.SetCurSel(i);
			DeviceChange();
			return;
		}
	}
	controls.deviceCombo.SetCurSel(controls.deviceCombo.GetCount() > 1 ? 1 : 0);
	DeviceChange();
}


CString	CMainDlg::GetMediaTypeString(IMAPI_MEDIA_PHYSICAL_TYPE mediaType)
{
	CString s;
	switch(mediaType) {
		case IMAPI_MEDIA_TYPE_UNKNOWN:
		default:
			s.LoadString(IDS_MEDIA_TYPE_UNKNOWN);
			break;
		case IMAPI_MEDIA_TYPE_CDROM:
			s = TEXT("CD-ROM");
			break;
		case IMAPI_MEDIA_TYPE_CDR:
			s = TEXT("CD-R");
			break;
		case IMAPI_MEDIA_TYPE_CDRW:
			s = TEXT("CD-RW");
			break;
		case IMAPI_MEDIA_TYPE_DVDROM:
			s = TEXT("DVD ROM");
			break;
		case IMAPI_MEDIA_TYPE_DVDRAM:
			s = TEXT("DVD-RAM");
			break;
		case IMAPI_MEDIA_TYPE_DVDPLUSR:
			s = TEXT("DVD+R");
			break;
		case IMAPI_MEDIA_TYPE_DVDPLUSRW:
			s = TEXT("DVD+RW");
			break;
		case IMAPI_MEDIA_TYPE_DVDPLUSR_DUALLAYER:
			s = TEXT("DVD+R DL");
			break;
		case IMAPI_MEDIA_TYPE_DVDDASHR:
			s = TEXT("DVD-R");
			break;
		case IMAPI_MEDIA_TYPE_DVDDASHRW:
			s = TEXT("DVD-RW");
			break;
		case IMAPI_MEDIA_TYPE_DVDDASHR_DUALLAYER:
			s = TEXT("DVD-R DL");
			break;
		case IMAPI_MEDIA_TYPE_DISK:
			s.LoadString(IDS_MEDIA_TYPE_DISK);
			break;
		case IMAPI_MEDIA_TYPE_DVDPLUSRW_DUALLAYER:
			s = TEXT("DVD+RW DL");
			break;
		case IMAPI_MEDIA_TYPE_HDDVDROM:
			s = TEXT("HD DVD-ROM");
			break;
		case IMAPI_MEDIA_TYPE_HDDVDR:
			s = TEXT("HD DVD-R");
			break;
		case IMAPI_MEDIA_TYPE_HDDVDRAM:
			s = TEXT("HD DVD-RAM");
			break;
		case IMAPI_MEDIA_TYPE_BDROM:
			s = TEXT("BD-ROM");
			break;
		case IMAPI_MEDIA_TYPE_BDR:
			s = TEXT("BD-R");
			break;
		case IMAPI_MEDIA_TYPE_BDRE:
			s = TEXT("BD-RE");
			break;
	}
	return s;
}

bool CMainDlg::AddRecordersToComboBox()
{
	CDiscMaster discMaster;

	int itemCount = controls.deviceCombo.GetCount();
	for(int i = 0; i < itemCount; i++) {
		CDiscRecorder* p = reinterpret_cast<CDiscRecorder*>(controls.deviceCombo.GetItemDataPtr(i));
		if(p)
			delete p;
	}
	controls.deviceCombo.ResetContent();
	controls.deviceCombo.AddString(IMAGE_RECORDER_ID);

	if(!discMaster.Initialize()) {
		AfxMessageBox(discMaster.GetErrorMessage(), MB_OK | MB_ICONERROR);
		return true;//false;
	}

	// add devices to ComboBox
	LONG totalDevices = discMaster.GetTotalDevices();
	if(totalDevices == 0 && FAILED(discMaster.GetHresult()))
		AfxMessageBox(discMaster.GetErrorMessage(), MB_OK | MB_ICONERROR);

	for(long deviceIndex = 0; deviceIndex < totalDevices; deviceIndex++) {
		CString recorderUniqueID = discMaster.GetDeviceUniqueID(deviceIndex);
		if(recorderUniqueID.IsEmpty()) {
			CString errorMessage(discMaster.GetErrorMessage());
			if(!errorMessage.IsEmpty()) {
				AfxMessageBox(errorMessage, MB_OK | MB_ICONERROR);
				continue;
			}
		}

		// create an IDiscRecorder2
		CDiscRecorder* pDiscRecorder = new CDiscRecorder();
		ASSERT(pDiscRecorder != NULL);
		if(!pDiscRecorder)
			continue;

		if(!pDiscRecorder->Initialize(recorderUniqueID)) {
			if(totalDevices == 1 && FAILED(pDiscRecorder->GetHresult()))
				AfxMessageBox(pDiscRecorder->GetErrorMessage(), MB_OK | MB_ICONERROR);
			delete pDiscRecorder;
			continue;
		}

		// check for CD/DVD writing features
		CDiscFormatData discFormatData;
		if(!discFormatData.Initialize(pDiscRecorder, CLIENT_NAME, *this)) {
			//AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
			delete pDiscRecorder;
			continue;
		}

		// get the volume path(s)
		CString volumeList;
		ULONG totalVolumePaths = pDiscRecorder->GetTotalVolumePaths();
		for(ULONG volIndex = 0; volIndex < totalVolumePaths; volIndex++) {
			if(volIndex)
				volumeList += TEXT(",");
			volumeList += pDiscRecorder->GetVolumePath(volIndex);
		}

		// add drive to combo and IDiscRecorder as data
		CString vendorId = pDiscRecorder->GetVendorID().Trim();
		CString productId = pDiscRecorder->GetProductID().Trim();
		CString productRevision = pDiscRecorder->GetProductRevision().Trim();
		CString strName;
		strName.Format(TEXT("%s [%s %s %s]"), volumeList.Left(2), vendorId, productId, productRevision);
		int recorderIndex = controls.deviceCombo.AddString(strName);
		controls.deviceCombo.SetItemDataPtr(recorderIndex, pDiscRecorder);
	}
	return true;
}

void CMainDlg::AddDriveFeaturesToListBox()
{
	controls.featuresList.ResetContent();

	int selection = controls.deviceCombo.GetCurSel();
	if(selection == 0) {
		CString mediaType = GetMediaTypeString(IMAPI_MEDIA_TYPE_CDR);
		controls.featuresList.AddString(mediaType);
		mediaType = GetMediaTypeString(IMAPI_MEDIA_TYPE_DVDPLUSR);
		controls.featuresList.AddString(mediaType);
		mediaType = GetMediaTypeString(IMAPI_MEDIA_TYPE_DVDPLUSR_DUALLAYER);
		controls.featuresList.AddString(mediaType);
		mediaType = GetMediaTypeString(IMAPI_MEDIA_TYPE_DVDDASHR);
		controls.featuresList.AddString(mediaType);
		mediaType = GetMediaTypeString(IMAPI_MEDIA_TYPE_DVDDASHR_DUALLAYER);
		controls.featuresList.AddString(mediaType);
		mediaType = GetMediaTypeString(IMAPI_MEDIA_TYPE_HDDVDR);
		controls.featuresList.AddString(mediaType);
		mediaType = GetMediaTypeString(IMAPI_MEDIA_TYPE_BDR);
		controls.featuresList.AddString(mediaType);
	} else if(selection > 0) {
		CDiscRecorder* pRecorder = reinterpret_cast<CDiscRecorder*>(controls.deviceCombo.GetItemDataPtr(selection));
		if(pRecorder) {
			CDiscFormatData discFormatData;
			if(!discFormatData.Initialize(pRecorder, CLIENT_NAME, *this)) {
				AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
				return;
			}

			ULONG c = discFormatData.GetTotalSupportedMediaTypes();
			for(ULONG i = 0; i < c; i++) {
				CString mediaType = GetMediaTypeString(discFormatData.GetSupportedMediaType(i));
				controls.featuresList.AddString(mediaType);
			}
		}
	}
}

void CMainDlg::RetrieveMediaType()
{
	controls.mediaType.SetWindowText(TEXT(""));

	int selection = controls.deviceCombo.GetCurSel();
	if(selection == 0)
		controls.mediaType.SetWindowText(TEXT(""));
	else if(selection > 0) {
		CDiscRecorder* pRecorder = reinterpret_cast<CDiscRecorder*>(controls.deviceCombo.GetItemDataPtr(selection));
		if(pRecorder) {
			CDiscFormatData discFormatData;
			if(!discFormatData.Initialize(pRecorder, CLIENT_NAME, *this)) {
				AfxMessageBox(discFormatData.GetErrorMessage(), MB_ICONERROR | MB_OK);
				return;
			}

			CString s;
			s.Format(IDS_MEDIA_TYPE, GetMediaTypeString(discFormatData.GetCurrentPhysicalMediaType()));
			controls.mediaType.SetWindowText(s);
		}
	}
}


bool CMainDlg::ProcessXmlCommand(const CString& commandString, CBurnTaskDialog* pReceivingDialog)
{
	MSXML2::IXMLDOMDocument2Ptr xmlDoc(__uuidof(MSXML2::DOMDocument30));
	const _bstr_t bstr = commandString;
	if(xmlDoc->loadXML(bstr) == VARIANT_FALSE)
		return false;

	CString commandType;

	MSXML2::IXMLDOMElementPtr envelope = xmlDoc->GetdocumentElement();
	if(envelope) {
		MSXML2::IXMLDOMNodePtr attribute = envelope->attributes->getNamedItem(_bstr_t("Type"));
		if(attribute) {
			commandType = static_cast<LPCTSTR>(attribute->Gettext());

			if(commandType == TEXT("AddData")) {
				if(pReceivingDialog) {
					for(long i = 0; i < envelope->childNodes->length; i++) {
						MSXML2::IXMLDOMNodePtr baseDirElement = envelope->childNodes->Getitem(i);
						if(!baseDirElement)
							continue;
						CString nodeType = baseDirElement->GetnodeName();
						if(nodeType != TEXT("BaseDir"))
							continue;
						attribute = baseDirElement->attributes->getNamedItem(_bstr_t("Dir"));
						if(!attribute)
							continue;
						CString baseDir = attribute->Gettext();

						if(pReceivingDialog->BeginAddData(baseDir)) {
							for(long j = 0; j < baseDirElement->childNodes->length; j++) {
								MSXML2::IXMLDOMNodePtr element = baseDirElement->childNodes->Getitem(j);
								if(!element)
									continue;
								CString nodeType = element->GetnodeName();
								CString path;

								if(nodeType == TEXT("Directory")) {
									attribute = element->attributes->getNamedItem(_bstr_t("Path"));
								} else if(nodeType == TEXT("DirectoryContent")) {
									attribute = element->attributes->getNamedItem(_bstr_t("Path"));
								} else if(nodeType == TEXT("NewDirectory")) {
									attribute = element->attributes->getNamedItem(_bstr_t("Name"));
								} else if(nodeType == TEXT("File")) {
									attribute = element->attributes->getNamedItem(_bstr_t("Path"));
								} else
									continue;

								if(attribute)
									path = static_cast<LPCTSTR>(attribute->Gettext());
								pReceivingDialog->AddData(nodeType, path);
							}
							pReceivingDialog->CommitAddedData();
						}
					}
				}
			}
		}
	}

	return (commandType != TEXT("ClosePipe"));
}