#pragma once
#include "afxcmn.h"

#include "common.h"
#include "IGalbaWindow.h"
#include "TaskDialog.h"
#include "MyDropTarget.h"


// CMainDlg dialog

class CMainDlg;

typedef struct THREAD_PARAMS_PIPESERVER
{
	CMainDlg* pThis;
	HWND hNotificationWnd;
	CBurnTaskDialog* pReceivingDialog;
} THREAD_PARAMS_PIPESERVER, *PTHREAD_PARAMS_PIPESERVER;


class CMainDlg :
    public CDialog,
    public IGalbaWindow
{
	DECLARE_DYNAMIC(CMainDlg)

public:
	CMainDlg(CWnd* pParent = NULL);   // standard constructor
	virtual ~CMainDlg();

	// Dialog Data
	enum { IDD = IDD_GALBA_DIALOG };

	BOOL PreTranslateMessage(MSG* pMsg);

	// implementation of IGalbaWindow
	virtual HRESULT DeviceChange();
	virtual HRESULT IsImageRecorderSelected(BOOL& imageRecorderSelected);
	virtual HRESULT GetRecorder(__deref_out CDiscRecorder** ppRecorder);
	virtual HRESULT GetRecorderPath(CString& path);
	virtual HRESULT EnableUI(BOOL enable);
	virtual HRESULT EnableCancelButton(BOOL enable);
	virtual HRESULT SetupProgressMarquee(BOOL enable);
	virtual HRESULT SetProgress(int percentsDone, CString& text);
	virtual HRESULT SetSystemBufferUsage(int percents);
	virtual HRESULT SetTimes(LONG timeElapsed, LONG timeRemaining);

	static UINT __cdecl PipeServerThread(LPVOID pParam);

	DROPEFFECT OnFileDragOver(COleDataObject* pDataObject, DWORD keyState, CPoint point);
	void OnDragLeave(void);
	BOOL OnFileDrop(COleDataObject* pDataObject, DWORD keyState, CPoint point);

protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

	DECLARE_MESSAGE_MAP()

protected:
	CString	GetMediaTypeString(IMAPI_MEDIA_PHYSICAL_TYPE mediaType);
	bool AddRecordersToComboBox();
	void AddDriveFeaturesToListBox();
	void RetrieveMediaType();

	bool ProcessXmlCommand(const CString& commandString, CBurnTaskDialog* pReceivingDialog);

	ITaskbarList3* pTaskBarList;
	CMyDropTarget dropTarget;
	//bool isAlwaysOnTop;
	HICON hIcon;
	CBurnTaskDialog* pCurrentTaskDialog;
	int currentDeviceSelection;
	CWinThread* pPipeServerThread;

	struct
	{
		CListBox taskList;
		CComboBox deviceCombo;
		CButton ejectButton;
		CButton reloadButton;
		CStatic mediaType;
		CListBox featuresList;
		CButton alwaysOnTopCheck;
		CStatic dialogPlaceHolder;
		CProgressCtrl progress;
		CStatic progressText;
		CStatic timeElapsed;
		CStatic timeRemaining;
		CProgressCtrl bufferProgress;
		CButton cancelButton;
		CImageList reloadDevicesImageList;
	} controls;

	// Generated message map functions
	virtual BOOL OnInitDialog();
	virtual void OnCancel();
	virtual void OnOK();
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnDestroy();
	afx_msg BOOL OnDeviceChange(UINT nEventType, DWORD_PTR dwData);
	afx_msg LRESULT OnDeviceChange(WPARAM /*wParam*/, LPARAM /*lParam*/);
	afx_msg LRESULT OnPipeServerInitialized(WPARAM /*wParam*/, LPARAM /*lParam*/);
	afx_msg void OnLbnSelchangeTaskList();
	afx_msg void OnCbnSelchangeDeviceCombo();
	afx_msg void OnBnClickedAlwaysOnTopCheck();
	afx_msg void OnEject();
	afx_msg void OnReloadDevices();
};