#pragma once

#include "common.h"
#include "DiscRecorder.h"
#include "DiscFormatData.h"


class CRecordingController;

typedef struct THREAD_PARAMS_BURN
{
	CRecordingController* pThis;
	CString recorderID;
	CString clientName;
	HWND hNotificationWnd;
	bool enableBurnProof;
	bool closeMedia;
	bool ejectMedia;
	LONG writeSpeed;
	LPSTREAM pData;
	// for image recorder only
	LONG totalBlocks;
	LONG blockSize;
	CString volumeName;
} THREAD_PARAMS_BURN, *PTHREAD_PARAMS_BURN;


class CRecordingController
{
public:
	CRecordingController(void);
	~CRecordingController(void);

	inline HRESULT GetHresult()
	{
		CSingleLock singleLock(&criticalSection);
		return lastError;
	}

	inline CString GetErrorMessage()
	{
		CSingleLock singleLock(&criticalSection);
		return lastErrorMessage;
	}

	inline void SetRecorderID(CString discRecorderID)
	{
		this->recorderID = discRecorderID;
	}

	inline void SetClientName(const CString& clientName)
	{
		this->clientName = clientName;
	}

	inline void SetNotificationWindow(HWND hNotificationWnd)
	{
		this->hNotificationWnd = hNotificationWnd;
	}

	inline void EnableBurnProof(bool enable)
	{
		this->enableBurnProof = enable;
	}

	inline void SetCloseMedia(bool closeMedia)
	{
		this->closeMedia = closeMedia;
	}

	inline void SetEjectMedia(bool ejectMedia)
	{
		this->ejectMedia = ejectMedia;
	}

	inline void SetWriteSpeed(LONG writeSpeed)
	{
		this->writeSpeed = writeSpeed;
	}

	inline void SetTotalBlocks(LONG totalBlocks)
	{
		this->totalBlocks = totalBlocks;
	}

	inline void SetBlockSize(LONG blockSize)
	{
		if(blockSize <= 0)
			blockSize = IMAPI_SECTOR_SIZE;
		this->blockSize = blockSize;
	}

	inline void SetVolumeName(const CString& volumeName)
	{
		this->volumeName = volumeName;
	}

	bool GetCancelFlag();
	void SetCancelFlag(bool cancel);
	void SetLastError(HRESULT lastError);
	void SetLastError(HRESULT lastError, CString lastErrorMessage);

	bool Burn(__in LPSTREAM pData);
	static UINT __cdecl BurnThread(LPVOID pParam);
	static UINT __cdecl BurnThread_ImageRecorder(LPVOID pParam);
	static HRESULT WriteImage(const CString& fileName, __in PTHREAD_PARAMS_BURN pThreadData);

protected:
	CWinThread* pWorkerThread;
	CCriticalSection criticalSection;
	HRESULT lastError;
	CString lastErrorMessage;
	bool cancel;

	CString recorderID;
	CString clientName;
	HWND hNotificationWnd;
	bool enableBurnProof;
	bool closeMedia;
	bool ejectMedia;
	LONG writeSpeed;
	// for image recorder only
	LONG totalBlocks;
	LONG blockSize;
	CString volumeName;
};