#pragma once

#include <map>
#include "comutil.h"
#import <msxml3.dll>


typedef std::map<CString, _variant_t> CONTAINER_SETTINGS;

class CXmlSettings
{
	CONTAINER_SETTINGS settings;

public:
	CXmlSettings(void)
	{
	}

	~CXmlSettings(void)
	{
	}

	_variant_t& GetSetting(CString name)
	{
		name = TEXT("Settings\\") + name;
		return settings[name];
	}

	void SetSetting(CString name, const _variant_t& setting)
	{
		name = TEXT("Settings\\") + name;
		settings[name] = setting;
	}

	void Save(const CString& fileName)
	{
		MSXML2::IXMLDOMDocument2Ptr xmlDoc(__uuidof(MSXML2::DOMDocument30));

		for(CONTAINER_SETTINGS::const_iterator it = settings.begin(); it != settings.end(); ++it) {
			CString settingName = it->first;
			MSXML2::IXMLDOMElementPtr xmlElement = NULL;
			MSXML2::IXMLDOMElementPtr e = NULL;
			int p1 = 0, p2 = 0;
			CString element;
			BOOL first = TRUE;
			do {
				p2 = it->first.Find(TEXT("\\"), p1);
				if(p2 > 0)
					element = settingName.Mid(p1, p2 - p1);
				else
					element = settingName.Mid(p1);
				p1 = p2 + 1;

				if(first) {
					xmlElement = xmlDoc->selectSingleNode(_bstr_t(element));
					if(!xmlElement) {
						xmlElement = xmlDoc->createElement(_bstr_t(element));
						xmlDoc->appendChild(xmlElement);
					}
					first = FALSE;
				} else {
					MSXML2::IXMLDOMElementPtr e = xmlElement->selectSingleNode(_bstr_t(element));
					if(!e) {
						e = xmlDoc->createElement(_bstr_t(element));
						xmlElement->appendChild(e);
					}
					xmlElement = e;
				}
			} while(p2 > 0);

			// store value
			xmlElement->setAttribute(_bstr_t("value"), it->second);
		}

		// make sure directory exists
		TCHAR pDir[1024];
		StringCchCopy(pDir, 1024, fileName);
		PathRemoveFileSpec(pDir);
		CreateDirectory(pDir, NULL);

		MSXML2::IMXWriterPtr xmlWriter(__uuidof(MSXML2::MXXMLWriter30));
		xmlWriter->byteOrderMark = VARIANT_TRUE;
		xmlWriter->omitXMLDeclaration = VARIANT_FALSE;
		xmlWriter->indent = VARIANT_TRUE;
		MSXML2::ISAXXMLReaderPtr saxReader(__uuidof(MSXML2::SAXXMLReader30));
		CComQIPtr<MSXML2::ISAXContentHandler> pCH = xmlWriter;
		saxReader->putContentHandler(pCH);
		CComQIPtr<MSXML2::ISAXDTDHandler> pDH = xmlWriter;
		saxReader->putDTDHandler(pDH);
		CComQIPtr<MSXML2::ISAXErrorHandler> pEH = xmlWriter;
		saxReader->putErrorHandler(pEH);
		LPSTREAM pOutputStream = NULL;
		HRESULT hr = SHCreateStreamOnFileEx(fileName, STGM_CREATE | STGM_WRITE | STGM_SHARE_DENY_WRITE, FILE_ATTRIBUTE_NORMAL, FALSE, NULL, &pOutputStream);
		if(SUCCEEDED(hr)) {
			xmlWriter->output = pOutputStream;
			variant_t v(xmlDoc, true);
			saxReader->parse(v);
			pOutputStream->Commit(STGC_DEFAULT);
			pOutputStream->Release();
		}
	} 

	void Load(const MSXML2::IXMLDOMElementPtr& xmlElement, CString nodeName)
	{
		for(long i = 0; i < xmlElement->childNodes->length; ++i) {
			MSXML2::IXMLDOMElementPtr e = xmlElement->childNodes->item[i];
			CString settingName = nodeName;
			if(!settingName.IsEmpty())
				settingName += TEXT("\\");
			CString s = e->nodeName;
			settingName += s;

			for(long i2 = 0; i2 < e->attributes->length; ++i2) {
				CString attributeName = e->attributes->item[i2]->nodeName;
				if(attributeName == TEXT("value")) {
					settings[settingName] = e->attributes->item[i2]->nodeValue;
					break;
				}
			}

			Load(e, settingName);
		}
	}

	void Load(const CString& fileName)
	{
		MSXML2::IXMLDOMDocument2Ptr xmlDoc(__uuidof(MSXML2::DOMDocument30));
		const _bstr_t bstr = fileName;
		if(xmlDoc->load(bstr) == VARIANT_FALSE)
			return;
		Load(xmlDoc->documentElement, TEXT("Settings"));
	}
};